 /******************************************************************************
  * plik    BMP_enkoder.c 
  * funkcja	Plik procedur zwizanych z utworzeniem pliku .BMP
  * autor 	ARIES Ryszard Szymaniak www.arskam.com
  * wersja 	1.0
  * data    15-09-2012
  * opis   	ARS3-MT9D111 sterownik ARS3 obsugujcy modu kamery MT9D111 firmy APTINA
  * 		STM32F217VGT
  ******************************************************************************
  */  

#include "Definicje_sprzet.h"
#include "BMP_enkoder.h"
#include "EEPROM_procedury.h"
#include "stm32f2xx_conf.h"

struct Parametry
{
	uint8_t *p_bufor_RAM_danych_obrazka;	//wskanik do pocztku buforu danych ramki obrazka
	int ilosc_pikseli_w_linii;				//szeroko w pikselach obrazka ktrego dane ramki s w buforze
	int ilosc_linii;						//wysoko w liniach obrazka ktrego dane ramki s w buforze
	char ilosc_bajtow_na_piksel;			//ilo bajtw na piksel
	uint8_t *p_bufor_RAM_pliku;				//bufor wyjciowy pliku obrazka BMP
	int rozmiar_MAX_buf;					//max. rozmiar buforu wyjciowego	
	int ilosc_danych_w_buf;					//ilo danych w buforze wyjciowym obrazka BMP, 0 bd konwersji
	int numer_przetwarzanej_linii;			//numer przetwarzanej linii ramki
	int numer_przetwarzanego_piksela_linii;	//numer przetwarzanego piksela linii ramki
	int location;							//pozycja przetwarzanego bajtu w buforze danych ramki
} parametry_bmp __attribute__((section("RW")));

#define MARGINES_BUFORU_WYJSCIOWEGO		24

//-----------------------------------------------------------------------------------------
//kodowanie danych ramki kamery z wewntrznego RAM na plik obrazu BMP i zapis do buforu pliku
int Kodowanie_z_bufor_RAM_danych_na_BMP_do_RAM_pliku(
												  uint8_t *p_bufor_RAM_danych_obrazka,//bufor danych ramki obrazka
												  int image_width, 		  			  //szeroko obrazka w pikselach
												  int image_height,		  			  //wysoko obrazka w liniach
												  char bajty_na_pix,				  //ilo bajtw na piksel
												  uint8_t *p_bufor_RAM_pliku, 	  	  //bufor wyjciowy pliku obrazka BMP
												  int rozmiar_buforu	  			  //max. rozmiar buforu wyjciowego												  
												  )	
//we: adres buforu i parametry danych obrazka w buforze, adres buforu pliku obrazka po konwersji
//wy: ilo danych w buforze pliku obrazka po konwersji 												  	 
{
		 
	parametry_bmp.ilosc_pikseli_w_linii =image_width;
	parametry_bmp.ilosc_linii =image_height;
	parametry_bmp.p_bufor_RAM_pliku =p_bufor_RAM_pliku;
	parametry_bmp.rozmiar_MAX_buf =rozmiar_buforu;
	parametry_bmp.ilosc_danych_w_buf =0;
	parametry_bmp.p_bufor_RAM_danych_obrazka =p_bufor_RAM_danych_obrazka;
	parametry_bmp.ilosc_bajtow_na_piksel =bajty_na_pix;
	parametry_bmp.location =0;

	Naglowek_BMP();

	parametry_bmp.numer_przetwarzanej_linii =0;
	parametry_bmp.numer_przetwarzanego_piksela_linii =0;

	Zapis_danych_BMP();

	if (parametry_bmp.ilosc_danych_w_buf >parametry_bmp.rozmiar_MAX_buf -MARGINES_BUFORU_WYJSCIOWEGO)
	{
		//ilo dnych w buforze wyjciowym wskazuje, e obrazek mg nie zosta przekonwertowany w caoci
		parametry_bmp.ilosc_danych_w_buf =0;
	}
	return parametry_bmp.ilosc_danych_w_buf;
}
//----------------------------
//tworzenie nagwka pliku BMP
void Naglowek_BMP(void)
{
int size;

	size =54 + parametry_bmp.ilosc_pikseli_w_linii * parametry_bmp.ilosc_linii;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) ='B';
	parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) ='M';
	parametry_bmp.ilosc_danych_w_buf++;

	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0xFF & size;
	parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0xFF & (size>>8);
	parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0xFF & (size>>16);
	parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0xFF & (size>>24);
    parametry_bmp.ilosc_danych_w_buf++;

	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;

	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =54;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;

	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =40;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;

	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =
	   0xFF & parametry_bmp.ilosc_pikseli_w_linii;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =
	   0xFF & (parametry_bmp.ilosc_pikseli_w_linii>>8);
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;

	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =
	   0xFF & parametry_bmp.ilosc_linii;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =
	   0xFF & (parametry_bmp.ilosc_linii>>8);
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;

	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =1;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =24;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;

	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;

	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;

	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;

	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;

	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;

	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;
	*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
    parametry_bmp.ilosc_danych_w_buf++;

}
//zapis danych obrazu BMP
void Zapis_danych_BMP(void)
{
int i, j, tmp;
uint8_t R,G,B;
char dane_piksela_tab[2], *p_dane_piksela_tab;

	i =parametry_bmp.numer_przetwarzanej_linii;
	j =parametry_bmp.numer_przetwarzanego_piksela_linii;
	p_dane_piksela_tab =&dane_piksela_tab[0];

	for (; i<parametry_bmp.ilosc_linii; i++)
	{
		parametry_bmp.location =((((parametry_bmp.ilosc_linii -i -1) *parametry_bmp.ilosc_pikseli_w_linii)+
		              j) *parametry_bmp.ilosc_bajtow_na_piksel)
					  -parametry_bmp.ilosc_bajtow_na_piksel; //-parametry_bmp.ilosc_bajtow_na_piksel przesunicie o 1 piksel 
		for (; j<parametry_bmp.ilosc_pikseli_w_linii; j++)
		{
			//2 bajty na piksel obrazu
			*p_dane_piksela_tab =*(__IO uint8_t*) (parametry_bmp.p_bufor_RAM_danych_obrazka +parametry_bmp.location);
			parametry_bmp.location++;
			*(p_dane_piksela_tab+1) =*(__IO uint8_t*) (parametry_bmp.p_bufor_RAM_danych_obrazka +parametry_bmp.location);
			parametry_bmp.location++;
			R =0xF8 & dane_piksela_tab[0];
			G =(0x07 & dane_piksela_tab[0]) <<5;
			G =G + ((0xE0 & dane_piksela_tab[1]) >>3);
			B =(0x1F & dane_piksela_tab[1]) <<3;			

			*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =B;//B
			parametry_bmp.ilosc_danych_w_buf++;
			*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =G;//G
			parametry_bmp.ilosc_danych_w_buf++;
			*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =R;//R
			parametry_bmp.ilosc_danych_w_buf++;
			if (parametry_bmp.ilosc_danych_w_buf >parametry_bmp.rozmiar_MAX_buf -MARGINES_BUFORU_WYJSCIOWEGO)
			{	  
				parametry_bmp.numer_przetwarzanej_linii =i;
				parametry_bmp.numer_przetwarzanego_piksela_linii =j+1;
				return;		
			}
		}
		tmp = 4 - ((parametry_bmp.ilosc_pikseli_w_linii * 3) & 3);

		if (tmp != 4)
		{
			for (j=0; j<tmp; j++)
			{
				*(parametry_bmp.p_bufor_RAM_pliku +parametry_bmp.ilosc_danych_w_buf) =0;
				parametry_bmp.ilosc_danych_w_buf++;
			}
		}
		j =0;
	}
	parametry_bmp.numer_przetwarzanej_linii =i;
	parametry_bmp.numer_przetwarzanego_piksela_linii =j;
	//zawarto wszystkich linii zostaa przekonwertowana do BMP
}
	
