/**
  ******************************************************************************
  * plik    EEPROM_procedury.c
  * funkcja	Plik procedur zwizanych z programow obsug pamici EEPROM 
  *         z 0/1/2 bajtowym adresowaniem
  * autor 	ARIES Ryszard Szymaniak www.arskam.com
  * wersja 	1.0
  * data    10-08-2012
  * opis   	ARS3-MT9D111 sterownik ARS3 obsugujcy modu kamery MT9D111 firmy APTINA
  * 		STM32F217VGT
  ******************************************************************************
  */  
#include "EEPROM_procedury.h"
#include "Definicje_sprzet.h"
#include "Procedury Ogolne.h"
#include "GPIO_procedury.h"

struct {
         unsigned char adres;
         unsigned char subadres;
         unsigned char subadres_second;
         unsigned char ile;
         char *p_bufor;
       }i2c;
unsigned char i2c_r_w;

char bufor_posredni[ROZMIAR_BUF_POSREDNI];

//procedury zewntrzne
//	main.c
extern void Delay_ms(int czas);
extern void Delay_us(int czas);

//-----------------
//odczyt z EEPROM-u
char EepOdczyt(unsigned int adres_int, unsigned int ilosc_bajtow, char *p_bufor)
{
char status=TRUE;
unsigned char ile_do_odczytu;

	while (ilosc_bajtow !=0)
	{
		if (ilosc_bajtow <=ROZMIAR_STRONY_EEPROM) ile_do_odczytu =(0xFF &ilosc_bajtow);
		else ile_do_odczytu =ROZMIAR_STRONY_EEPROM;
		status =EepOdczytBloku(adres_int, ile_do_odczytu, p_bufor);
		adres_int =(adres_int +ile_do_odczytu);
		p_bufor =(p_bufor +ile_do_odczytu);
		ilosc_bajtow =(ilosc_bajtow -ile_do_odczytu);	
	}  
	return status;
}
//-----------------------------------------
//procedura odczytu bloku danych z EEPROM-u
char EepOdczytBloku(unsigned int adres_int, unsigned char ilosc_bajtow, char *p_bufor)
{
//wy: 0 -bd, 1 -sukces procedury
char status;

  i2c.adres =0xa0;
  i2c.subadres =(unsigned char)(adres_int >>8);
  i2c.subadres_second =(unsigned char)adres_int;  
  i2c.p_bufor =p_bufor;
  i2c.ile =ilosc_bajtow;
  i2c_r_w =1; 
  status =I2c_(ADRES_I2C_2BAJTY);
  return status;
}  
//-----------------
//zapis do EEPROM-u
char EepZapis(unsigned int adres_int, unsigned int ilosc_bajtow, char *p_bufor)
{
char status=TRUE;
unsigned char ile_do_zapisu, ile_do_zapisu_na_stronie;

	while (ilosc_bajtow !=0)
	{
	    ile_do_zapisu_na_stronie =(ROZMIAR_STRONY_EEPROM -(adres_int &(ROZMIAR_STRONY_EEPROM-1)));
		if (ilosc_bajtow <=ile_do_zapisu_na_stronie) ile_do_zapisu =(0xFF &ilosc_bajtow);
		else ile_do_zapisu =ile_do_zapisu_na_stronie;
		status =EepZapisBloku(adres_int, ile_do_zapisu, p_bufor);
		if (status !=TRUE) break;
		adres_int =(adres_int +ile_do_zapisu);
		p_bufor =(p_bufor +ile_do_zapisu);
		ilosc_bajtow =(ilosc_bajtow -ile_do_zapisu);	
	}  
	return status;
}
//-----------------------------------------
//procedura zapisu bloku danych do EEPROM-u
char EepZapisBloku(unsigned int adres_int, unsigned char ilosc_bajtow, char *p_bufor)
{
//wy: 0 -bd, 1 -sukces procedury
unsigned char status;

  i2c.adres =0xa0;
  i2c.subadres =(unsigned char)(adres_int >>8);
  i2c.subadres_second =(unsigned char)adres_int;  
  i2c.p_bufor =p_bufor;
  i2c.ile =ilosc_bajtow;
  i2c_r_w =0; 
  status =I2c_(ADRES_I2C_2BAJTY);
  Delay_ms(15);
  return status;
}
//--------------------------
//g-wna procedura I2C
unsigned char I2c_(unsigned char ile_bajtow_adresu)
//wy: 0 -bd, 1 -sukces procedury
{
unsigned char x;
unsigned char test,ack;

  while (1 !=2)
  {
    test =I2c_cnl();
    if (test ==1) break;//bd
	Delay_us(5);
    I2c_start();
    test =I2c_byte_wr(i2c.adres);
    if (test ==1) break;//bd
    if (ile_bajtow_adresu !=ADRES_I2C_0BAJT)
    {
      test =I2c_byte_wr(i2c.subadres);
    }
    if (test ==1) break;//bd
    if (ile_bajtow_adresu ==ADRES_I2C_2BAJTY)//wymiana danych z EEPROMEM 
    {
      test =I2c_byte_wr(i2c.subadres_second);
      if (test ==1) break;//bd
    }
    
    if (i2c_r_w ==1)     //odbir 'ile' bajtw
    {
      I2c_stop();
      I2c_start();
      test =I2c_byte_wr(i2c.adres +1);//adres z bitem r/w=1
      if (test ==1) break;//bd
    }
    for (x=0;x<i2c.ile;x++)
    {
      if (i2c_r_w ==1) //odbir
      {
        if (x !=(i2c.ile-1)) ack =1;
        else ack =0;//dla ostatniego bajtu odbir bez wystawiania ACK
        *(i2c.p_bufor +x) =I2c_byte_rd(ack);
      }
      else             //transmisja
      {
        test =I2c_byte_wr(*(i2c.p_bufor +x));
        if (test ==1) break;//bd
      }
    }
    I2c_stop();
    break;
  }
  if (test ==1) return 0;     //bd EEPROM-u
  else return 1;              //sukces
}
//------------------------------------        
//procedura I2C transmisji bajtu
unsigned char I2c_byte_wr(unsigned char bajt)
//we: bajt -bajt do wysania
//wy flaga stanu 0 -sukces, 1 -bd transmisji
{
unsigned char bit_count,maska;
unsigned char flaga;

  maska =0x80;
  for (bit_count=8;bit_count>0;bit_count--)//transmisja 8 bitw MSB first
  {
    if ((maska & bajt) ==0) GPIO_Linie_IO_Low (LINIA_EEP_SDA);//SDA =0
    else 					GPIO_Linie_IO_High(LINIA_EEP_SDA);//SDA =1
    Delay_us(1);
    GPIO_Linie_IO_High(LINIA_EEP_SCL);//SCL =1;
    Delay_us(5);
    GPIO_Linie_IO_Low (LINIA_EEP_SCL);//SCL =0;
    Delay_us(5);
    maska =maska/2;
  }
  GPIO_Linie_IO_High(LINIA_EEP_SDA);//SDA =in
  Delay_us(1);
  GPIO_Linie_IO_High(LINIA_EEP_SCL);//SCL =1;
  Delay_us(10);
  if (GPIO_Linie_IO_Odczyt(LINIA_EEP_SDA) ==POZIOM_L) flaga =0;//detekcja ACK 
  else flaga =1;
  Delay_us(1);
  GPIO_Linie_IO_Low (LINIA_EEP_SCL);//SCL =0;
  					  //SDA =out
  Delay_us(1);
  GPIO_Linie_IO_Low (LINIA_EEP_SDA);//SDA =0
  Delay_us(5);
  return flaga;
}
//---------------------------------
//procedura I2C odbioru bajtu
unsigned char I2c_byte_rd(unsigned char ack_on)
//we: ack_on 1 -wylij ACK po odbiorze bajtu, 0 -nie wysyaj ACK
//wy: odebrany bajt 
{
unsigned char bit_count,maska,bajt;

  maska =0x80;
  bajt =0;
  GPIO_Linie_IO_High(LINIA_EEP_SDA);//SDA =in
  Delay_us(1);
  for (bit_count=8;bit_count>0;bit_count--)//odczyt 8 bitw MSB first
  {
    GPIO_Linie_IO_High(LINIA_EEP_SCL);//SCL =1;
    Delay_us(5);
    if (GPIO_Linie_IO_Odczyt(LINIA_EEP_SDA) !=POZIOM_L) bajt =(bajt | maska);
    GPIO_Linie_IO_Low (LINIA_EEP_SCL);//SCL =0;
    Delay_us(5);
    maska =maska/2;
  }
  								 //SDA =out
  if (ack_on ==1)
  {
    GPIO_Linie_IO_Low (LINIA_EEP_SDA);//SDA =0
    Delay_us(5);
    GPIO_Linie_IO_High(LINIA_EEP_SCL);//SCL =1;
    Delay_us(5);
    GPIO_Linie_IO_Low (LINIA_EEP_SCL);//SCL =0;
    Delay_us(1);
  }
  GPIO_Linie_IO_Low (LINIA_EEP_SDA);//SDA =0
  Delay_us(5);
  return bajt;
}
//---------------------------------------------------------
//sprawdzenie czy linie SDA i SCL s w stanie wysokim
unsigned char I2c_cnl(void)
//wy: flaga stanu 0 -sukces, 1 -bd
{
  I2c_start();
  I2c_stop();
  if ((GPIO_Linie_IO_Odczyt(LINIA_EEP_SDA) ==POZIOM_H) && 
                              (GPIO_Linie_IO_Odczyt(LINIA_EEP_SCL) ==POZIOM_H)) return 0;
  else return 1;
}
//-------------------------
//procedura I2C START
void I2c_start(void)
{
  GPIO_Linie_IO_Low (LINIA_EEP_SDA);//SDA =0
  Delay_us(50);
  GPIO_Linie_IO_Low (LINIA_EEP_SCL);//SCL =0;
  Delay_us(10);
  return;
}
//------------------------
//procedura I2C STOP
void I2c_stop(void)
{
    GPIO_Linie_IO_High(LINIA_EEP_SCL);//SCL =1;
    Delay_us(5);
  	GPIO_Linie_IO_High(LINIA_EEP_SDA);//SDA =1
	Delay_us(5);
  	return;
}
