/**
  ******************************************************************************
  * plik    Kamera.c
  * funkcja	procedury poredniczce przy obsudze rnych moduw kamer
  * autor 	ARIES Ryszard Szymaniak www.arskam.com
  * wersja 	1.0
  * data    16-08-2012
  * opis   	ARS3-MT9D111 sterownik ARS3 obsugujcy modu kamery MT9D111 firmy APTINA
  * 		STM32F217VGT
  ******************************************************************************
  */  
#include "Kamera.h"
#include "stm32f2xx_conf.h"
#include "TIMER_procedury.h"
#include "Kamera_DMA.h"
#include "MT9D111.h"
#include "Procedury Ogolne.h"
#include "Definicje_sprzet.h"

const uint32_t GPIO_KAM_DAT_CLK[KAM_DATn]	  ={KAM_DAT0_CLK, KAM_DAT1_CLK, KAM_DAT2_CLK,
												KAM_DAT3_CLK, KAM_DAT4_CLK, KAM_DAT5_CLK,
												KAM_DAT6_CLK, KAM_DAT7_CLK};
const uint16_t GPIO_KAM_DAT_PIN[KAM_DATn]	  ={KAM_DAT0_PIN, KAM_DAT1_PIN, KAM_DAT2_PIN,
												KAM_DAT3_PIN, KAM_DAT4_PIN, KAM_DAT5_PIN,
												KAM_DAT6_PIN, KAM_DAT7_PIN};
GPIO_TypeDef* GPIO_KAM_DAT_PORT[KAM_DATn]	  ={KAM_DAT0_PORT, KAM_DAT1_PORT, KAM_DAT2_PORT,
												KAM_DAT3_PORT, KAM_DAT4_PORT, KAM_DAT5_PORT,
												KAM_DAT6_PORT, KAM_DAT7_PORT};

const uint32_t GPIO_KAM_LINE_CLK[KAM_IOn]	  ={KAM_MCLK_CLK, KAM_HSYNC_CLK, KAM_STDB_CLK,
												KAM_VSYNC_CLK, KAM_PCLK_CLK, KAM_RSTB_CLK};
const uint16_t GPIO_KAM_LINE_PIN[KAM_IOn]	  ={KAM_MCLK_PIN, KAM_HSYNC_PIN, KAM_STDB_PIN,
												KAM_VSYNC_PIN, KAM_PCLK_PIN, KAM_RSTB_PIN};
GPIO_TypeDef* GPIO_KAM_LINE_PORT[KAM_IOn]	  ={KAM_MCLK_PORT, KAM_HSYNC_PORT, KAM_STDB_PORT,
												KAM_VSYNC_PORT, KAM_PCLK_PORT, KAM_RSTB_PORT};

/*__IO */uint8_t bufor_RAM_danych_obrazka[ROZMIAR_BUF_OBRAZU +100];
/*__IO */uint8_t bufor_RAM_pliku_obrazka_format[ROZMIAR_BUF_PLIKU_FORMAT_OBRAZ];

//--------------------------
//procedura inicjacji kamery
void Kamera_Inicjacja(int image_width, int image_height, char bajty_na_piksel)
{

	//inicjacja linii gwnego zegara KAM_MCLK taktujcego kamer
	KAM_IO_Linie_Inicjacja  (KAM_MCLK, GPIO_Mode_AF, GPIO_OType_PP, GPIO_PuPd_UP); //wy. sygna gwnego zegara MCLK
	RCC_MCO1Config  ( RCC_MCO1Source_PLLCLK, RCC_MCO1Div_3/*RCC_MCO1Div_5*/);

	Delay_ms(100);

	//inicjacja rwnolegej magistrali danych obrazu
	RCC_AHB1PeriphClockCmd(GPIO_KAM_DAT_CLK[KAM_DAT0], ENABLE);
	GPIO_PinAFConfig (KAM_DAT0_PORT, KAM_DAT0_PIN_SOURCE, GPIO_AF_DCMI);
	KAM_Magistrala_danych_inicjacja(KAM_DAT0, GPIO_Mode_AF, GPIO_OType_PP, GPIO_PuPd_UP);

	RCC_AHB1PeriphClockCmd(GPIO_KAM_DAT_CLK[KAM_DAT1], ENABLE);
	GPIO_PinAFConfig (KAM_DAT1_PORT, KAM_DAT1_PIN_SOURCE, GPIO_AF_DCMI);
	KAM_Magistrala_danych_inicjacja(KAM_DAT1, GPIO_Mode_AF, GPIO_OType_PP, GPIO_PuPd_UP);

	RCC_AHB1PeriphClockCmd(GPIO_KAM_DAT_CLK[KAM_DAT2], ENABLE);
	GPIO_PinAFConfig (KAM_DAT2_PORT, KAM_DAT2_PIN_SOURCE, GPIO_AF_DCMI);
	KAM_Magistrala_danych_inicjacja(KAM_DAT2, GPIO_Mode_AF, GPIO_OType_PP, GPIO_PuPd_UP);

	RCC_AHB1PeriphClockCmd(GPIO_KAM_DAT_CLK[KAM_DAT3], ENABLE);
	GPIO_PinAFConfig (KAM_DAT3_PORT, KAM_DAT3_PIN_SOURCE, GPIO_AF_DCMI);
	KAM_Magistrala_danych_inicjacja(KAM_DAT3, GPIO_Mode_AF, GPIO_OType_PP, GPIO_PuPd_UP);

	RCC_AHB1PeriphClockCmd(GPIO_KAM_DAT_CLK[KAM_DAT4], ENABLE);
	GPIO_PinAFConfig (KAM_DAT4_PORT, KAM_DAT4_PIN_SOURCE, GPIO_AF_DCMI);
	KAM_Magistrala_danych_inicjacja(KAM_DAT4, GPIO_Mode_AF, GPIO_OType_PP, GPIO_PuPd_UP);

	RCC_AHB1PeriphClockCmd(GPIO_KAM_DAT_CLK[KAM_DAT5], ENABLE);
	GPIO_PinAFConfig (KAM_DAT5_PORT, KAM_DAT5_PIN_SOURCE, GPIO_AF_DCMI);
	KAM_Magistrala_danych_inicjacja(KAM_DAT5, GPIO_Mode_AF, GPIO_OType_PP, GPIO_PuPd_UP);

	RCC_AHB1PeriphClockCmd(GPIO_KAM_DAT_CLK[KAM_DAT6], ENABLE);
	GPIO_PinAFConfig (KAM_DAT6_PORT, KAM_DAT6_PIN_SOURCE, GPIO_AF_DCMI);
	KAM_Magistrala_danych_inicjacja(KAM_DAT6, GPIO_Mode_AF, GPIO_OType_PP, GPIO_PuPd_UP);

	RCC_AHB1PeriphClockCmd(GPIO_KAM_DAT_CLK[KAM_DAT7], ENABLE);
	GPIO_PinAFConfig (KAM_DAT7_PORT, KAM_DAT7_PIN_SOURCE, GPIO_AF_DCMI);
	KAM_Magistrala_danych_inicjacja(KAM_DAT7, GPIO_Mode_AF, GPIO_OType_PP, GPIO_PuPd_UP);

	//inicjacja linii sterujcych
	RCC_AHB1PeriphClockCmd(GPIO_KAM_LINE_CLK[KAM_HSYNC], ENABLE);
	GPIO_PinAFConfig (KAM_HSYNC_PORT, KAM_HSYNC_PIN_SOURCE, GPIO_AF_DCMI);
	KAM_IO_Linie_Inicjacja	(KAM_HSYNC, GPIO_Mode_AF, GPIO_OType_PP, GPIO_PuPd_UP);//we. sygna aktywnej linii obrazu

	//KAM_IO_Linie_Inicjacja	(KAM_STDB, GPIO_Mode_OUT, GPIO_OType_PP, GPIO_PuPd_UP);//wy. sygna STANDBY

	RCC_AHB1PeriphClockCmd(GPIO_KAM_LINE_CLK[KAM_VSYNC], ENABLE);
	GPIO_PinAFConfig (KAM_VSYNC_PORT, KAM_VSYNC_PIN_SOURCE, GPIO_AF_DCMI);
	KAM_IO_Linie_Inicjacja	(KAM_VSYNC, GPIO_Mode_AF, GPIO_OType_PP, GPIO_PuPd_UP);//we. sygna aktywnej ramki obrazu

	RCC_AHB1PeriphClockCmd(GPIO_KAM_LINE_CLK[KAM_PCLK], ENABLE);
	GPIO_PinAFConfig (KAM_PCLK_PORT, KAM_PCLK_PIN_SOURCE, GPIO_AF_DCMI);
    KAM_IO_Linie_Inicjacja	(KAM_PCLK, GPIO_Mode_AF, GPIO_OType_PP, GPIO_PuPd_UP); //we. sygna taktujcy dane pikseli

	//inicjacja DMA
	Inicjacja_DMA_MT9D111_BMP_RAM((uint8_t *)&bufor_RAM_danych_obrazka[0], image_width, image_height, bajty_na_piksel);

}

//--------------------------------------------------
//procedura inicjacji linii magistrali danych kamery
void KAM_Magistrala_danych_inicjacja(Kam_DAT_TypeDef Linia, GPIOMode_TypeDef tryb_pracy, GPIOOType_TypeDef typ_wyjscia, 
									 GPIOPuPd_TypeDef typ_podciagania)
{
   GPIO_InitTypeDef  GPIO_InitStructure;
   
   /* Enable the GPIO_LCD_STER Clock */
   RCC_AHB1PeriphClockCmd(GPIO_KAM_DAT_CLK[Linia], ENABLE);
 
   /* Configure the GPIO_KAM_DAT pin */
   GPIO_InitStructure.GPIO_Pin = GPIO_KAM_DAT_PIN[Linia];
   GPIO_InitStructure.GPIO_Mode =tryb_pracy;
   GPIO_InitStructure.GPIO_OType = typ_wyjscia;
   GPIO_InitStructure.GPIO_PuPd = typ_podciagania;
   GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
 
   GPIO_Init(GPIO_KAM_DAT_PORT[Linia], &GPIO_InitStructure);	
}
//-------------------------------------------------------------
//procedura inicjacji linii sterujcych KAM
void KAM_IO_Linie_Inicjacja(Kam_IO_TypeDef Linia, GPIOMode_TypeDef tryb_pracy, GPIOOType_TypeDef typ_wyjscia, 
							GPIOPuPd_TypeDef typ_podciagania)
{
	GPIO_InitTypeDef  GPIO_InitStructure;
   
   	/* Enable the GPIO_KAM_LINIE Clock */
   	RCC_AHB1PeriphClockCmd(GPIO_KAM_LINE_CLK[Linia], ENABLE);

   	/* Configure the GPIO_KAM_LINIE pin*/
	GPIO_InitStructure.GPIO_Pin = GPIO_KAM_LINE_PIN[Linia];
   	GPIO_InitStructure.GPIO_Mode = tryb_pracy;
   	GPIO_InitStructure.GPIO_OType = typ_wyjscia;
   	GPIO_InitStructure.GPIO_PuPd = typ_podciagania;
   	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
 
   	GPIO_Init(GPIO_KAM_LINE_PORT[Linia], &GPIO_InitStructure);	
}
//---------------------------------------
//ustawienie stanu wysokiego na linii IO sterujcej KAM
void KAM_IO_Linie_High(Kam_IO_TypeDef Linia)
{
	GPIO_KAM_LINE_PORT[Linia]->BSRRL = GPIO_KAM_LINE_PIN[Linia];
}
//---------------------------------------
//ustawienie stanu niskiego na linii IO sterujcej KAM
void KAM_IO_Linie_Low(Kam_IO_TypeDef Linia)
{
	GPIO_KAM_LINE_PORT[Linia]->BSRRH = GPIO_KAM_LINE_PIN[Linia];
}
//----------------------------------------
//procedura odczytu stanu linii IO sterujcej KAM
uint32_t KAM_IO_Linie_Odczyt(Kam_IO_TypeDef Linia)
{
  	return GPIO_ReadInputDataBit(GPIO_KAM_LINE_PORT[Linia], GPIO_KAM_LINE_PIN[Linia]);
}
//---------------------------------------
//procedura zmiany standardowego formatu obrazu kamery
char Kamera_Zmiana_Formatu_Obrazu(char num_formatu_obrazu)
{
char status;

					  
int version=0, fw_version=0;
char result[2];

    readRegister(CHIP_VERSION, result);
	version = (result[0] << 8) + (result[1] & 0xFF);  
	if (version!=CHIP_ID)
	{
		status =FALSE;
		return status;
	}
	readMCUMemory(MON_VER, result); 
	fw_version = (result[0] << 8) + (result[1] & 0xFF); 
	if (fw_version ==0) 
	{
		return FALSE;
	}
	mt9d111_init();
	Delay_ms(1000);//!!!

	return status;
}
//-------------------------------------
//procedura zmiany parametru JASKRAWO	kamery
unsigned char Kamera_Jaskrawosc_regulacja(unsigned char param_jaskrawosc)
{
unsigned char status;

   status =MT9D111_Jaskrawosc_regulacja(param_jaskrawosc);
   return status;
}
//-------------------------------------
//procedura zmiany parametru KONTRAST kamery
unsigned char Kamera_Kontrast_regulacja(unsigned char param_kontrast)
{
unsigned char status;

   status =MT9D111_Kontrast_regulacja(param_kontrast);
   return status;
}
//-------------------------------------
//procedura zmiany parametru NASYCENIE kamery
unsigned char Kamera_Nasycenie_regulacja(unsigned char param_nasycenie)
{
unsigned char status;

   status =MT9D111_Nasycenie_regulacja(param_nasycenie);
   return status;
}

