/**
  ******************************************************************************
  * plik    Kamera_DMA.c
  * funkcja	Plik procedur zwizanych z transferem DMA danych kamery
  * autor 	ARIES Ryszard Szymaniak www.arskam.com
  * wersja 	1.0
  * data    19-08-2012
  * opis   	ARS3-MT9D111 sterownik ARS3 obsugujcy modu kamery MT9D111 firmy APTINA
  * 		STM32F217VGT
  ******************************************************************************
  */ 
#include "Kamera_DMA.h"
#include "stm32f2xx_conf.h"   
#include "Definicje_sprzet.h"
#include "Kamera.h"
#include "GPIO_procedury.h"
#include "Procedury Ogolne.h"
#include "TIMER_procedury.h"

/* Private define ------------------------------------------------------------*/
#define DCMI_DR_ADDRESS       0x50050028


DMA_InitTypeDef DMA_InitStructure;
DCMI_InitTypeDef DCMI_InitStructure;

uint8_t	  dcmi_flag_frameri_bak, dcmi_flag_lineri_bak, dcmi_flag_errri_bak, dcmi_flag_ovfri_bak, dcmi_flag_vsyncri_bak;

extern int obraz_parametry_ozo_tab[][3];

//---------------------------------------------
//inicjacja DMA	dla danych kamery MT9D111 -> RAM
void Inicjacja_DMA_MT9D111_BMP_RAM(uint8_t *bufor_RAM_danych_obrazka, int image_width, int image_height, char bajty_na_piksel)
{
//wersja inicjacji DMA dla wewntrznej pamici RAM 
//dane z wyjcia kamery MT9D111 zapisywane s kanaem DMA do buforu w wntrznej pamici RAM
//rozmiar buforu pojedynczego przesania =ilo bajtw danych linii obrazu * ilo linii <=0xFFFF

uint16_t  ile_bajtow_jednej_linii, calkowita_ilosc_linii_obrazka;
int rozmiar_danych_raw;
uint16_t  rozmiar_przeslania_DMA;


  	ile_bajtow_jednej_linii =image_width * bajty_na_piksel;
	calkowita_ilosc_linii_obrazka =image_height;

  	rozmiar_danych_raw =ile_bajtow_jednej_linii * calkowita_ilosc_linii_obrazka;
  	if (rozmiar_danych_raw >=0x10000)
  	{
  		//rozmiar kolejnych przesa DMA
  		//ile_linii_obrazu_w_przeslaniu_DMA =0x10000/ ile_bajtow_jednej_linii;
  		//rozmiar_przeslania_DMA =ile_linii_obrazu_w_przeslaniu_DMA * ile_bajtow_jednej_linii;
		//UWAGA!!! w tej chwili nie mona prawidowo skonfigurowa 
		//wielokrotnych przesa danych
		rozmiar_przeslania_DMA =ile_bajtow_jednej_linii;
		//poniewa kade przesanie to 4 bajtowe sowo ilo przesa /4
		rozmiar_przeslania_DMA =rozmiar_przeslania_DMA /4;
  	}
  	else
  	{
  		//cae dane przesane jednym przesaniem DMA
  		rozmiar_przeslania_DMA =rozmiar_danych_raw;
		//poniewa kade przesanie to 4 bajtowe sowo ilo przesa /4
		//rozmiar_przeslania_DMA =rozmiar_przeslania_DMA /4;
  	}

  	/*** Configures the DCMI to interface with the MT9D111 camera module ***/
  	/* Enable DCMI clock */
  	RCC_AHB2PeriphClockCmd(RCC_AHB2Periph_DCMI, ENABLE);

  	/* DCMI configuration */ 
  	DCMI_DeInit();

	DCMI_InitStructure.DCMI_CaptureMode = DCMI_CaptureMode_SnapShot;//DCMI_CaptureMode_Continuous;
	DCMI_InitStructure.DCMI_SynchroMode = DCMI_SynchroMode_Hardware;
	DCMI_InitStructure.DCMI_PCKPolarity =/*DCMI_PCKPolarity_Falling;*/DCMI_PCKPolarity_Rising;
	DCMI_InitStructure.DCMI_VSPolarity = DCMI_VSPolarity_Low;//DCMI_VSPolarity_High;
	DCMI_InitStructure.DCMI_HSPolarity = DCMI_HSPolarity_Low;//DCMI_HSPolarity_High;//?!Digh
	DCMI_InitStructure.DCMI_CaptureRate =DCMI_CaptureRate_All_Frame;
	DCMI_InitStructure.DCMI_ExtendedDataMode = DCMI_ExtendedDataMode_8b;

  	//mask interrupt for DCMI
  	DCMI_ITConfig(DCMI_IT_VSYNC, ENABLE);
  	DCMI_ITConfig(DCMI_IT_LINE, ENABLE);
 	DCMI_ITConfig(DCMI_IT_FRAME, ENABLE);
  	DCMI_ITConfig(DCMI_IT_OVF, ENABLE);
  	DCMI_ITConfig(DCMI_IT_ERR, ENABLE);

  	/* Configures the DMA2 to transfer Data from DCMI */
  	/* Enable DMA2 clock */
  	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA2, ENABLE);

  	/* DMA2 Stream1 Configuration */
  	DMA_DeInit(DMA2_Stream1);

  	DMA_InitStructure.DMA_Channel = DMA_Channel_1;  
  	DMA_InitStructure.DMA_PeripheralBaseAddr = DCMI_DR_ADDRESS;	
  	DMA_InitStructure.DMA_Memory0BaseAddr = (uint32_t)bufor_RAM_danych_obrazka;
 	DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralToMemory;
 	DMA_InitStructure.DMA_BufferSize = rozmiar_przeslania_DMA;
  	DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  	DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
  	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Word;
  	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
  	DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
  	DMA_InitStructure.DMA_Priority = DMA_Priority_High;
  	DMA_InitStructure.DMA_FIFOMode = DMA_FIFOMode_Disable;
  	DMA_InitStructure.DMA_FIFOThreshold = DMA_FIFOThreshold_Full;
  	DMA_InitStructure.DMA_MemoryBurst = DMA_MemoryBurst_Single;
  	DMA_InitStructure.DMA_PeripheralBurst = DMA_PeripheralBurst_Single;

  	/* DCMI configuration */
  	DCMI_Init(&DCMI_InitStructure);
	DCMI_JPEGCmd  ( DISABLE );

  	/* DMA2 IRQ channel Configuration */
  	DMA_Init(DMA2_Stream1, &DMA_InitStructure);

  	DMA_ClearITPendingBit ( DMA2_Stream1, 	 DMA_IT_TCIF1  |
                                             DMA_IT_HTIF1  |
                                             DMA_IT_TEIF1  |
                                             DMA_IT_DMEIF1 |
                                             DMA_IT_FEIF1  );
 

  	/* Enable DMA2 stream 1 and DCMI interface then start image capture */
  	DMA_Cmd(DMA2_Stream1, ENABLE); 
  	while ( DISABLE == DMA_GetCmdStatus ( DMA2_Stream1 ) );
  	DCMI_Cmd(ENABLE); 
  	/* Insert 100ms delay: wait 100ms */
  	Delay_ms(200); 
}
//-------------------------------------------------------------------
//procedura przesania danych obrazu z kamery MT9D111 i zapis do wewntrznego buforu w pamici RAM
//dla QQVGA pojedyncze przesanie danych obrazu
void Start_DMA_MT9D111_BMP_do_Int_RAM(int image_width, int image_height, char bajty_na_piksel)
{
uint32_t rozmiar_danych_raw_formatu_obrazu;
uint16_t licznik_DMA, piksele_linii, licznik_linii;
char flaga;


	GPIO_Linie_IO_Low(LINIA_LED_ZIELONY_WY);
	GPIO_Linie_IO_Low(LINIA_LED_CZERWONY_WY);

	rozmiar_danych_raw_formatu_obrazu =(image_width *bajty_na_piksel) *image_height;
	if (rozmiar_danych_raw_formatu_obrazu <=0x10000) image_height =1;//pojedyncze przesanie DMA wszystkich linii

	licznik_DMA =DMA_GetCurrDataCounter  (DMA2_Stream1); 
	DCMI_ClearITPendingBit(DCMI_IT_FRAME | DCMI_IT_OVF | DCMI_IT_ERR | DCMI_IT_VSYNC | DCMI_IT_LINE);
	DCMI_ClearFlag (DCMI_FLAG_FRAMERI | DCMI_FLAG_OVFRI | DCMI_FLAG_ERRRI | DCMI_FLAG_VSYNCRI | DCMI_FLAG_LINERI);

	dcmi_flag_frameri_bak =FALSE;
	dcmi_flag_lineri_bak =FALSE;
	dcmi_flag_errri_bak =FALSE;
	dcmi_flag_ovfri_bak =FALSE;

	flaga =FALSE;
	licznik_linii =0;

	DCMI_CaptureCmd(ENABLE);

    while ( dcmi_flag_frameri_bak ==FALSE )
	{
		if (dcmi_flag_errri_bak ==TRUE || dcmi_flag_ovfri_bak ==TRUE) 
		{
			break;
		}
		if (dcmi_flag_lineri_bak ==TRUE)
		{
			licznik_linii++;
			if (flaga ==FALSE)
			{
				piksele_linii =DMA_GetCurrDataCounter  (DMA2_Stream1); 
				if (piksele_linii ==licznik_DMA) continue;
				flaga =TRUE;
			}
		}
	}
   
    DCMI_CaptureCmd ( DISABLE );   
    DCMI_Cmd ( DISABLE );
	licznik_DMA =DMA_GetCurrDataCounter  (DMA2_Stream1); 



	GPIO_Linie_IO_High(LINIA_LED_ZIELONY_WY);
	GPIO_Linie_IO_High(LINIA_LED_CZERWONY_WY);

}
//--------------------------------------------------
//inicjacja DMA	dla danych JPG kamery MT9D111 -> RAM
void Inicjacja_DMA_MT9D111_JPG_RAM(uint8_t *bufor_RAM_pliku_obrazka_format, int rozmiar_pojedynczego_przeslania_DMA)
{
//wersja inicjacji DMA dla wewntrznej pamici RAM 
//dane JPG z wyjcia kamery MT9D111 zapisywane s kanaem DMA do buforu w wntrznej pamici RAM

  	/*** Configures the DCMI to interface with the MT9D111 camera module ***/
  	/* Enable DCMI clock */
  	RCC_AHB2PeriphClockCmd(RCC_AHB2Periph_DCMI, ENABLE);

  	/* DCMI configuration */ 
  	DCMI_DeInit();

	DCMI_InitStructure.DCMI_CaptureMode = DCMI_CaptureMode_SnapShot;//DCMI_CaptureMode_Continuous;
	DCMI_InitStructure.DCMI_SynchroMode = DCMI_SynchroMode_Hardware;
	DCMI_InitStructure.DCMI_PCKPolarity =/*DCMI_PCKPolarity_Falling;*/DCMI_PCKPolarity_Rising;
	DCMI_InitStructure.DCMI_VSPolarity = DCMI_VSPolarity_Low;//DCMI_VSPolarity_High;
	DCMI_InitStructure.DCMI_HSPolarity = DCMI_HSPolarity_Low;//DCMI_HSPolarity_High;//?!Digh
	DCMI_InitStructure.DCMI_CaptureRate =DCMI_CaptureRate_All_Frame;
	DCMI_InitStructure.DCMI_ExtendedDataMode = DCMI_ExtendedDataMode_8b;

  	//mask interrupt for DCMI
  	DCMI_ITConfig(DCMI_IT_VSYNC, ENABLE);
  	DCMI_ITConfig(DCMI_IT_LINE, ENABLE);
 	DCMI_ITConfig(DCMI_IT_FRAME, ENABLE);
  	DCMI_ITConfig(DCMI_IT_OVF, ENABLE);
  	DCMI_ITConfig(DCMI_IT_ERR, ENABLE);

  	/* Configures the DMA2 to transfer Data from DCMI */
  	/* Enable DMA2 clock */
  	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA2, ENABLE);

  	/* DMA2 Stream1 Configuration */
  	DMA_DeInit(DMA2_Stream1);

//UWAGA!!!
//DMA_BufferSize zostaje okrelony jako DMA_PeripheralDataSize_Word a rozmiar_przeslania_DMA to bajty
//dlatego rozmiar_przeslania_DMA/4

  	DMA_InitStructure.DMA_Channel = DMA_Channel_1;  
  	DMA_InitStructure.DMA_PeripheralBaseAddr = DCMI_DR_ADDRESS;	
  	DMA_InitStructure.DMA_Memory0BaseAddr = /*FSMC_LCD_ADDRESS;*/(uint32_t )bufor_RAM_pliku_obrazka_format;
 	DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralToMemory;
 	DMA_InitStructure.DMA_BufferSize = rozmiar_pojedynczego_przeslania_DMA/4;//ROZMIAR_BUF_OBRAZU;//1;
  	DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  	DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;//DMA_MemoryInc_Disable;
  	DMA_InitStructure.DMA_PeripheralDataSize = /*DMA_PeripheralDataSize_Byte;*/DMA_PeripheralDataSize_Word;
  	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;//DMA_MemoryDataSize_HalfWord;
  	DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;//DMA_Mode_Circular;
  	DMA_InitStructure.DMA_Priority = DMA_Priority_High;
  	DMA_InitStructure.DMA_FIFOMode = /*DMA_FIFOMode_Enable;*/DMA_FIFOMode_Disable;
  	DMA_InitStructure.DMA_FIFOThreshold = DMA_FIFOThreshold_Full;
  	DMA_InitStructure.DMA_MemoryBurst = DMA_MemoryBurst_Single;
  	DMA_InitStructure.DMA_PeripheralBurst = DMA_PeripheralBurst_Single;

    //konfiguracja i uruchomienie trybu Double Buffer mode 
	//po przesaniu caego buforu DMA (np. 0xFF00) nastpi automatyczne przeczenie 
	//na DMA_Memory1BaseAddr wskazujcy nastpn lokalizacj bufora danych dla nastpnego przesania DMA
	//Stream, wskanik buf.dla kolejnego przesania DMA, jako pierwszy bdzie uywany wskanik DMA_Memory0BaseAddr
	DMA_DoubleBufferModeConfig(DMA2_Stream1, (uint32_t )(bufor_RAM_pliku_obrazka_format+rozmiar_pojedynczego_przeslania_DMA+1),
                                DMA_Memory_0);
	//w-czenie trybu Double Buffer mode 
	DMA_DoubleBufferModeCmd(DMA2_Stream1,  ENABLE );

  	/* DCMI configuration */
	DCMI_JPEGCmd  ( ENABLE );//zmiana  0612
  	DCMI_Init(&DCMI_InitStructure);
	DCMI_ClearITPendingBit(DCMI_IT_FRAME | DCMI_IT_OVF | DCMI_IT_ERR | DCMI_IT_VSYNC | DCMI_IT_LINE);
	DCMI_ClearFlag (DCMI_FLAG_FRAMERI | DCMI_FLAG_OVFRI | DCMI_FLAG_ERRRI | DCMI_FLAG_VSYNCRI | DCMI_FLAG_LINERI | DCMI_FLAG_VSYNCRI);

	dcmi_flag_frameri_bak =FALSE;
	dcmi_flag_lineri_bak =FALSE;
	dcmi_flag_errri_bak =FALSE;
	dcmi_flag_ovfri_bak =FALSE;
	dcmi_flag_vsyncri_bak =FALSE;

  	/* DMA2 IRQ channel Configuration */
  	DMA_Init(DMA2_Stream1, &DMA_InitStructure);

  	DMA_ClearITPendingBit ( DMA2_Stream1, 	 DMA_IT_TCIF1  |
                                             DMA_IT_HTIF1  |
                                             DMA_IT_TEIF1  |
                                             DMA_IT_DMEIF1 |
                                             DMA_IT_FEIF1  );
 

  	/* Enable DMA2 stream 1 and DCMI interface then start image capture */
  	DMA_Cmd(DMA2_Stream1, ENABLE); 
  	while ( DISABLE == DMA_GetCmdStatus ( DMA2_Stream1 ) );
  	DCMI_Cmd(ENABLE); 
}
//-------------------------------------------------------------------
//procedura przesania danych obrazu JPG z kamery MT9D111 i zapis do wewntrznego buforu w pamici RAM
uint32_t Start_DMA_Dane_JPG_Ramki_MT9D111_do_Int_RAM(uint32_t *p_rozmiar_przeslania_max,  
													 uint32_t rozmiar_pojedynczego_przeslania_DMA)
//wy: ilo danych obrazu JPG w bajtach

//we:	-p_rozmiar_przeslania_max wskanik do zmiennej z max. rozmiarem bufora dla danych JPG
//		-rozmiar_pojedynczego_przeslania_DMA pojedyncze przesanie w trybie podwjnego buforu DMA
//wy:	-p_rozmiar_przeslania_max zmienna przekazuje ilo danych JPG w buforze
//		-status status operacji 

{
uint32_t licznik_DMA, ile_danych_jpg;
uint32_t ilosc_przeslanych_danych, rozmiar_przeslania_max;
uint32_t status;
uint8_t start_zegara_max_oczekiwania =TRUE;
extern char flaga_stopera1;


	GPIO_Linie_IO_Low(LINIA_LED_ZIELONY_WY);
	GPIO_Linie_IO_Low(LINIA_LED_CZERWONY_WY);

	rozmiar_przeslania_max =*p_rozmiar_przeslania_max;
	ilosc_przeslanych_danych =0;
	status =ERR_JPG_DANE_BRAK_DANYCH;
	DMA_ClearFlag( DMA2_Stream1, DMA_FLAG_TCIF1); 
	DCMI_CaptureCmd(ENABLE);
	
    while ( dcmi_flag_frameri_bak ==FALSE )
	{
		if (dcmi_flag_errri_bak ==TRUE || dcmi_flag_ovfri_bak ==TRUE) 
		{
			status =ERR_JPG_DANE_DCMI_BLAD;
			break;
		}
		if (DMA_GetFlagStatus(DMA2_Stream1, DMA_FLAG_TCIF1) ==SET)
		{
			ilosc_przeslanych_danych =ilosc_przeslanych_danych + rozmiar_pojedynczego_przeslania_DMA;
			DMA_ClearFlag( DMA2_Stream1, DMA_FLAG_TCIF1); 
		}
		licznik_DMA =DMA_GetCurrDataCounter  (DMA2_Stream1);
		ile_danych_jpg =rozmiar_pojedynczego_przeslania_DMA - (licznik_DMA*4); 
		if (rozmiar_przeslania_max <=(ilosc_przeslanych_danych +ile_danych_jpg))
		{
			status =ERR_JPG_DANE_BUFOR_ZA_MALY;
		 	break;
		}
	    status =ERR_JPG_DANE_BRAK_BLEDU;

		if (start_zegara_max_oczekiwania ==TRUE)
		{
			start_zegara_max_oczekiwania =FALSE;
			Start_Stoper1(5000);
		}
		if (flaga_stopera1 ==TRUE)
		{
			//przekroczony max.czas oczekiwania na odczyt ramki
			status =ERR_JPG_DANE_BUFOR_ZA_MALY;
		 	break;
		}
	}
   
    DCMI_CaptureCmd ( DISABLE );   
    DCMI_Cmd ( DISABLE );
	DCMI_JPEGCmd  ( DISABLE );//dodane	
	DMA_Cmd(DMA2_Stream1, DISABLE);
	DMA_DoubleBufferModeCmd(DMA2_Stream1,  DISABLE );


	licznik_DMA =DMA_GetCurrDataCounter  (DMA2_Stream1);
	ile_danych_jpg =rozmiar_pojedynczego_przeslania_DMA - (licznik_DMA*4);  //*4 bo jednostki przesania DMA to word a ilo danych to bajty
	ilosc_przeslanych_danych = ilosc_przeslanych_danych +ile_danych_jpg;

	GPIO_Linie_IO_High(LINIA_LED_ZIELONY_WY);
	GPIO_Linie_IO_High(LINIA_LED_CZERWONY_WY);
    
	//return ilosc_przeslanych_danych;
	*p_rozmiar_przeslania_max =ilosc_przeslanych_danych;
	return status;
}

