/**
  ******************************************************************************
  * plik    MT9D111.c
  * funkcja	procedury obsugi moduu kamery MT9D111
  * autor 	ARIES Ryszard Szymaniak www.arskam.com
  * wersja 	1.01
  * data    10-10-2012
  * opis   	ARS3-MT9D111 sterownik ARS3 obsugujcy modu kamery MT9D111 firmy APTINA
  * 		STM32F217VGT
  ******************************************************************************
  */  
#include "stm32f2xx_conf.h"
#include "MT9D111.h"
#include "Procedury Ogolne.h"
#include "EEPROM_procedury.h"
#include "Definicje_sprzet.h"
#include "Kamera.h"
#include "GPIO_procedury.h"
#include "Kamera_DMA.h"

extern struct {
         unsigned char adres;
         unsigned char subadres;
         unsigned char subadres_second;
         unsigned char ile;
         char *p_bufor;
       }i2c;
extern unsigned char i2c_r_w;

extern struct OZO_STRUCT{
	char *p_ozo_buf_plik_obraz;//wskanik do buforu z plikiem jpg obrazka
	char ozo_format;			//format ozo
	char ozo_format_bak;		//poprzedni format ozo
	int ozo_obraz_szer_pix;		//szeroko obrazka w pikselach
	int ozo_obraz_wys_lin;		//wysoko obrazka w liniach
	char ozo_obraz_bajty_na_pix;//ilo bajtw na pikselc w tym formacie obrazka
	int rozmiar_pliku_ozo;		//rozmiar pliku obrazka
	char ozo_plik_obraz_typ;	//typ pliku obrazka ozo
	}ozo_struct;


int currentPage =0xFF;

//--------------------
//zmiana adresu strony
unsigned char pageChange(int page)
//wy: 0 -bd, 1 -sukces
{
char cmd[3];
unsigned char status;
  
  	cmd[0] = 0xF0; // set register address
  	cmd[1] = 0;  // upper 8 bits;
  	cmd[2] = page; // lower 8 bits;

  	i2c.adres 	=0xBA;
  	i2c.p_bufor =&cmd[0];
  	i2c.ile 	=3;
  	i2c_r_w 	=0;//zapis 
  	status 		=I2c_(ADRES_I2C_0BAJT);
    if (status !=FALSE)
	{
  		currentPage = page; // set page
	}
	else
	{
		currentPage =10;
	}
  	return status;
}
//--------------------------------------------
unsigned short readMCU(unsigned short address)
{
unsigned short result = 0;
char data[2], status;
  
	status =readMCUMemory(address, data); 
	if (status ==FALSE) 
	{
		return 0;
	}
    result = (data[0] << 8) + (data[1] & 0xFF);
    
    return result;
}
//--------------------------------------------------------------
unsigned char readMCUMemory(unsigned short address, char *value)
//wy: 0 -bd, 1 -sukces
{
unsigned char status;
  
  	status =writeRegister(1,0xC6, address);
	if (status ==FALSE) return FALSE; 
	                     
	i2c.adres 		=0xBA;
  	i2c.subadres    =0xC8;
	i2c.p_bufor 	=value;
	i2c.ile 		=2;
	i2c_r_w 		=1;//odczyt
	status 			=I2c_(ADRES_I2C_1BAJT);  

	return status;
}
//---------------------------------------------------------------------------------------------
unsigned char writeMCUMemory(unsigned short address, unsigned short value, unsigned short mask)
//wy: 0 -bd, 1 -sukces
{
char temp[2];
unsigned short tempvalue;
unsigned char status;
   
  	if (mask != 0xFFFF)
  	{
		// read first
      	status =readMCUMemory(address, temp);
		if (status ==FALSE) return FALSE; 

      	if (!address & 0x8000)
        	tempvalue = temp[0] << 8 + temp[1]; // 16 bit value
      	else
        	tempvalue = temp[1];                // 8 bit value
      
      	tempvalue &= ~mask; // set mask bits to 0
      	value |= tempvalue; // store original bits
  	}
   
  	status =writeRegister(1,0xC6, address);
	if (status ==FALSE) 
	{
		return FALSE; 
	}
  	writeRegister(1,0xC8,value);
  
  	return status;
}
//----------------------------------------------------------------------
unsigned char readRegister(int page, unsigned char address, char *value)
//wy: 0 -bd, 1 -sukces
{
unsigned char status;

  
	if (currentPage != page)
    	status =pageChange(page);
	if (status ==FALSE) return FALSE; 

	i2c.adres 		=0xBA;
  	i2c.subadres 	=address; // set register address
	i2c.p_bufor	 	=value;
	i2c.ile 		=2;
	i2c_r_w 		=1;//odczyt
	status =I2c_(ADRES_I2C_1BAJT); 
	if (status ==FALSE) 
	{
		return FALSE; 
	}
  
  	return status;
}
//-----------------------------------------------------------
unsigned short usReadRegister(int page,unsigned char address)
{
char data[2], status;
unsigned short result;

  
  	if (currentPage != page)
    	status =pageChange(page);
	if (status ==FALSE) return 0; 

	i2c.adres 		=0xBA;
  	i2c.subadres 	=address; // set register address
	i2c.p_bufor	 	=&data[0];
	i2c.ile 		=2;
	i2c_r_w 		=1;//odczyt
	status =I2c_(ADRES_I2C_1BAJT); 
	if (status ==FALSE) 
	{
		return 0; 
	}

  	return result;
}
//-----------------------------------------------------------------------------------------------------
unsigned char writeRegister(int page, unsigned char address, unsigned short value)
//wy: 0 -bd, 1 -sukces
{
char cmd[3];
unsigned char status =TRUE;
  
  	if (currentPage != page)
    	status =pageChange(page);
	if (status ==FALSE) return status;
  	cmd[0] = address; // set register address
  	cmd[1] = value  >> 8;  // upper 8 bits;
  	cmd[2] = value & 0xFF; // lower 8 bits;
  	i2c.adres 	=0xBA;
  	i2c.p_bufor =&cmd[0];
  	i2c.ile 	=3;
  	i2c_r_w 	=0;//zapis 
  	status		=I2c_(ADRES_I2C_0BAJT);  
     
  	return status;       
}
//-----------------------------------------------------------------------------------------------------
unsigned char writeRegisterMask(int page, unsigned char address, unsigned short value, unsigned short mask)
//wy: 0 -bd, 1 -sukces
{
char cmd[3];
char temp[2];
unsigned short tempvalue;
unsigned char status;
  
  	if (currentPage != page)
    	status =pageChange(page);
	if (status ==FALSE) return status;

  	if (mask != 0xFFFF)
  	{
      	// read first
      	readRegister(page,address, temp);
      	tempvalue = (temp[0] << 8) + temp[1]; // 16 bit value
      	tempvalue &= ~mask; // set mask bits to 0
      	value |= tempvalue; // store original bits
  	}
    
  	cmd[0] = address; // set register address
  	cmd[1] = value  >> 8;  // upper 8 bits;
  	cmd[2] = value & 0xFF; // lower 8 bits;
  	i2c.adres 	=0xBA;
  	i2c.p_bufor =&cmd[0];
  	i2c.ile 	=3;
  	i2c_r_w 	=0;//zapis 
  	status		=I2c_(ADRES_I2C_0BAJT);  
     
  	return status;       
}

//--------------------
//inicjacja
void mt9d111_init(void)
{
   /* soft reset 
   1. Bypass the PLL, R0x65:0 = 0xA000, if it is currently used.
    2. Perform MCU reset by setting R0xC3:1 = 0x0501.
    3. Enable soft reset by setting R0x0D:0 = 0x0021. Bit 0 is used for the sensor core reset
    while bit 5 refers to SOC reset.
    4. Disable soft reset by setting R0x0D:0 = 0x0000.
    5. Wait 24 clock cycles before using the two-wire serial interface.
    */
    writeRegister(0,0x65,0xA000);
    writeRegister(1,0xC3,0x0501);
    writeRegister(0,0x0D, 0x0021);
    writeRegister(0,0x0D, 0x0000);
    Delay_ms(1);

//This file was generated by: MT9D111 (SOC2010) Register Wizard
//  Version: 4.3.2.30322    Build Date: 01/11/2012
//
//[MODE A PARAMETERS]
//
//Max Frame Time: n/a
//Max Frame Clocks: 150000000.0 clocks (3 MHz)
//No. of ADCs: 2
//Skip Mode: 1x cols, 1x rows, Bin Mode: Yes
//Active Sensor Columns: 808 pixels
//Active Sensor Rows: 608 rows
//Horiz Blanking: 8019 pixels
//Vert Blanking: 16384 rows
//Extra Delay: 11616 clocks
//
//Actual Frame Clocks: 150000000 clocks
//Row Time: 2942.333 usec / 8827 clocks
//Frame time: 50000 msec
//Frames per Sec: 0.020 fps
//
//Max Shutter Delay: 7508
//50Hz Flicker Period: 3.40 lines
//60Hz Flicker Period: 2.83 lines
//
//[MODE B PARAMETERS]
//
//Max Frame Time: 9803.92 msec
//Max Frame Clocks: 29411764.7 clocks (3 MHz)
//No. of ADCs: 1
//Skip Mode: 2x cols, 2x rows, Bin Mode: No
//Active Sensor Columns: 408 pixels / 816 clocks
//Active Sensor Rows: 308 rows
//Horiz Blanking: 4005 pixels / 8011 clocks
//Vert Blanking: 3024 rows
//Extra Delay: 100 clocks
//
//Actual Frame Clocks: 29411764 clocks
//Row Time: 2942.333 usec / 8827 clocks
//Frame time: 9803.921333 msec
//Frames per Sec: 0.102 fps
//
//Max Shutter Delay: 7962
//50Hz Flicker Period: 3.40 lines
//60Hz Flicker Period: 2.83 lines
//
//

	writeRegister(0, 0x05, 0x0FA5);        //HBLANK (B) = 4005
	writeRegister(0, 0x06, 0x0BD0);        //VBLANK (B) = 3024
	writeRegister(0, 0x07, 0x1F53);        //HBLANK (A) = 8019
	writeRegister(0, 0x08, 0x4000);        //VBLANK (A) = 16384
	writeRegister(0, 0x20, 0x0790);        //Read Mode (B) = 1936
	writeRegister(0, 0x21, 0x8000);        //Read Mode (A) = 32768
	writeRegister(0, 0x66, 0x1400);        //PLL Control 1 = 5120
	writeRegister(0, 0x67, 0x51F/*0x509*/);         //PLL Control 2 = 1289
	writeRegister(0, 0x65, 0xA000);        //Clock CNTRL: PLL ON = 40960
	writeRegister(0, 0x65, 0x2000);        //Clock CNTRL: USE PLL = 8192

	writeRegisterMask(0, 0x0A, 0x00F0, 0x00F0);

	writeRegister(1, 0xC6, 0xA122);        //Enter Preview: Auto Exposure
	writeRegister(1, 0xC8, 0x01);//0x01);        //      = 1
	writeRegister(1, 0xC6, 0xA123);        //Enter Preview: Flicker Detection
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA124);        //Enter Preview: Auto White Balance
	writeRegister(1, 0xC8, 0x01);        //      = 1
	writeRegister(1, 0xC6, 0xA125);        //Enter Preview: Auto Focus
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA126);        //Enter Preview: Histogram
	writeRegister(1, 0xC8, 0x01);        //      = 1
	writeRegister(1, 0xC6, 0xA127);        //Enter Preview: Strobe Control
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA128);        //Enter Preview: Skip Control
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA129);        //In Preview: Auto Exposure
	writeRegister(1, 0xC8, 0x03);        //      = 3
	writeRegister(1, 0xC6, 0xA12A);        //In Preview: Flicker Detection
	writeRegister(1, 0xC8, 0x02);        //      = 2
	writeRegister(1, 0xC6, 0xA12B);        //In Preview: Auto White Balance
	writeRegister(1, 0xC8, 0x03);        //      = 3
	writeRegister(1, 0xC6, 0xA12C);        //In Preview: Auto Focus
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA12D);        //In Preview: Histogram
	writeRegister(1, 0xC8, 0x03);        //      = 3
	writeRegister(1, 0xC6, 0xA12E);        //In Preview: Strobe Control
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA12F);        //In Preview: Skip Control
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA130);        //Exit Preview: Auto Exposure
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA131);        //Exit Preview: Flicker Detection
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA132);        //Exit Preview: Auto White Balance
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA133);        //Exit Preview: Auto Focus
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA134);        //Exit Preview: Histogram
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA135);        //Exit Preview: Strobe Control
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA136);        //Exit Preview: Skip Control
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA137);        //Capture: Auto Exposure
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA138);        //Capture: Flicker Detection
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA139);        //Capture: Auto White Balance
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA13A);        //Capture: Auto Focus
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA13B);        //Capture: Histogram
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA13C);        //Capture: Strobe Control
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA13D);        //Capture: Skip Control
	writeRegister(1, 0xC8, 0x00);        //      = 0


	writeRegister(1, 0xC6, 0x2703);        //Output Width (A)
	writeRegister(1, 0xC8, 0x0320);        //      = 800
	writeRegister(1, 0xC6, 0x2705);        //Output Height (A)
	writeRegister(1, 0xC8, 0x0258);        //      = 600
	writeRegister(1, 0xC6, 0x2707);        //Output Width (B)
	writeRegister(1, 0xC8, 0x0190);        //      = 400
	writeRegister(1, 0xC6, 0x2709);        //Output Height (B)
	writeRegister(1, 0xC8, 0x012C);        //      = 300
	writeRegister(1, 0xC6, 0x270B);        //mode_config
	writeRegister(1, 0xC8, 0x0030);        //      = 48
	writeRegister(1, 0xC6, 0x270F);        //Row Start (A)
	writeRegister(1, 0xC8, 0x01C);        //      = 28
	writeRegister(1, 0xC6, 0x2711);        //Column Start (A)
	writeRegister(1, 0xC8, 0x03C);        //      = 60
	writeRegister(1, 0xC6, 0x2713);        //Row Height (A)
	writeRegister(1, 0xC8, 0x4B0);        //      = 1200
	writeRegister(1, 0xC6, 0x2715);        //Column Width (A)
	writeRegister(1, 0xC8, 0x640);        //      = 1600
	writeRegister(1, 0xC6, 0x2717);        //Extra Delay (A)
	writeRegister(1, 0xC8, 0x560);        //      = 1376
	writeRegister(1, 0xC6, 0x2719);        //Row Speed (A)
	writeRegister(1, 0xC8, 0x0011);        //      = 17
	writeRegister(1, 0xC6, 0x271B);        //Row Start (B)
	writeRegister(1, 0xC8, 0x01C);        //      = 28
	writeRegister(1, 0xC6, 0x271D);        //Column Start (B)
	writeRegister(1, 0xC8, 0x03C);        //      = 60
	writeRegister(1, 0xC6, 0x271F);        //Row Height (B)		 
	writeRegister(1, 0xC8, 0x4B0/*0x258*/);        //      = 600
	writeRegister(1, 0xC6, 0x2721);        //Column Width (B)
	writeRegister(1, 0xC8, 0x640/*0x320*/);        //      = 800
	writeRegister(1, 0xC6, 0x2723);        //Extra Delay (B)	
	writeRegister(1, 0xC8, 0x064);         //      = 100
	writeRegister(1, 0xC6, 0x2725);        //Row Speed (B)
	writeRegister(1, 0xC8, 0x0011);        //      = 17
	writeRegister(1, 0xC6, 0x2727);        //Crop_X0 (A)
	writeRegister(1, 0xC8, 0x0000);        //      = 0
	writeRegister(1, 0xC6, 0x2729);        //Crop_X1 (A)
	writeRegister(1, 0xC8, 0x0320);        //      = 800
	writeRegister(1, 0xC6, 0x272B);        //Crop_Y0 (A)
	writeRegister(1, 0xC8, 0x0000);        //      = 0
	writeRegister(1, 0xC6, 0x272D);        //Crop_Y1 (A)
	writeRegister(1, 0xC8, 0x0258);        //      = 600
	writeRegister(1, 0xC6, 0x2735);        //Crop_X0 (B)
	writeRegister(1, 0xC8, 0x0000);        //      = 0
	writeRegister(1, 0xC6, 0x2737);        //Crop_X1 (B)
	writeRegister(1, 0xC8, 0x0190);        //      = 400
	writeRegister(1, 0xC6, 0x2739);        //Crop_Y0 (B)
	writeRegister(1, 0xC8, 0x0000);        //      = 0
	writeRegister(1, 0xC6, 0x273B);        //Crop_Y1 (B)
	writeRegister(1, 0xC8, 0x012C);        //      = 300
	writeRegister(1, 0xC6, 0xA743);        //Gamma and Contrast Settings (A)
	writeRegister(1, 0xC8, 0x42);        //      = 66
	writeRegister(1, 0xC6, 0xA744);        //Gamma and Contrast Settings (B)
	writeRegister(1, 0xC8, 0x42);        //      = 66

	writeRegister(1, 0xC6, 0xA745);        //Gamma Table 0 (A)
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA746);        //Gamma Table 1 (A)
	writeRegister(1, 0xC8, 0x14);        //      = 20
	writeRegister(1, 0xC6, 0xA747);        //Gamma Table 2 (A)
	writeRegister(1, 0xC8, 0x23);        //      = 35
	writeRegister(1, 0xC6, 0xA748);        //Gamma Table 3 (A)
	writeRegister(1, 0xC8, 0x3A);        //      = 58
	writeRegister(1, 0xC6, 0xA749);        //Gamma Table 4 (A)
	writeRegister(1, 0xC8, 0x5E);        //      = 94
	writeRegister(1, 0xC6, 0xA74A);        //Gamma Table 5 (A)
	writeRegister(1, 0xC8, 0x76);        //      = 118
	writeRegister(1, 0xC6, 0xA74B);        //Gamma Table 6 (A)
	writeRegister(1, 0xC8, 0x88);        //      = 136
	writeRegister(1, 0xC6, 0xA74C);        //Gamma Table 7 (A)
	writeRegister(1, 0xC8, 0x96);        //      = 150
	writeRegister(1, 0xC6, 0xA74D);        //Gamma Table 8 (A)
	writeRegister(1, 0xC8, 0xA3);        //      = 163
	writeRegister(1, 0xC6, 0xA74E);        //Gamma Table 9 (A)
	writeRegister(1, 0xC8, 0xAF);        //      = 175
	writeRegister(1, 0xC6, 0xA74F);        //Gamma Table 10 (A)
	writeRegister(1, 0xC8, 0xBA);        //      = 186
	writeRegister(1, 0xC6, 0xA750);        //Gamma Table 11 (A)
	writeRegister(1, 0xC8, 0xC4);        //      = 196
	writeRegister(1, 0xC6, 0xA751);        //Gamma Table 12 (A)
	writeRegister(1, 0xC8, 0xCE);        //      = 206
	writeRegister(1, 0xC6, 0xA752);        //Gamma Table 13 (A)
	writeRegister(1, 0xC8, 0xD7);        //      = 215
	writeRegister(1, 0xC6, 0xA753);        //Gamma Table 14 (A)
	writeRegister(1, 0xC8, 0xE0);        //      = 224
	writeRegister(1, 0xC6, 0xA754);        //Gamma Table 15 (A)
	writeRegister(1, 0xC8, 0xE8);        //      = 232
	writeRegister(1, 0xC6, 0xA755);        //Gamma Table 16 (A)
	writeRegister(1, 0xC8, 0xF0);        //      = 240
	writeRegister(1, 0xC6, 0xA756);        //Gamma Table 17 (A)
	writeRegister(1, 0xC8, 0xF8);        //      = 248
	writeRegister(1, 0xC6, 0xA757);        //Gamma Table 18 (A)
	writeRegister(1, 0xC8, 0xFF);        //      = 255
	writeRegister(1, 0xC6, 0xA758);        //Gamma Table 0 (B)
	writeRegister(1, 0xC8, 0x00);        //      = 0
	writeRegister(1, 0xC6, 0xA759);        //Gamma Table 1 (B)
	writeRegister(1, 0xC8, 0x14);        //      = 20
	writeRegister(1, 0xC6, 0xA75A);        //Gamma Table 2 (B)
	writeRegister(1, 0xC8, 0x23);        //      = 35
	writeRegister(1, 0xC6, 0xA75B);        //Gamma Table 3 (B)
	writeRegister(1, 0xC8, 0x3A);        //      = 58
	writeRegister(1, 0xC6, 0xA75C);        //Gamma Table 4 (B)
	writeRegister(1, 0xC8, 0x5E);        //      = 94
	writeRegister(1, 0xC6, 0xA75D);        //Gamma Table 5 (B)
	writeRegister(1, 0xC8, 0x76);        //      = 118
	writeRegister(1, 0xC6, 0xA75E);        //Gamma Table 6 (B)
	writeRegister(1, 0xC8, 0x88);        //      = 136
	writeRegister(1, 0xC6, 0xA75F);        //Gamma Table 7 (B)
	writeRegister(1, 0xC8, 0x96);        //      = 150
	writeRegister(1, 0xC6, 0xA760);        //Gamma Table 8 (B)
	writeRegister(1, 0xC8, 0xA3);        //      = 163
	writeRegister(1, 0xC6, 0xA761);        //Gamma Table 9 (B)
	writeRegister(1, 0xC8, 0xAF);        //      = 175
	writeRegister(1, 0xC6, 0xA762);        //Gamma Table 10 (B)
	writeRegister(1, 0xC8, 0xBA);        //      = 186
	writeRegister(1, 0xC6, 0xA763);        //Gamma Table 11 (B)
	writeRegister(1, 0xC8, 0xC4);        //      = 196
	writeRegister(1, 0xC6, 0xA764);        //Gamma Table 12 (B)
	writeRegister(1, 0xC8, 0xCE);        //      = 206
	writeRegister(1, 0xC6, 0xA765);        //Gamma Table 13 (B)
	writeRegister(1, 0xC8, 0xD7);        //      = 215
	writeRegister(1, 0xC6, 0xA766);        //Gamma Table 14 (B)
	writeRegister(1, 0xC8, 0xE0);        //      = 224
	writeRegister(1, 0xC6, 0xA767);        //Gamma Table 15 (B)
	writeRegister(1, 0xC8, 0xE8);        //      = 232
	writeRegister(1, 0xC6, 0xA768);        //Gamma Table 16 (B)
	writeRegister(1, 0xC8, 0xF0);        //      = 240
	writeRegister(1, 0xC6, 0xA769);        //Gamma Table 17 (B)
	writeRegister(1, 0xC8, 0xF8);        //      = 248
	writeRegister(1, 0xC6, 0xA76A);        //Gamma Table 18 (B)
	writeRegister(1, 0xC8, 0xFF);        //      = 255

	writeRegister(1, 0xC6, 0x276D);        //FIFO_Conf1 (A)
	writeRegister(1, 0xC8, 0xE0E1);        //      = 57569
	writeRegister(1, 0xC6, 0xA76F);        //FIFO_Conf2 (A)
	writeRegister(1, 0xC8, 0xE1);        //      = 225
	writeRegister(1, 0xC6, 0x2774);        //FIFO_Conf1 (B)
	writeRegister(1, 0xC8, 0xE0E1);        //      = 57569
	writeRegister(1, 0xC6, 0xA776);        //FIFO_Conf2 (B)
	writeRegister(1, 0xC8, 0xE1);        //      = 225
	writeRegister(1, 0xC6, 0x220B);        //Max R12 (B)(Shutter Delay)
	writeRegister(1, 0xC8, 0x1D54);        //      = 7508
	writeRegister(1, 0xC6, 0xA217);        //IndexTH23
	writeRegister(1, 0xC8, 0x08);        //      = 8
	writeRegister(1, 0xC6, 0x2228);        //RowTime (msclk per)/4	****
	writeRegister(1, 0xC8, 0x089C);        //      = 2204  def =531
	writeRegister(1, 0xC6, 0x222F);        //R9 Step
	writeRegister(1, 0xC8, 0x009D);        //      = 2	   def =157
	writeRegister(1, 0xC6, 0xA408);        //search_f1_50
	writeRegister(1, 0xC8, 0xFF);        //      = 255
	writeRegister(1, 0xC6, 0xA409);        //search_f2_50
	writeRegister(1, 0xC8, 0x01);        //      = 1
	writeRegister(1, 0xC6, 0xA40A);        //search_f1_60		  ***
	writeRegister(1, 0xC8, 0xFF);        //      = 255
	writeRegister(1, 0xC6, 0xA40B);        //search_f2_60
	writeRegister(1, 0xC8, 0x01);        //      = 1
	writeRegister(1, 0xC6, 0x2411);        //R9_Step_60
	writeRegister(1, 0xC8, 0x0002);        //      = 2
	writeRegister(1, 0xC6, 0x2413);        //R9_Step_50
	writeRegister(1, 0xC8, 0x0003);        //      = 3

	Delay_ms(1);//wait(0.5);
	writeRegister(1, 0xC6, 0xA103);        //Refresh Sequencer Mode
	writeRegister(1, 0xC8, 0x06);        //      = 6
	Delay_ms(1);//wait(0.5);
	writeRegister(1, 0xC6, 0xA103);        //Refresh Sequencer
	writeRegister(1, 0xC8, 0x05);        //      = 5
	Delay_ms(100);//!!!
} 
//------------------------
//zmiana parametrw kamery kontekstu B
char mt9d111_Zmiana_Standardowych_Rozmiarow_Obrazu_Kontekst_B(char format_obrazu)
//we: numer ustawianego formatu obrazu
//wy: TRUE -format ustawiony, FALSE -bd ustawienia formatu obrazu
{
unsigned int image_width_tmp, image_height_tmp;
unsigned char status;
unsigned short state;

//extern int image_width, image_height;
extern int obraz_ozo_parametry_tab[][4];
  
	if (format_obrazu ==ozo_struct.ozo_format_bak) return TRUE;
	else ozo_struct.ozo_format_bak =format_obrazu;

	if ((format_obrazu ==NUMER_FORMATU_OZO_VGA_BMP) || (format_obrazu ==NUMER_FORMATU_OZO_VGA_JPG))
	{
		image_width_tmp =obraz_ozo_parametry_tab[format_obrazu][OBRAZ_PARAM_PIKSELE_POZ];
		image_height_tmp =obraz_ozo_parametry_tab[format_obrazu][OBRAZ_PARAM_LINIE_POZ];
		status =mt9d111_zmiana_rozdzielczosci_obrazu_kontekstu(KONTEKST_B, image_width_tmp, image_height_tmp);
		if (status ==FALSE) return FALSE;
		ozo_struct.ozo_obraz_szer_pix =image_width_tmp;
		ozo_struct.ozo_obraz_wys_lin =image_height_tmp;
		ozo_struct.ozo_obraz_bajty_na_pix =obraz_ozo_parametry_tab[NUMER_FORMATU_OZO_VGA_BMP][OBRAZ_PARAM_BAJTY_NA_PIKS];

		mt9d111_zmiana_rozdzielczosci_obrazu_kontekstu(KONTEKST_A, image_width_tmp, image_height_tmp);
	}
	if ((format_obrazu ==NUMER_FORMATU_OZO_QVGA_BMP) || (format_obrazu ==NUMER_FORMATU_OZO_QVGA_JPG))
	{	
		image_width_tmp =obraz_ozo_parametry_tab[format_obrazu][OBRAZ_PARAM_PIKSELE_POZ];
		image_height_tmp =obraz_ozo_parametry_tab[format_obrazu][OBRAZ_PARAM_LINIE_POZ];
		status ==mt9d111_zmiana_rozdzielczosci_obrazu_kontekstu(KONTEKST_B, image_width_tmp, image_height_tmp);
		if (status ==FALSE) return FALSE;
		ozo_struct.ozo_obraz_szer_pix =image_width_tmp;
		ozo_struct.ozo_obraz_wys_lin =image_height_tmp;
		ozo_struct.ozo_obraz_bajty_na_pix =obraz_ozo_parametry_tab[NUMER_FORMATU_OZO_QVGA_BMP][OBRAZ_PARAM_BAJTY_NA_PIKS];

		mt9d111_zmiana_rozdzielczosci_obrazu_kontekstu(KONTEKST_A, image_width_tmp, image_height_tmp);
	}
	if ((format_obrazu ==NUMER_FORMATU_OZO_QQVGA_BMP) || (format_obrazu ==NUMER_FORMATU_OZO_QQVGA_JPG))
	{
		image_width_tmp =obraz_ozo_parametry_tab[format_obrazu][OBRAZ_PARAM_PIKSELE_POZ];
		image_height_tmp =obraz_ozo_parametry_tab[format_obrazu][OBRAZ_PARAM_LINIE_POZ];

		status =mt9d111_zmiana_rozdzielczosci_obrazu_kontekstu(KONTEKST_B, image_width_tmp, image_height_tmp);
		if (status ==FALSE) return FALSE;
		ozo_struct.ozo_obraz_szer_pix =image_width_tmp;
		ozo_struct.ozo_obraz_wys_lin =image_height_tmp;

		mt9d111_zmiana_rozdzielczosci_obrazu_kontekstu(KONTEKST_A, image_width_tmp, image_height_tmp);
		ozo_struct.ozo_obraz_bajty_na_pix =obraz_ozo_parametry_tab[NUMER_FORMATU_OZO_QQVGA_BMP][OBRAZ_PARAM_BAJTY_NA_PIKS];
	}

	//po zmianie wymiarw obrazu odczyt 1 ramki dla przypieszenia dziasania automatyki AE
	//clear the capture video mode bit
    writeMCUMemory(SEQ_CAPTUREPARAMS_MODE, 0, BRAK_MASKI_ZAPISU_DO_MEM);
	writeRegister(CONTEXT_CONTROL,0xB);

	//ilo ramek obrazu zapisywanych do buforu =1
    writeMCUMemory(SEQ_CAPTUREPARAMS_NUMFRAMES, 1, BRAK_MASKI_ZAPISU_DO_MEM);
    writeMCUMemory(SEQ_CMD,6, BRAK_MASKI_ZAPISU_DO_MEM);
	Delay_ms(1);
	writeMCUMemory(SEQ_CMD,5, BRAK_MASKI_ZAPISU_DO_MEM); 		// refresh
  	
	//call the "CAPTURE" command
	state =writeMCUMemory(SEQ_CMD, 2, BRAK_MASKI_ZAPISU_DO_MEM);
	while (readMCU(MODE_CONTEXT) !=1) 
	{
		writeMCUMemory(SEQ_CMD, 2, BRAK_MASKI_ZAPISU_DO_MEM);
	}
	Delay_ms(1);//!!!

	do
	{       
    	state = readMCU(SEQ_STATE);  
	}                       
    while (state == 3);	
 
    /* capture done. Wait until state is preview again */
    while (state != 3)
    {
      state = readMCU(SEQ_STATE);  
	}

	return TRUE;
}
//-----------------------------
//zmiana wymiarw obrazu
unsigned char mt9d111_zmiana_rozdzielczosci_obrazu_kontekstu(char kontekst, int szerokosc, int wysokosc)
//wy: status
{
unsigned short tst_szerokosc=0, tst_wysokosc=0;
char result[2];

    //sprawdzenie poprawnoci parametrw
	if ((kontekst !=KONTEKST_A && kontekst !=KONTEKST_B) ||
		(szerokosc >SZEROKOSC_OBRAZU_MAX || wysokosc >WYSOKOSC_OBRAZU_MAX) ||
		(szerokosc <=SZEROKOSC_OBRAZU_MIN || wysokosc <=WYSOKOSC_OBRAZU_MIN))
	{
		return FALSE;
	}
	//Table 17: Driver Variables-Mode/Context Driver (ID = 7)
	if (kontekst ==KONTEKST_A)
	{
		writeRegister(1, 0xC6, MODE_OUTPUT_WIDTH_A);        	//Output Width (A)
		writeRegister(1, 0xC8, (unsigned short)szerokosc);      //      = szerokosc
		writeRegister(1, 0xC6, MODE_OUTPUT_HEIGHT_A);        	//Output Height (A)
		writeRegister(1, 0xC8, (unsigned short)wysokosc);       //      = wysokosc
		//rozmiar obrazu kontekstu A powinien by ju wczeniej ustawiony
		//obraz nie bdzie przycinany
		writeMCUMemory(MODE_CROP_X0_A, 0, BRAK_MASKI_ZAPISU_DO_MEM);
		writeMCUMemory(MODE_CROP_X1_A, 800, BRAK_MASKI_ZAPISU_DO_MEM);
		writeMCUMemory(MODE_CROP_Y0_A, 0, BRAK_MASKI_ZAPISU_DO_MEM);
		writeMCUMemory(MODE_CROP_Y1_A, 600, BRAK_MASKI_ZAPISU_DO_MEM);

	    writeRegister(1, 0xC6, MODE_OUTPUT_WIDTH_A);
	    readRegister(1, 0xC8, result);
		tst_szerokosc = (result[0] << 8) + (result[1] & 0xFF);
		writeRegister(1, 0xC6, MODE_OUTPUT_HEIGHT_A);
		readRegister(1, 0xC8, result);
		tst_wysokosc = (result[0] << 8) + (result[1] & 0xFF);
		if (tst_szerokosc !=szerokosc || tst_wysokosc !=tst_wysokosc)
		{
			return FALSE;
		}

	}
	else
	{	   
		writeRegister(1, 0xC6, MODE_OUTPUT_WIDTH_B);        	//Output Width (B)
		writeRegister(1, 0xC8, (unsigned short)szerokosc);      //      = szerokosc
		writeRegister(1, 0xC6, MODE_OUTPUT_HEIGHT_B);        	//Output Height (B)
		writeRegister(1, 0xC8, (unsigned short)wysokosc);       //      = wysokosc
		//rozmiar obrazu kontekstu B powinien by ju wczeniej ustawiony
		//obraz nie bdzie przycinany
		writeMCUMemory(MODE_CROP_X0_B, 0, BRAK_MASKI_ZAPISU_DO_MEM);
		writeMCUMemory(MODE_CROP_X1_B, 800, BRAK_MASKI_ZAPISU_DO_MEM);
		writeMCUMemory(MODE_CROP_Y0_B, 0, BRAK_MASKI_ZAPISU_DO_MEM);
		writeMCUMemory(MODE_CROP_Y1_B, 600, BRAK_MASKI_ZAPISU_DO_MEM);

	    writeRegister(1, 0xC6, MODE_OUTPUT_WIDTH_B);
	    readRegister(1, 0xC8, result);
		tst_szerokosc = (result[0] << 8) + (result[1] & 0xFF);
		writeRegister(1, 0xC6, MODE_OUTPUT_HEIGHT_B);
		readRegister(1, 0xC8, result);
		tst_wysokosc = (result[0] << 8) + (result[1] & 0xFF);
		if (tst_szerokosc !=szerokosc || tst_wysokosc !=tst_wysokosc)
		{
			return FALSE;
		}
	}

    writeMCUMemory(SEQ_CMD,6, BRAK_MASKI_ZAPISU_DO_MEM);
	Delay_ms(1);
	writeMCUMemory(SEQ_CMD,5, BRAK_MASKI_ZAPISU_DO_MEM); // refresh
	Delay_ms(100);//!!!

	return TRUE;
}
//--------------------------------
//ustawienie formatu danych obrazu
unsigned char mt9d111_zmian_formatu_danych_obrazu_kontekstu(char kontekst)
{

	//sprawdzenie poprawnoci parametrw
	if (kontekst !=KONTEKST_A && kontekst !=KONTEKST_B) return FALSE;
	//ustawienie formatu RGB565
	if (kontekst ==KONTEKST_A)
	{
		writeRegister(1, 0xC6, MODE_OUTPUT_FORMAT_A);        	//format danych kontekst (A)
	}
	else
	{
		writeRegister(1, 0xC6, MODE_OUTPUT_FORMAT_B);        	//format danych kontekst (B)
	}
	writeRegister(1, 0xC8, 0x20);      							//b7-6	00	//16-bit RGB565
																//b5	1	//output RGB
																//b4	0	//NIE ITU-R BT.601 codes when bypassing FIFO
																//b3	0	//NIE Monochrome output
																//b2	0	//Reserved
																//b1	0	//NIE Swaps chrominance byte with luminance byte in YUV output. 
																			//In RGB mode, swaps odd and even bytes
																//b0	0	//NIE In YUV output mode, swaps Cb and Cr channels. 
																			//In RGB mode, swaps R and B  
    writeMCUMemory(SEQ_CMD,6, BRAK_MASKI_ZAPISU_DO_MEM);
	Delay_ms(1);
	writeMCUMemory(SEQ_CMD,5, BRAK_MASKI_ZAPISU_DO_MEM); 		// refresh
	Delay_ms(100);//!!!

	return TRUE;
}
//-------------------------------------------------
//zapis do buforu 1 ramki obrazu RGB nie kodowanego
unsigned char mt9d111_zapis_do_buf_ramki_RGB(void)
{
unsigned short state;

	//clear the capture video mode bit
    writeMCUMemory(SEQ_CAPTUREPARAMS_MODE, 0, BRAK_MASKI_ZAPISU_DO_MEM);
	writeRegister(CONTEXT_CONTROL,0xB);

	//ilo ramek obrazu zapisywanych do buforu =1
    writeMCUMemory(SEQ_CAPTUREPARAMS_NUMFRAMES, 1, BRAK_MASKI_ZAPISU_DO_MEM);
    writeMCUMemory(SEQ_CMD,6, BRAK_MASKI_ZAPISU_DO_MEM);
	Delay_ms(1);
	writeMCUMemory(SEQ_CMD,5, BRAK_MASKI_ZAPISU_DO_MEM); 		// refresh
  	
	Delay_ms(1);//!!!

	//call the "CAPTURE" command
	state =writeMCUMemory(SEQ_CMD, 2, BRAK_MASKI_ZAPISU_DO_MEM);
	while (readMCU(MODE_CONTEXT) !=1) 
	{
		writeMCUMemory(SEQ_CMD, 2, BRAK_MASKI_ZAPISU_DO_MEM);
	}
	Delay_ms(1);//!!!

    /* capture in progress when state > preview */ 
	do
	{       
    	state = readMCU(SEQ_STATE);  
	}                       
    while (state == 3);	
 
    /* capture done. Wait until state is preview again */
    while (state != 3)
    {
      state = readMCU(SEQ_STATE);  
	  if (state ==7)
	  {
		Start_DMA_MT9D111_BMP_do_Int_RAM(ozo_struct.ozo_obraz_szer_pix, 
							 			   ozo_struct.ozo_obraz_wys_lin, 
							 			   ozo_struct.ozo_obraz_bajty_na_pix);
		break;
	  }      
	  if (state ==5)
	  {
	  }        
	  if (state ==6)
	  {
	  }        
	    
    }
 
	return TRUE;
}
//-------------------------------------
//procedura zmiany parametru JASKRAWO
unsigned char MT9D111_Jaskrawosc_regulacja(unsigned char param_jaskrawosc)
{
#define MT9D111_JASKRAWOSC_MAX	255
unsigned short param_znormalizowany;
unsigned char status;
char flagi, maska_pozycji;

	status =EepOdczyt(EEP_ADR_FLAGI_OPCJI_PRACY_KAMERY, 1, &flagi);
	if (status ==0) return FALSE;//bd odczytu z EEPROM
	maska_pozycji =1<<EEP_POZ_KAM_FLAG_AE_STANDARD;
	if ((flagi &maska_pozycji) !=0)
	{
		//ustawiona flaga AE_standard blokujca zmiany ustawienia jaskrawoci
		return TRUE;// brak bdu ale ustwienia nie zostan zmienione
	}
	status =EepZapis(EEP_ADR_KAMERA_JASKRAWOSC, 
			1, (char *)&param_jaskrawosc);
	if (status ==0)	return FALSE;//bd zapisu do EEPROM

	param_znormalizowany =(param_jaskrawosc * MT9D111_JASKRAWOSC_MAX) /KAMERA_POZIOM_JASKRAWOSCI_MAX;
	if (param_znormalizowany <10) param_znormalizowany =10;	
	status =writeMCUMemory(AE_TARGET_JASKRAWOSC, param_znormalizowany, BRAK_MASKI_ZAPISU_DO_MEM);

	return status;
}
//-------------------------------------
//procedura zmiany parametru KONTRAST
unsigned char MT9D111_Kontrast_regulacja(unsigned char param_kontrast)
{
#define MT9D111_KONTRAST_MAX	4
#define MASKA_ZAPISU_KONTRASTU	0x70
unsigned short param_znormalizowany;
unsigned char status;
char flagi, maska_pozycji;

	status =EepOdczyt(EEP_ADR_FLAGI_OPCJI_PRACY_KAMERY, 1, &flagi);
	if (status ==0) return FALSE;//bd odczytu z EEPROM
	maska_pozycji =1<<EEP_POZ_KAM_FLAG_AE_STANDARD;
	if ((flagi &maska_pozycji) !=0)
	{
		//ustawiona flaga AE_standard blokujca zmiany ustawienia jaskrawoci
		return TRUE;// brak bdu ale ustwienia nie zostan zmienione
	}
	status =EepZapis(EEP_ADR_KAMERA_KONTRAST, 
			1, (char *)&param_kontrast);
	if (status ==0)	return FALSE;//bd zapisu do EEPROM

	param_znormalizowany =(param_kontrast * MT9D111_KONTRAST_MAX) /KAMERA_POZIOM_KONTRASTU_MAX;
	param_znormalizowany =(param_znormalizowany <<4) & MASKA_ZAPISU_KONTRASTU;
	status =writeMCUMemory(GAM_CONT_B_KONTRAST, param_znormalizowany, MASKA_ZAPISU_KONTRASTU);
	writeRegister(1, 0xC6, 0xA103);        //Refresh Sequencer Mode
	writeRegister(1, 0xC8, 0x06);        //      = 6
	Delay_ms(1);
	writeRegister(1, 0xC6, 0xA103);        //Refresh Sequencer
	writeRegister(1, 0xC8, 0x05);        //      = 5

	return status;
}
//-------------------------------------
//procedura zmiany parametru NASYCENIE
unsigned char MT9D111_Nasycenie_regulacja(unsigned char param_nasycenie)
{
#define MT9D111_NASYCENIE_MAX	128
unsigned short param_znormalizowany;
unsigned char status;
char flagi, maska_pozycji;

	status =EepOdczyt(EEP_ADR_FLAGI_OPCJI_PRACY_KAMERY, 1, &flagi);
	if (status ==0) return FALSE;//bd odczytu z EEPROM
	maska_pozycji =1<<EEP_POZ_KAM_FLAG_AE_STANDARD;
	if ((flagi &maska_pozycji) !=0)
	{
		//ustawiona flaga AE_standard blokujca zmiany ustawienia jaskrawoci
		return TRUE;// brak bdu ale ustwienia nie zostan zmienione
	}
	status =EepZapis(EEP_ADR_KAMERA_NASYCENIE, 
			1, (char *)&param_nasycenie);
	if (status ==0)	return FALSE;//bd zapisu do EEPROM

	param_znormalizowany =(param_nasycenie * MT9D111_NASYCENIE_MAX) /KAMERA_POZIOM_NASYCENIA_MAX;
	status =writeMCUMemory(AWB_SATURATION_NASYCENIE, param_znormalizowany, BRAK_MASKI_ZAPISU_DO_MEM);

	return status;
}
//------------------------------------------------------
//ustawienie luster (mirorw) obrazu w poziomie i pionie
unsigned char mt9d111_ustawienie_luster_obrazu(char flagi_luster)
{
char maska_pozycji;
unsigned short value=0;

	maska_pozycji =1<<EEP_POZ_KAM_FLAG_LUSTRO_POZIOM;
	if ((flagi_luster &maska_pozycji) !=0)
	{
		value =value | 0x02;
	}
	maska_pozycji =1<<EEP_POZ_KAM_FLAG_LUSTRO_PION;
	if ((flagi_luster &maska_pozycji) !=0)
	{
		value =value | 0x01;
	}
	writeRegisterMask(READ_MODE_B, value, 0x03);

	return TRUE;
}

//-------------------------------------------
//kontrola i wczenie/wyczenie automatu AE
unsigned char mt9d111_kontrola_ustawienia_AE_standard(char flaga_AE)
{
char maska_pozycji;
char value=FALSE;
unsigned short param_znormalizowany;
unsigned char param;
#define JASKRAWOSC_STANDARD		0x3C
#define KONTRAST_STANDARD		0x42
#define NASYCENIE_STANDARD		0x80

//char akt_ustaw_reg[2];
//unsigned short aktualne_ustawienie;


	maska_pozycji =1<<EEP_POZ_KAM_FLAG_AE_STANDARD;
	if ((flaga_AE &maska_pozycji) !=0)
	{
		value =TRUE;
	}
	if (value ==FALSE)
	{
	}
	else
	{
		//powrt do standardowych ustawie jaskrawoci, kontrastu, nasycenia
		param_znormalizowany =JASKRAWOSC_STANDARD;
		writeMCUMemory(AE_TARGET_JASKRAWOSC, param_znormalizowany, BRAK_MASKI_ZAPISU_DO_MEM);
		param =(KAMERA_POZIOM_JASKRAWOSCI_MAX * param_znormalizowany) /MT9D111_JASKRAWOSC_MAX;
		EepZapis(EEP_ADR_KAMERA_JASKRAWOSC, 1, (char *)&param);

		param_znormalizowany =KONTRAST_STANDARD;
		writeMCUMemory(GAM_CONT_B_KONTRAST, param_znormalizowany, MASKA_ZAPISU_KONTRASTU);
		param =(KAMERA_POZIOM_KONTRASTU_MAX * param_znormalizowany) /MT9D111_KONTRAST_MAX;
		EepZapis(EEP_ADR_KAMERA_KONTRAST, 1, (char *)&param);

		param_znormalizowany =NASYCENIE_STANDARD;
		writeMCUMemory(AWB_SATURATION_NASYCENIE, param_znormalizowany, BRAK_MASKI_ZAPISU_DO_MEM);
		param =(KAMERA_POZIOM_NASYCENIA_MAX * param_znormalizowany) /MT9D111_NASYCENIE_MAX;
		EepZapis(EEP_ADR_KAMERA_NASYCENIE, 1, (char *)&param);

	}

    return TRUE;
}
//-----------------------------------------------------------
//odczyt ustawionego pocztkowego wspczynnika kompresji JPG
unsigned char mt9d111_odczyt_wspolczynnika_kompresji(char flaga_wpolczynnika_kompresji)
{
char maska_pozycji;	
unsigned char wspolczynnik_kompresji;
	
	maska_pozycji =1<<EEP_POZ_KAM_FLAG_KOMPRESJA_JPG;
	if ((flaga_wpolczynnika_kompresji &maska_pozycji) !=0)
	{
		wspolczynnik_kompresji =JPG_QSCALE_WSPOLCZYNNIK_36;
	}
	else
	{
		wspolczynnik_kompresji =JPG_QSCALE_WSPOLCZYNNIK_6;
	}
	return wspolczynnik_kompresji;
}
