/**
  ******************************************************************************
  * plik    Tryb_AT.c
  * funkcja	Plik procedur zwizanych z Trybem AT (serwisowym)
  * autor 	ARIES Ryszard Szymaniak www.arskam.com
  * Version: V1.01
  * data:    1-1-2013
  * opis   	ARS3-MT9D111 sterownik ARS3 obsugujcy modu kamery MT9D111 firmy APTINA
  * 		STM32F217VGT
  ******************************************************************************
  */  
#include "Tryb_AT.h"
#include "stm32f2xx_conf.h"
#include "Definicje_sprzet.h"
#include "Definicje_program.h"
#include "USART_procedury.h"
#include "Tryb_AT_etykiety.h"
#include "TIMER_procedury.h"
#include "GPIO_procedury.h"
#include "EEPROM_procedury.h"
#include "Procedury Ogolne.h"
#include "Kamera.h"
#include "MT9D111.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>


//zmienne USART1
extern char *p_BUFOR_TX_USART_GP;//extern char *p_bufor_tx_USART1;//wskanik do pocztku buforu transmisji UARTA1
extern int BUFOR_TX_USART_GP_ROZMIAR;//extern int bufor_tx_USART1_rozmiar;//bajt okrelajcy cakowity rozmiar buforu transmisji UARTA1
extern char *p_TX_USART_GP_IN;//extern char *p_tx_USART1_in;//wskanik biecej pozycji w buf.nadawczym dla danych zapisywanych w buf.
extern char *p_TX_USART_GP_OUT;//extern char *p_tx_USART1_out;//wskanik biecej pozycji w buf.nadawczym dla danych odczytywanych z buf.
extern char tx_USART1_restart_flag;//poprzednia transmisja zakoczona, nowa musi by zainicjowana

//zmienne Procedur Ogolnych
extern uint8_t	proc_ogol_stan_urzadzenia_info;

//Globalne Stae Zmienne
extern const char info_parametry_kamery_tab[];
extern const char wersja_oprogramowania_tab[];
extern const char info_parametry_kamery_tab[];


//etykiety rozkazw sterujcych AT
const char etykieta_ATO_tab[]  		={"ATO"};
const char etykieta_ATsR_tab[] 		={"AT$R"};
const char etykieta_ATRp_tab[] 		={"ATR?"};
const char znacznik_poczatku_przypisan_reg_radio_tab[] ={"reg_radio"};
const char znacznik_konca_przypisan_tab[] ={"END"};
const char etykieta_ATRm_tab[] 		={"ATR-"};
const char etykieta_ATV_tab[]  		={"AT/V"};
const char etykieta_ATS201eq_tab[]	={"ATS201="};
const char etykieta_ATS201p_tab[]	={"ATS201?"};
const char etykieta_ATS201w_tab[]	={"ATS201>"};
const char etykieta_ATsP_tab[] 		={"AT$P"};
const char etykieta_ATPp_tab[] 		={"ATP?"};
const char znacznik_poczatku_przypisan_pow_trans_rad_tab[] ={"reg_power"};
const char etykieta_POW_tab[]		={"POW_"};
const char etykieta_ATS202eq_tab[]	={"ATS202="};
const char etykieta_ATS202p_tab[]	={"ATS202?"};
const char etykieta_ATS202w_tab[]	={"ATS202>"};
const char etykieta_ATS200eq_tab[]	={"ATS200="};
const char etykieta_ATS200p_tab[]	={"ATS200?"};
const char etykieta_ATS200w_tab[]	={"ATS200>"};
const char etykieta_FREQ_tab[]		={"FREQ_"};
const char etykieta_SPAC_tab[]		={"SPAC_"};
const char etykieta_CHMAX_tab[]		={"CHMAX_"};
const char etykieta_RSPD_tab[]		={"RSPD_"};
const char etykieta_ATsC_tab[]		={"AT$C="};
const char etykieta_ATCp_tab[]		={"ATC?"};
const char etykieta_ATCw_tab[]		={"ATC>"};
const char etykieta_ATS210eq_tab[]	={"ATS210="};
const char etykieta_ATS210p_tab[]	={"ATS210?"};
const char etykieta_ATS210w_tab[]	={"ATS210>"};
const char etykieta_ATS228eq_tab[]	={"ATS228="};
const char etykieta_ATS228p_tab[]	={"ATS228?"};
const char etykieta_ATS228w_tab[]	={"ATS228>"};
const char etykieta_ATS221eq_tab[]	={"ATS221="};
const char etykieta_ATS221p_tab[]	={"ATS221?"};
const char etykieta_ATS221w_tab[]	={"ATS221>"};
const char etykieta_ATS222eq_tab[]	={"ATS222="};
const char etykieta_ATS222p_tab[]	={"ATS222?"};
const char etykieta_ATS222w_tab[]	={"ATS222>"};
const char etykieta_ATS223eq_tab[]	={"ATS223="};
const char etykieta_ATS223p_tab[]	={"ATS223?"};
const char etykieta_ATS223w_tab[]	={"ATS223>"};
const char etykieta_ATS216eq_tab[]	={"ATS216="};
const char etykieta_ATS216p_tab[]	={"ATS216?"};
const char etykieta_ATS216w_tab[]	={"ATS216>"};
const char etykieta_ATS220eq_tab[]	={"ATS220="};
const char etykieta_ATS220p_tab[]	={"ATS220?"};
const char etykieta_ATS220w_tab[]	={"ATS220>"};
const char etykieta_ATD_tab[]  		={"ATD="};
const char etykieta_ATDp_tab[]  	={"ATD?"};
const char etykieta_ATDw_tab[]  	={"ATD>"};
const char etykieta_ATDeq_tab[]  	={"ATD="};
const char etykieta_ATN_tab[]  		={"AT/N"};
const char etykieta_ATS400eq_tab[]	={"ATS400="};
const char etykieta_ATS400p_tab[]	={"ATS400?"};
const char etykieta_ATS400w_tab[]	={"ATS400>"};
const char etykieta_ATS401eq_tab[]	={"ATS401="};
const char etykieta_ATS401p_tab[]	={"ATS401?"};
const char etykieta_ATS401w_tab[]	={"ATS401>"};
const char etykieta_ATS402eq_tab[]	={"ATS402="};
const char etykieta_ATS402p_tab[]	={"ATS402?"};
const char etykieta_ATS402w_tab[]	={"ATS402>"};
const char etykieta_ATS403eq_tab[]	={"ATS403="};
const char etykieta_ATS403p_tab[]	={"ATS403?"};
const char etykieta_ATS403w_tab[]	={"ATS403>"};
const char etykieta_ATS404eq_tab[]	={"ATS404="};
const char etykieta_ATS404p_tab[]	={"ATS404?"};
const char etykieta_ATS404w_tab[]	={"ATS404>"};
const char etykieta_ATK_tab[]		={"AT/K"};
const char etykieta_ATsS230eq_tab[]	={"AT$S230="};
const char etykieta_ATS230w_tab[]	={"ATS230>"};
const char etykieta_ATsS231eq_tab[]	={"AT$S231="};
const char etykieta_ATS231w_tab[]	={"ATS231>"};
const char etykieta_ATS300eq_tab[]	={"AT$S300="};
const char etykieta_ATS300p_tab[]	={"ATS300?"};
const char etykieta_ATS300w_tab[]	={"ATS300>"};
const char etykieta_ATS300px_tab[]	={"ATS300?:"};
const char etykieta_ATS301eq_tab[]	={"AT$S301=:"};
const char etykieta_ATS301p_tab[]	={"ATS301?:"};
const char etykieta_ATS301w_tab[]	={"ATS301>:"};
const char etykieta_ATS302eq_tab[]	={"AT$S302=:"};
const char etykieta_ATS302p_tab[]	={"ATS302?:"};
const char etykieta_ATS302w_tab[]	={"ATS302>:"};
const char etykieta_ATS303eq_tab[]	={"AT$S303=;"};
const char etykieta_ATS303p_tab[]	={"ATS303?;"};
const char etykieta_ATS303w_tab[]	={"ATS303>;"};


//----------------------------
//kontrola odbioru komendy +++
char Kontrola_komendy_tryb_AT_on(char *p_bufor, uint16_t bufor_rozmiar, char *p_dane_pocz, char *p_dane_kon)
//we:	*p_bufor 		-wskanik do pocztku buforu danych	  	bufor_rozmiar	-cakowity rozmiar buforu	
//		*p_dane_pocz	-wskanik do pocztku danych w buforze	*p_dane_kon 	-wskanik do koca danych w buforze
//wy: 	status TRUE -komenda +++ inicjacja Trybu AT odebrana
{
int8_t wynik_porownan;
uint16_t ile_danych;
#define ROZMIAR_BUFORU_TEMP_CHAR_TAB	30
char *p_buf_badany, temp_char_tab[ROZMIAR_BUFORU_TEMP_CHAR_TAB];

	//obliczenie iloci danych w buforze koowym
	ile_danych =Oblicz_Ilosc_Danych_w_Buf_Kolowym(p_bufor, bufor_rozmiar, p_dane_pocz, p_dane_kon);
	//ilo danych zbyt dua jak na komend +++R(alias)
	if (ile_danych >ROZMIAR_BUFORU_TEMP_CHAR_TAB) return FALSE;
	//przepisanie danych do buf.pomocniczego
	Odczyt_z_Buf_Kolowego_do_Buf_Liniowego(p_bufor, bufor_rozmiar, p_dane_pocz, 
                                             ile_danych, &temp_char_tab[0]);
	p_buf_badany =&temp_char_tab[0];

	//sprawdzenie czy na 3 pierwszych pozycjach w buforze s znaki +++
	//i czy ilo danych w buforze =3
	wynik_porownan =strncmp(p_buf_badany, "+++", 3);
	if ((wynik_porownan ==0) && (ile_danych ==3)) return TRUE;//rozkaz +++ skierowany bezporednio do urzdzenia
    if ((wynik_porownan ==0) && (ile_danych >3)) return TRUE+ile_danych;//rozkaz +++ z dodatkowymi bajtami 
	else return FALSE;
}
//----------------------------------------------
//obliczenie iloci danych w buforze koowym
uint16_t Oblicz_Ilosc_Danych_w_Buf_Kolowym(char *p_bufor, uint16_t bufor_rozmiar, char *p_dane_pocz, char *p_dane_kon)
//we:	*p_bufor 		-wskanik do pocztku buforu danych	  	bufor_rozmiar	-cakowity rozmiar buforu	
//		*p_dane_pocz	-wskanik do pocztku danych w buforze	*p_dane_kon 	-wskanik do koca danych w buforze
//wy:	ilo bajtw danych w buforze
{
uint16_t ile_danych;

	if (p_dane_pocz <=p_dane_kon)
	{
		ile_danych =p_dane_kon - p_dane_pocz;
	} 
	else
	{
		//blok aktualnych danych jest zapisany na kocu i pocztku obszaru bufora tzn. w trakcie
		//zapisu bloku danych wskanik pozycji zapisu przewin si 
	    ile_danych =((p_bufor+bufor_rozmiar) -p_dane_pocz) + (p_dane_kon - p_bufor);
	}
	return ile_danych;
}
//---------------------------------------
//przepisanie wskazanej iloci danych z buforu koowego do innego buforu liniowego i zwrcenie 
//nowej pozycji wskanika danych odczytywanych z bufora koowgo
char *Odczyt_z_Buf_Kolowego_do_Buf_Liniowego(char *p_bufor, uint16_t bufor_rozmiar, char *p_dane_pocz, 
                                             uint16_t ile_do_odczytu, char *p_buf_liniowy)
//we:	*p_bufor 		-wskanik do pocztku buforu danych	  	bufor_rozmiar	-cakowity rozmiar buforu	
//		*p_dane_pocz	-wskanik do pocztku danych w buforze	ile_do_odczytu 	-licznik iloci danych do odczytu
//		*p_buf_liniowy	-wskanik do bufora liniowego
//wy:	wskanik nowej pozycji danych do odczytu w buforze koowym po przesuniciu zwizanym z odczytem
{
uint16_t x;

	for (x=0; x<ile_do_odczytu; x++)
	{
		*(p_buf_liniowy+x) =*p_dane_pocz;
		p_dane_pocz++;
		if (p_dane_pocz >=(p_bufor+bufor_rozmiar)) p_dane_pocz =p_bufor;//pocztek buf.koowego
	}
	return p_dane_pocz;
}
//--------------------------------------
//obsuga Trybu AT (Serwis)
void Obsluga_Trybu_AT(char *p_bufor, uint16_t bufor_rozmiar, char *p_dane_pocz, char *p_dane_kon)
//we:	*p_bufor 		-wskanik do pocztku buforu danych	  	bufor_rozmiar	-cakowity rozmiar buforu	
//		*p_dane_pocz	-wskanik do pocztku danych w buforze	*p_dane_kon 	-wskanik do koca danych w buforze
{
#define ROZMIAR_BUFORU_SERWISOWEGO	 500
char buf_serwis_tmp[ROZMIAR_BUFORU_SERWISOWEGO];
char rozmiar_etykiety_AT;
//char nr_linii;
const char *p_etykieta_AT_tab;
uint16_t ile_danych, x;

		ile_danych =Oblicz_Ilosc_Danych_w_Buf_Kolowym(p_bufor, bufor_rozmiar, p_dane_pocz, p_dane_kon);
		if (ile_danych ==0) return;
		if (ile_danych >ROZMIAR_BUFORU_SERWISOWEGO)
		{
			//dziwny bd
			while (1);
		}
		Odczyt_z_Buf_Kolowego_do_Buf_Liniowego(p_bufor, bufor_rozmiar, p_dane_pocz, 
                                             ile_danych, &buf_serwis_tmp[0]);
		//identyfikacja etykiety odebranego rozkazu
		x=0;
		while(x<ILOSC_ROZKAZOW_AT)
		{
			p_etykieta_AT_tab =Odczyt_Stringu_Rozkazu(x);
			rozmiar_etykiety_AT =strlen(p_etykieta_AT_tab);

			if (rozmiar_etykiety_AT !=0)
			{
				if (memcmp(&buf_serwis_tmp[0], p_etykieta_AT_tab, 
			      	rozmiar_etykiety_AT) ==0)
				{
					//rozkaz zosta zidentyfikowany
					break;
				}
			}
			x++;
		}
		if (x !=ILOSC_ROZKAZOW_AT)
		{
			//obsuga zidentyfikowanego rozkazu AT
			Obsluga_Rozkazu_AT(&buf_serwis_tmp[0], x);
		}
		else
		{
			//rozkaz nie zosta zidentyfikowany
			Tryb_AT_komunikat_ERROR(BLAD_ROZKAZ_NIE_ROZPOZNANY);
			TIMER4_Detekcja_przerwy_trans_start(CZAS_AUTOMAT_ZAMKNIECIA_TRYBU_AT_MS);
			return;
		}
		TIMER4_Detekcja_przerwy_trans_start(CZAS_AUTOMAT_ZAMKNIECIA_TRYBU_AT_MS);
}
//----------------------------------------------
//odczyt kolejnego stringu rozpoznawanego przez system rozkazu
const char *Odczyt_Stringu_Rozkazu(uint16_t numer_rozkazu)
//we: numer_rozkazu -numer badanego rozkazu
//wy: *p_etykieta_AT_tab -zwracany wskanik do etykiety rozkazu, 
{
//char rozmiar_etykiety_AT;
const char *p_etykieta_AT_tab;

			p_etykieta_AT_tab =NULL;

			switch (numer_rozkazu)
			{
				case ETYKIETA_ATO:
				{
					p_etykieta_AT_tab =&etykieta_ATO_tab[0];
					break;
				}
				case ETYKIETA_ATsC:
				{
					p_etykieta_AT_tab =&etykieta_ATsC_tab[0];
					break;
				}
				case ETYKIETA_ATCp:
				{
					p_etykieta_AT_tab =&etykieta_ATCp_tab[0];
					break;
				}
				case ETYKIETA_ATCw:
				{
					p_etykieta_AT_tab =&etykieta_ATCw_tab[0];
					break;
				}
				case ETYKIETA_ATDeq:
				{
					//rozmiar_etykiety_AT =(sizeof(etykieta_ATD_tab)-1);
					p_etykieta_AT_tab =&etykieta_ATD_tab[0];
					break;
				}
				case ETYKIETA_ATDw:
				{
					p_etykieta_AT_tab =&etykieta_ATDw_tab[0];
					break;
				}
				case ETYKIETA_ATDp:
				{
					p_etykieta_AT_tab =&etykieta_ATDp_tab[0];
					break;
				}
				case ETYKIETA_ATK:
				{
					p_etykieta_AT_tab =&etykieta_ATK_tab[0];
					break;
				}
				case ETYKIETA_ATN:
				{
					p_etykieta_AT_tab =&etykieta_ATN_tab[0];
					break;
				}
				case ETYKIETA_ATV:
				{
					p_etykieta_AT_tab =&etykieta_ATV_tab[0];
					break;
				}
				case ETYKIETA_ATS210eq:
				{
					p_etykieta_AT_tab =&etykieta_ATS210eq_tab[0];
					break;
				}
				case ETYKIETA_ATS210p:
				{
					p_etykieta_AT_tab =&etykieta_ATS210p_tab[0];
					break;
				}
				case ETYKIETA_ATS210w:
				{
					p_etykieta_AT_tab =&etykieta_ATS210w_tab[0];
					break;
				}
				case ETYKIETA_ATS400eq:
				{
					p_etykieta_AT_tab =&etykieta_ATS400eq_tab[0];
					break;
				}
				case ETYKIETA_ATS400p:
				{
					p_etykieta_AT_tab =&etykieta_ATS400p_tab[0];
					break;
				}
				case ETYKIETA_ATS400w:
				{
					p_etykieta_AT_tab =&etykieta_ATS400w_tab[0];
					break;
				}
				case ETYKIETA_ATS401eq:
				{
					p_etykieta_AT_tab =&etykieta_ATS401eq_tab[0];
					break;
				}
				case ETYKIETA_ATS401p:
				{
					p_etykieta_AT_tab =&etykieta_ATS401p_tab[0];
					break;
				}
				case ETYKIETA_ATS401w:
				{
					p_etykieta_AT_tab =&etykieta_ATS401w_tab[0];
					break;
				}
				case ETYKIETA_ATS402eq:
				{
					p_etykieta_AT_tab =&etykieta_ATS402eq_tab[0];
					break;
				}
				case ETYKIETA_ATS402p:
				{
					p_etykieta_AT_tab =&etykieta_ATS402p_tab[0];
					break;
				}
				case ETYKIETA_ATS402w:
				{
					p_etykieta_AT_tab =&etykieta_ATS402w_tab[0];
					break;
				}
				case ETYKIETA_ATS404eq:
				{
					p_etykieta_AT_tab =&etykieta_ATS404eq_tab[0];
					break;
				}
				case ETYKIETA_ATS404p:
				{
					p_etykieta_AT_tab =&etykieta_ATS404p_tab[0];
					break;
				}
				case ETYKIETA_ATS404w:
				{
					p_etykieta_AT_tab =&etykieta_ATS404w_tab[0];
					break;
				}

				default: 
				{
					p_etykieta_AT_tab =NULL;
					break;	
				}
			}
			return p_etykieta_AT_tab;
}
//----------------------------------------------
//w Trybie AT (Serwisowym) wysyanie potwierdzenie OK
void Tryb_AT_potwierdzenie_OK(void)
{
		
	strcpy(p_BUFOR_TX_USART_GP, "OK\r");
	//p_BUFOR_TX_USART_GP =&BUFOR_TX_USART_GP[0];
	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +3;
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	Selektor_Start_Tx();  
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wys-anie odpowiedzi 
}
//----------------------------------------------
//w Trybie Serwisowym wysyanie komunikatu bdu
void Tryb_AT_komunikat_ERROR(char kod_bledu)
{

	snprintf(p_BUFOR_TX_USART_GP, ROZMIAR_BUF_RX_TX, "ERROR%d\r", kod_bledu);
	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +strlen(p_BUFOR_TX_USART_GP);
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wys-anie odpowiedzi 
} 
//---------------------------------------------
//obsuga rozkazu ATO -koniec trybu serwisowego
void Obsluga_rozkazu_ATO(void)
{
	GPIO_Linie_IO_High(LINIA_LED_ZIELONY_WY);
	GPIO_Linie_IO_High(LINIA_LED_CZERWONY_WY);
	Tryb_AT_potwierdzenie_OK();
	proc_ogol_stan_urzadzenia_info =STAN_URZADZENIA_RX_TX;
}
//----------------------------------------------------------------
//obsuga rozkazu AT$C=	-zapis acucha z nowym aliasem urzdzenia
void Obsluga_rozkazu_ATsC(char *p_buf)
{
char rozmiar_str_alias, status;
char *p_tmp;

	rozmiar_str_alias =sizeof(etykieta_ATsC_tab)-1;
	p_buf =p_buf +rozmiar_str_alias;
	//usunicie kocowych znakw "\r" 
	p_tmp =strstr(p_buf, "\r");
	if (p_tmp !=NULL) *p_tmp ='\0';
	rozmiar_str_alias =strlen(p_buf);
	if (rozmiar_str_alias >=ROZMIAR_ALIASU_MAX)
	{
		//bd, przesany string aliasu ma rozmiar wikszy od dopuszczalnego
		Tryb_AT_komunikat_ERROR(BLAD_ZAPISU_DO_PAMIECI_KONFIG);
		return;
	}
	//zapis stringu ze znakiem jego zakoczenia '\0'
	status =EepZapis(EEP_ADR_ALIAS, 
			rozmiar_str_alias+1, p_buf);
	if (status ==0)
	{
		Tryb_AT_komunikat_ERROR(BLAD_ZAPISU_DO_PAMIECI_KONFIG);
	} 
	else Tryb_AT_potwierdzenie_OK();
}
//--------------------------------------------------------------
//obsuga rozkazu ATC?	-odczyt maksymalnego rozmiaru aliasu
void Obsluga_rozkazu_ATCp(void)
{
int ile_znakow_wysylanych;

    ile_znakow_wysylanych =snprintf(p_BUFOR_TX_USART_GP,ROZMIAR_BUF_RX_TX, "%s%d max_rozmiar_aliasu\r", "ATC=", ROZMIAR_ALIASU_MAX);  
	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wysanie odpowiedzi 

}
//----------------------------------------------------------
//obsuga rozkazu ATC>	-odczyt stringu z aliasem urzdzenia
void Obsluga_rozkazu_ATCw(void)
{
char temp_char_tab[ROZMIAR_ALIASU_MAX+3], status;
char rozmiar_aliasu;
int ile_znakow_wysylanych;

	memset(&temp_char_tab[0], '\0', ROZMIAR_ALIASU_MAX+3);
	status =EepOdczyt(EEP_ADR_ALIAS, 
			ROZMIAR_ALIASU_MAX, &temp_char_tab[0]);
	if (status ==0)
	{
		Tryb_AT_komunikat_ERROR(BLAD_ODCZYTU_Z_PAMIECI_KONFIG);
		return;
	}
	rozmiar_aliasu =strlen(&temp_char_tab[0]);
	if (rozmiar_aliasu >ROZMIAR_ALIASU_MAX)
	{
	 	rozmiar_aliasu =ROZMIAR_ALIASU_MAX;
	}
	strcat(&temp_char_tab[rozmiar_aliasu], "\0");
	ile_znakow_wysylanych =snprintf(p_BUFOR_TX_USART_GP, ROZMIAR_ALIASU_MAX+3,"AT$C<%s\r", &temp_char_tab[0]);
	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wysanie odpowiedzi 

}
//---------------------------------------------------------------------------------
//obsuga rozkazu ATD=	-przywrcenie domylnych ustawie rejestrw
void Obsluga_rozkazu_ATDeq(void)
{
char symulacja_rozkazu_tab[60];
int ile_znakow_wysylanych;

	snprintf(&symulacja_rozkazu_tab[0], 60, "%sARS3-K1\r", etykieta_ATsC_tab); 
	Obsluga_Rozkazu_AT(symulacja_rozkazu_tab, ETYKIETA_ATsC);
	Delay_ms(400);

    snprintf(&symulacja_rozkazu_tab[0], 30, "%s1000\r", etykieta_ATS222eq_tab);//czas oczekiwania na potwierdzenie;
	Obsluga_Rozkazu_AT(symulacja_rozkazu_tab, ETYKIETA_ATS222eq);
	Delay_ms(400);
    snprintf(&symulacja_rozkazu_tab[0], 30, "%s3\r", etykieta_ATS223eq_tab);//ilo powtrze transmisji przy braku potwierdzenia
	Obsluga_Rozkazu_AT(symulacja_rozkazu_tab, ETYKIETA_ATS223eq);
	Delay_ms(400);
    snprintf(&symulacja_rozkazu_tab[0], 30, "%s%d\r", etykieta_ATS220eq_tab, TRYB_TRANS_RADIO_FORMATOWANY_Z_POTWIERDZENIEM);//tryb pracy urzdzenia
	Obsluga_Rozkazu_AT(symulacja_rozkazu_tab, ETYKIETA_ATS220eq);
	Delay_ms(400);
    snprintf(&symulacja_rozkazu_tab[0], 30, "%s1\r", etykieta_ATS200eq_tab);//kana radiowy =1
	Obsluga_Rozkazu_AT(symulacja_rozkazu_tab, ETYKIETA_ATS200eq);
	Delay_ms(400);
    snprintf(&symulacja_rozkazu_tab[0], 30, "%s1\r", etykieta_ATS202eq_tab);//prg mocy transmisji radiowej z dostpnych =1
	Obsluga_Rozkazu_AT(symulacja_rozkazu_tab, ETYKIETA_ATS202eq);
	Delay_ms(400);

	//wysanie potwierdzenia zakoczenia ustawiania wartoci domylnych rejestrw
	strcpy(POCZ_BUFOR_TX_USART_GP, "END\r");
	ile_znakow_wysylanych =4;

	p_BUFOR_TX_USART_GP =POCZ_BUFOR_TX_USART_GP;
	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	Selektor_Start_Tx();//UART_Start_Tx2();
	while (Test_tx_USART_restart_flag() ==0);//oczekiwanie na wysanie odpowiedzi
}
//---------------------------------------------------------------------------------
//obsuga rozkazu ATD>	-odczyt aktualnych ustawie rejestrw
void Obsluga_rozkazu_ATDw(void)
{
uint16_t x;
const char *p_etykieta_AT_tab;
char *poz_znaku/*, nr_zrodla, poz_zrodla, rozkaz_tab[30]*/;

	//kolejny odczyt etykiet wszystkich obsugiwanych rozkazw
	x=0;
	while(x<ILOSC_ROZKAZOW_AT)
	{
		p_etykieta_AT_tab =Odczyt_Stringu_Rozkazu(x);
		if (p_etykieta_AT_tab ==NULL)
		{
			x++;
			continue;
		}
		if (strstr(p_etykieta_AT_tab, etykieta_ATDw_tab) !=NULL)
		{
			//ominicie rekurencyjnego wywoania rozkazu przez samego siebie
			x++;
			continue;
		}
		//jeeli string zawiera znak '>' oznacza to e przesya ustawienia rejestru
		poz_znaku =strchr(p_etykieta_AT_tab, '>');
		if (poz_znaku !=NULL)
		{
			Obsluga_Rozkazu_AT(NULL, x);
			Delay_ms(100);
		}
		x++;
	}						
	Tryb_AT_potwierdzenie_OK();
}
//---------------------------------------------------------------------------------
//obsuga rozkazu ATD?	-odczyt zestawienia dostpnych rejestrw i zakresw ich parametrw
void Obsluga_rozkazu_ATDp(char *p_buf_rozkazu)
{
uint16_t x;
const char *p_etykieta_AT_tab;
char *poz_znaku;
char poz_str;


	poz_str =(sizeof(etykieta_ATDp_tab)-1);
	if (*(p_buf_rozkazu+poz_str) ==':')	
	{
		//obsuga rozkazu ATD w trybie numerowania zapyta o kolejne obsugiwane rozkazy
		Obsluga_rozkazu_ATDp_x(p_buf_rozkazu);
		return;
	}
	//obsuga rozkazu ATD w trybie jednorazowego kolejnego przesyania info o obsugiwanych rozkazach
	//kolejny odczyt etykiet wszystkich obsugiwanych rozkazw
	x=0;
	while(x<ILOSC_ROZKAZOW_AT)
	{
		p_etykieta_AT_tab =Odczyt_Stringu_Rozkazu(x);
		if (p_etykieta_AT_tab ==NULL)
		{
			x++;
			continue;
		}
		if (strstr(p_etykieta_AT_tab, etykieta_ATDp_tab) !=NULL)
		{
			//ominicie rekurencyjnego wywoania rozkazu przez samego siebie
			x++;
			continue;
		}
		//jeeli string zawiera znak '?' oznacza to e przesya opis rejestru i jego parametrw
		poz_znaku =strchr(p_etykieta_AT_tab, '?');
		if (poz_znaku !=NULL)
		{
			//jeeli w stringu znajduje si znak ':' oznacza to, e w rozkazie ma znale si indeks rda
			Obsluga_Rozkazu_AT(NULL, x);
			Delay_ms(100);
		}
		x++;
	}						
	Tryb_AT_potwierdzenie_OK();
}
//---------------------------------------------------------------------------------
//obsuga rozkazu ATD?:x	-przesyanie informacje o obsugiwanych rejestrach w trybie numerowanych zapyta 
void Obsluga_rozkazu_ATDp_x(char *p_buf_rozkazu)
{
char poz_str;
int num_na_liscie_poszukiwanego_rej, kolejny_num_rozkazu_p, num_rozkazu;
const char *p_etykieta_AT_tab;
#define ROZMIAR_ETYKIETY_MAX	12
char nazwa_rozkazu_tab[20], status, rodzina_etykiet_tab[4*3*12], *p_poz_str;
int ile_znakow_wysylanych;


	poz_str =(sizeof(etykieta_ATDp_tab)-1);
	poz_str++;//ominicie znaku ':' poprzedzajcego indeks listy kolejnego obsugiwanego rejestru
	num_na_liscie_poszukiwanego_rej =atoi(p_buf_rozkazu+poz_str);

    nazwa_rozkazu_tab[0] ='\0';
	num_rozkazu =0;
	kolejny_num_rozkazu_p =0;
	while (num_rozkazu <=ILOSC_ROZKAZOW_AT) 
	{
		p_etykieta_AT_tab =Odczyt_Stringu_Rozkazu(num_rozkazu);
		num_rozkazu++;
		if (p_etykieta_AT_tab ==NULL) //etykieta nie odnaleziona, rozkaz nie obsugiwany
		{
			if (num_rozkazu <ILOSC_ROZKAZOW_AT) continue;
			else 					//brak kolejnych etykiet do przesania
			{
				kolejny_num_rozkazu_p =-1;
				break;
			}
		}
		//kopiowanie nazwy kolejnej etykiety do buforu grupowego rnych wariantw tego samego rozkazu
		strcpy(&rodzina_etykiet_tab[0], p_etykieta_AT_tab);
		//ekstrakcja z etykiety nazwy roakazu (bez pocztkowych znakw "ATS" lub "AT$S" i kocowych "=...",">...","?...","-..."
		status =Obsluga_rozkazu_ATDp_x_ekstrakcja_nazwy((char *)p_etykieta_AT_tab, &nazwa_rozkazu_tab[0]);
		if (status ==FALSE) continue;//dziwny bd, brak w etykiecie rozkazu prefiksu "AT"
		//jeeli jako pierwsza z grupy odnaleziona zostaa etykieta indeksowana
		//usunicie jej wpisu z buforu rodziny, dekrementacja licznika num_rozkazu
		//i obsuga wszystkich etykiet indeksowanych w ramach wsplnej ptli
		if (strchr(p_etykieta_AT_tab, ':') !=NULL)
		{
			rodzina_etykiet_tab[0] ='\0';
			num_rozkazu--;
		}
		//poszukiwanie kolejnych etykiet z rodziny
		do
		{
			p_etykieta_AT_tab =Odczyt_Stringu_Rozkazu(num_rozkazu);
			p_poz_str =strstr(p_etykieta_AT_tab,  &nazwa_rozkazu_tab[0]);
			if (p_poz_str !=NULL)
			{
				{
					//etykieta bez indeksu
					strcat(&rodzina_etykiet_tab[0], " ");
					strcat(&rodzina_etykiet_tab[0], p_etykieta_AT_tab);
				}
				num_rozkazu++;
			}	
		}
		while ((p_poz_str !=NULL) && (num_rozkazu <ILOSC_ROZKAZOW_AT));
		if (kolejny_num_rozkazu_p ==num_na_liscie_poszukiwanego_rej)
		{
			break;//odszukana etykieta kolejnego rozkazu do przesania w odpowiedzi na ATD?:x
		}
		kolejny_num_rozkazu_p++;
	}
	//Delay_ms(100);
	//kolejna etykieta zapytania odpowiadajca kolejnemu obsugiwanemu rozkazowi nie zostaa odszukana
	if (kolejny_num_rozkazu_p !=num_na_liscie_poszukiwanego_rej)
	{
		Tryb_AT_komunikat_ERROR(BLAD_ROZKAZ_NIE_ROZPOZNANY);
		return;
	}
	//potwierdzenie z etykiet kolejnego obsugiwanego rozkazu
	//ile_znakow_wysylanych =snprintf(POCZ_BUFOR_TX_USART_GP, ROZMIAR_BUF_RX_TX, "AT$D:%i=%s\r",num_na_liscie_poszukiwanego_rej, p_etykieta_AT_tab);
	ile_znakow_wysylanych =snprintf(POCZ_BUFOR_TX_USART_GP, ROZMIAR_BUF_RX_TX, "AT$D:%i=%s\r",num_na_liscie_poszukiwanego_rej, &rodzina_etykiet_tab[0]);
	p_BUFOR_TX_USART_GP =POCZ_BUFOR_TX_USART_GP;
	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;  
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	Selektor_Start_Tx();
	while (Test_tx_USART_restart_flag() ==0);//oczekiwanie na wysanie odpowiedzi

}
//---------------------------------------------------------------------------------
//obsuga rozkazu ATD?:x	-ekstracja z etykiety rejestru nazwy rejestru
char Obsluga_rozkazu_ATDp_x_ekstrakcja_nazwy(char *p_etykieta_AT_tab, char *p_nazwa_rozkazu_tab)
//we: 	p_etykieta_AT_tab -wskanik do buforu etykiety rozkazu
//		p_nazwa_rozkazu_tab -wskanik do buforu nazwy rozkazu
//wy:	TRUE -w buforze jest wyekstrachowana z etykiety nazwa rejestru
//		FALSE -bd ekstrakcji		
{
char *poz_prefiks_sufiks;

	//ekstrakcja z etykiety nazwy roakazu (bez pocztkowych znakw "ATS" lub "AT$S" i kocowych "=...",">...","?...","-..."
	poz_prefiks_sufiks =strstr(p_etykieta_AT_tab, "AT");
	if (poz_prefiks_sufiks !=NULL)
	{
		strcpy(p_nazwa_rozkazu_tab,  p_etykieta_AT_tab+2);//usunicie z etykiety prefiksu "AT"
	}
	else return FALSE;//dziwny bd, brak w etykiecie rozkazu prefiksu "AT"

	if (*p_nazwa_rozkazu_tab =='$')
	{
		//usunicie z etykiety prefiksu '$'
		strcpy(p_nazwa_rozkazu_tab, (p_nazwa_rozkazu_tab+1));
	}
	if (*p_nazwa_rozkazu_tab =='S')
	{
		//usunicie z etykiety prefiksu 'S'
		strcpy(p_nazwa_rozkazu_tab, (p_nazwa_rozkazu_tab+1));
	}
	poz_prefiks_sufiks =strpbrk(p_nazwa_rozkazu_tab, "=>?-");
	if (poz_prefiks_sufiks !=NULL)
	{
		//usunicie z etykiety sufiksu rozpoczynajcego si ktrym ze znakw: "=>?-"
		*poz_prefiks_sufiks ='\0';
	}
	return TRUE;
}

//-------------------------------------------
//obsuga rozkazu AT/K -informacja o zinstalowanej kamerze
void Obsluga_rozkazu_ATK(void)
{
int ile_znakow_wysylanych;

	ile_znakow_wysylanych =snprintf(p_BUFOR_TX_USART_GP, 100, "AT$K=%s\r", &info_parametry_kamery_tab[0]);
	BUFOR_TX_USART_GP_ROZMIAR =100;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wys-anie odpowiedzi 
}
//---------------------------------------------------------
//obsuga rozkazu AT/N odczyt unikatowego numeru urzdzenie
void Obsluga_rozkazu_ATN(void)
{
#define U_ID_BASE	0x1FFF7A10
#define U_ID_NUM_REG	96 / 8

uint8_t u_id_tab[U_ID_NUM_REG];
uint8_t *p_tab;
char x;
int ile_znakow_wysylanych;

	p_tab =0;
	p_tab =p_tab +U_ID_BASE;
	for (x=0; x<U_ID_NUM_REG; x++)
	{
		u_id_tab[x] = __LDREXB (p_tab+x);
	}
	ile_znakow_wysylanych =snprintf(p_BUFOR_TX_USART_GP, 45, "AT/N<%02X%02X-%02X%02X-%02X%02X-%02X%02X-%02X%02X-%02X%02X\r",
	                           u_id_tab[0], u_id_tab[1], u_id_tab[2],
							   u_id_tab[3], u_id_tab[4], u_id_tab[5],
							   u_id_tab[6], u_id_tab[7], u_id_tab[8],
							   u_id_tab[9], u_id_tab[10], u_id_tab[11]);
	//p_BUFOR_TX_USART_GP =&BUFOR_TX_USART_GP[0];
	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wys-anie odpowiedzi 
}
//-------------------------------------------
//obsuga rozkazu AT/V -wersja oprogramowania
void Obsluga_rozkazu_ATV(void)
{
int ile_znakow_wysylanych;

	ile_znakow_wysylanych =snprintf(p_BUFOR_TX_USART_GP, ROZMIAR_BUF_RX_TX, "AT/V<%s\r", &wersja_oprogramowania_tab[0]);
	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;  
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	Selektor_Start_Tx();
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wys-anie odpowiedzi 

}
//--------------------------------------------------------
//obsuga rozkazu ATS210=	-wybr numeru szybkoci UART-a
void Obsluga_rozkazu_ATS210eq(char *p_buf)
{
char poz_str_cyfry, status, nr_szybkosci_char;
int nr_szybkosci;

	poz_str_cyfry =(sizeof(etykieta_ATS210eq_tab)-1);
	nr_szybkosci =atoi(p_buf+poz_str_cyfry);
    nr_szybkosci_char =(char)(nr_szybkosci);

	if (nr_szybkosci_char >ILOSC_SZYBKOSCI_TRANSMISJI_UART)
	{
		//bd proba wyboru niedostpnego numeru szybkoci transmisji
		Tryb_AT_komunikat_ERROR(BLAD_PRZESLANEGO_PARAMETRU);
		return;
	} 
	nr_szybkosci_char =(nr_szybkosci_char-1);//przesany numer liczony od 1 a w EEP od 0 

	status =EepZapis(EEP_ADR_NUMER_SZYBKOSCI_UART, 1, &nr_szybkosci_char);
	if (status ==0)
	{
		Tryb_AT_komunikat_ERROR(BLAD_ZAPISU_DO_PAMIECI_KONFIG);
	} 
	else Tryb_AT_potwierdzenie_OK();
	Delay_ms(100);

	//reset urzdzenia
	NVIC_SystemReset();
}
//----------------------------------------------------------
//obsuga rozkazu ATS210?	-odczyt dostpnych szybkoci UART-a
void Obsluga_rozkazu_ATS210p(void)
{
char x;
char konwert_tab[13];
int ile_znakow_wysylanych;
extern const int dostepne_szybkosci_uarta_tab[];

	//kopiowanie etykiety odpowiedzi rozkazu AT$S210=
	snprintf(p_BUFOR_TX_USART_GP, 100, "AT$S210=");

	for (x=0; x<ILOSC_SZYBKOSCI_TRANSMISJI_UART; x++)
	{
		snprintf(&konwert_tab[0], 13, "(%d)%d ", x+1, dostepne_szybkosci_uarta_tab[x]);
		strcat(p_BUFOR_TX_USART_GP, &konwert_tab[0]);
	}
	strcat(p_BUFOR_TX_USART_GP, "\r");
	ile_znakow_wysylanych =strlen(p_BUFOR_TX_USART_GP);

	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;  
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	Selektor_Start_Tx();
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wys-anie odpowiedzi 
}
//----------------------------------------------------------
//obsuga rozkazu ATS210>	-numer wybranej szybkoci UART-a
void Obsluga_rozkazu_ATS210w(void)
{
char status, nr_szybkosci_char;
int ile_znakow_wysylanych;
extern uint32_t uart_speed;
extern const int dostepne_szybkosci_uarta_tab[];

	status =EepOdczyt(EEP_ADR_NUMER_SZYBKOSCI_UART, 
			1, &nr_szybkosci_char);
	if (status ==0)
	{
		Tryb_AT_komunikat_ERROR(BLAD_ODCZYTU_Z_PAMIECI_KONFIG);
		return;
	} 
	//w sytuacji gdy odczytany numer szybkoci UART poza dopuszczalnym zakresem(np. sytuacja gdy nowy EEPROM =0xFF)
	//odsyanie jako ustawionej tej szybkoci UART po jakiej nawizano poczenie
    if ((nr_szybkosci_char+1) >ILOSC_SZYBKOSCI_TRANSMISJI_UART)
	{
		nr_szybkosci_char =0;
		while ((nr_szybkosci_char+1) <ILOSC_SZYBKOSCI_TRANSMISJI_UART)
		{
			if (uart_speed ==dostepne_szybkosci_uarta_tab[nr_szybkosci_char]) break;
			nr_szybkosci_char++;
		}
	}

	snprintf(p_BUFOR_TX_USART_GP, ROZMIAR_BUF_RX_TX, "ATS210< %d\r", (nr_szybkosci_char+1));
	ile_znakow_wysylanych =strlen(p_BUFOR_TX_USART_GP);

	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;  
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	Selektor_Start_Tx();
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wys-anie odpowiedzi 
}
//---------------------------------------------------------------------------------
//obsuga rozkazu ATS400=	-kamera ustawianie poziomu jaskrawoci
void Obsluga_rozkazu_ATS400eq(char *p_buf)
{
char poz_str_cyfry, poziom_jaskrawosci_char, status;
int poziom_jaskrawosci_int;

	poz_str_cyfry =(sizeof(etykieta_ATS400eq_tab)-1);
	poziom_jaskrawosci_int =atoi(p_buf+poz_str_cyfry);
	if ((poziom_jaskrawosci_int <KAMERA_POZIOM_JASKRAWOSCI_MIN) || (poziom_jaskrawosci_int >KAMERA_POZIOM_JASKRAWOSCI_MAX))
	{
		Tryb_AT_komunikat_ERROR(BLAD_PRZESLANEGO_PARAMETRU);
		return;
	}
	poziom_jaskrawosci_char =(char)poziom_jaskrawosci_int;
		status =Kamera_Jaskrawosc_regulacja(poziom_jaskrawosci_char);

		if (status ==TRUE) Tryb_AT_potwierdzenie_OK();
		else Tryb_AT_komunikat_ERROR(BLAD_PROGRAMOWANIA_UKLADU);
}
//---------------------------------------------------------------------------------
//obsuga rozkazu ATS400?	-kamera odczyt zakresu parametru ustawiania poziomu jaskrawoci
void Obsluga_rozkazu_ATS400p(char *p_buf)
{
int ile_znakow_wysylanych;

	ile_znakow_wysylanych =snprintf(p_BUFOR_TX_USART_GP, ROZMIAR_BUF_RX_TX, "AT$S400=%d-%d jaskrawo\r", 
	    KAMERA_POZIOM_JASKRAWOSCI_MIN, KAMERA_POZIOM_JASKRAWOSCI_MAX);

	ile_znakow_wysylanych =strlen(p_BUFOR_TX_USART_GP);

	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;  
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	Selektor_Start_Tx();
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wys-anie odpowiedzi 
}
//---------------------------------------------------------------------------------
//obsuga rozkazu ATS400>	-kamera odczyt parametru ustawionego poziomu jaskrawoci
void Obsluga_rozkazu_ATS400w(char *p_buf)
{
char status, poziom_jaskrawosci;
unsigned int ile_znakow_wysylanych;

	status =EepOdczyt(EEP_ADR_KAMERA_JASKRAWOSC, 
			1, &poziom_jaskrawosci);
	if (status ==0)
	{
		Tryb_AT_komunikat_ERROR(BLAD_ODCZYTU_Z_PAMIECI_KONFIG);
		return;
	} 
	if (((signed char)poziom_jaskrawosci <KAMERA_POZIOM_JASKRAWOSCI_MIN) || (poziom_jaskrawosci >KAMERA_POZIOM_JASKRAWOSCI_MAX))
	{
		poziom_jaskrawosci =KAMERA_POZIOM_JASKRAWOSCI_MAX /2;
	}
	ile_znakow_wysylanych =snprintf(p_BUFOR_TX_USART_GP, ROZMIAR_BUF_RX_TX, "%s%d\r", "ATS400< ", poziom_jaskrawosci);

	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;  
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	Selektor_Start_Tx();
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wys-anie odpowiedzi 
}
//---------------------------------------------------------------------------------
//obsuga rozkazu ATS401=	-kamera ustawianie poziomu kontrastu
void Obsluga_rozkazu_ATS401eq(char *p_buf)
{
char poz_str_cyfry, poziom_kontrastu_char, status;
int poziom_kontrastu_int;

	poz_str_cyfry =(sizeof(etykieta_ATS401eq_tab)-1);
	poziom_kontrastu_int =atoi(p_buf+poz_str_cyfry);
	if ((poziom_kontrastu_int <KAMERA_POZIOM_KONTRASTU_MIN) || (poziom_kontrastu_int >KAMERA_POZIOM_KONTRASTU_MAX))
	{
		Tryb_AT_komunikat_ERROR(BLAD_PRZESLANEGO_PARAMETRU);
		return;
	}
	poziom_kontrastu_char =(char)poziom_kontrastu_int;
		status =Kamera_Kontrast_regulacja(poziom_kontrastu_char);

		if (status ==TRUE) Tryb_AT_potwierdzenie_OK();
		else Tryb_AT_komunikat_ERROR(BLAD_PROGRAMOWANIA_UKLADU);
}
//---------------------------------------------------------------------------------
//obsuga rozkazu ATS401?	-kamera odczyt zakresu parametru ustawiania poziomu kontrastu
void Obsluga_rozkazu_ATS401p(char *p_buf)
{
int ile_znakow_wysylanych;

	ile_znakow_wysylanych =snprintf(p_BUFOR_TX_USART_GP, ROZMIAR_BUF_RX_TX, "AT$S401=%d-%d kontrast\r", 
	    KAMERA_POZIOM_KONTRASTU_MIN, KAMERA_POZIOM_KONTRASTU_MAX);

	ile_znakow_wysylanych =strlen(p_BUFOR_TX_USART_GP);

	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;  
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	Selektor_Start_Tx();
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wys-anie odpowiedzi 
}
//---------------------------------------------------------------------------------
//obsuga rozkazu ATS401>	-kamera odczyt parametru ustawionego poziomu kontrastu
void Obsluga_rozkazu_ATS401w(char *p_buf)
{
char status, poziom_kontrastu;
unsigned int ile_znakow_wysylanych;

	status =EepOdczyt(EEP_ADR_KAMERA_KONTRAST, 
			1, &poziom_kontrastu);
	if (status ==0)
	{
		Tryb_AT_komunikat_ERROR(BLAD_ODCZYTU_Z_PAMIECI_KONFIG);
		return;
	} 
	if (((signed char)poziom_kontrastu <KAMERA_POZIOM_KONTRASTU_MIN) || (poziom_kontrastu >KAMERA_POZIOM_KONTRASTU_MAX))
	{
		poziom_kontrastu =KAMERA_POZIOM_KONTRASTU_MAX /2;
	}
	ile_znakow_wysylanych =snprintf(p_BUFOR_TX_USART_GP, ROZMIAR_BUF_RX_TX, "%s%d\r", "ATS401< ", poziom_kontrastu);

	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;  
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	Selektor_Start_Tx();
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wys-anie odpowiedzi 
}
//---------------------------------------------------------------------------------
//obsuga rozkazu ATS402=	-kamera ustawianie poziomu nasycenia
void Obsluga_rozkazu_ATS402eq(char *p_buf)
{
char poz_str_cyfry, poziom_nasycenia_char, status;
int poziom_nasycenia_int;

	poz_str_cyfry =(sizeof(etykieta_ATS402eq_tab)-1);
	poziom_nasycenia_int =atoi(p_buf+poz_str_cyfry);
	if ((poziom_nasycenia_int <KAMERA_POZIOM_NASYCENIA_MIN) || (poziom_nasycenia_int >KAMERA_POZIOM_NASYCENIA_MAX))
	{
		Tryb_AT_komunikat_ERROR(BLAD_PRZESLANEGO_PARAMETRU);
		return;
	}
	poziom_nasycenia_char =(char)poziom_nasycenia_int;
		status =Kamera_Nasycenie_regulacja(poziom_nasycenia_char);

		if (status ==TRUE) Tryb_AT_potwierdzenie_OK();
		else Tryb_AT_komunikat_ERROR(BLAD_PROGRAMOWANIA_UKLADU);
}
//---------------------------------------------------------------------------------
//obsuga rozkazu ATS402?	-kamera odczyt zakresu parametru ustawiania poziomu nasycenia
void Obsluga_rozkazu_ATS402p(char *p_buf)
{
int ile_znakow_wysylanych;

	ile_znakow_wysylanych =snprintf(p_BUFOR_TX_USART_GP, ROZMIAR_BUF_RX_TX, "AT$S402=%d-%d nasycenie\r", 
	    KAMERA_POZIOM_NASYCENIA_MIN, KAMERA_POZIOM_NASYCENIA_MAX);

	ile_znakow_wysylanych =strlen(p_BUFOR_TX_USART_GP);

	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;  
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	Selektor_Start_Tx();
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wysanie odpowiedzi 
}
//---------------------------------------------------------------------------------
//obsuga rozkazu ATS402>	-kamera odczyt parametru ustawionego poziomu nasycenia
void Obsluga_rozkazu_ATS402w(char *p_buf)
{
char status, poziom_nasycenia;
unsigned int ile_znakow_wysylanych;

	status =EepOdczyt(EEP_ADR_KAMERA_NASYCENIE, 
			1, &poziom_nasycenia);
	if (status ==0)
	{
		Tryb_AT_komunikat_ERROR(BLAD_ODCZYTU_Z_PAMIECI_KONFIG);
		return;
	} 
	if (((signed char)poziom_nasycenia <KAMERA_POZIOM_NASYCENIA_MIN) || (poziom_nasycenia >KAMERA_POZIOM_NASYCENIA_MAX))
	{
		poziom_nasycenia =KAMERA_POZIOM_NASYCENIA_MAX /2;
	}
	ile_znakow_wysylanych =snprintf(p_BUFOR_TX_USART_GP, ROZMIAR_BUF_RX_TX, "%s%d\r", "ATS402< ", poziom_nasycenia);

	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;  
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	Selektor_Start_Tx();
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wys-anie odpowiedzi 
}
//----------------------------------------------------------
//obsuga rozkazu ATS404=	-ustawianie lub kasowanie flag opcji pracy kamery
void Obsluga_rozkazu_ATS404eq(char *p_buf)
{
char poz_str, status, numer_flagi, pozycja_bitu;
char flagi_opcji_kamery_stan;

	poz_str =(sizeof(etykieta_ATS404eq_tab)-1);
	status =EepOdczyt(EEP_ADR_FLAGI_OPCJI_PRACY_KAMERY, 
			1, &flagi_opcji_kamery_stan);
	if (status ==0)
	{
		Tryb_AT_komunikat_ERROR(BLAD_ODCZYTU_Z_PAMIECI_KONFIG);
		return;
	} 
	//ominicie znaku ' ' poprzedzajcego przesane ustawienia otwarcia/zamknicia rde
	while (isdigit(*(p_buf +poz_str)) ==0) poz_str++; 
	//konwersja przesanego stringu ustawienia/skasowania flag do momentu napotkania kodu '\r'
	//lub znaku nie odpowiadajcego formatowi
	while (*(p_buf +poz_str) !='\r')
	{
		numer_flagi =atoi((p_buf+poz_str));
		if ((numer_flagi ==0) ||( numer_flagi >ILOSC_OPCJI_PRACY_KAMERY))//bd nie obsugiwany numer opcji
		{
			Tryb_AT_komunikat_ERROR(BLAD_PRZESLANEGO_PARAMETRU);
			return;
		}
		poz_str++;
		if (*(p_buf+poz_str) !='=') break;//bd oczekiwanej sygnatury przypisania
		poz_str++;
		pozycja_bitu =1<<(numer_flagi-1);//poz.bitu liczona od 0 numer flagi liczony od 1
		if (*(p_buf+poz_str) ==FLAGA_USTAWIONA)
		{
			flagi_opcji_kamery_stan =
			   flagi_opcji_kamery_stan | pozycja_bitu;
		}
		else
		{
			flagi_opcji_kamery_stan =
			   flagi_opcji_kamery_stan & ~pozycja_bitu;//poz.bitu liczona od 0 numer flagi liczony od 1
		}
		poz_str++;//pozycja rozdzielajcej spacji
		if (*(p_buf +poz_str) =='\r') break;
		poz_str++;	
	}
	//zapis nowych ustawie flag opcji pracy kamery do EEPROM
	status =EepZapis(EEP_ADR_FLAGI_OPCJI_PRACY_KAMERY, 
			1, &flagi_opcji_kamery_stan);
	if (status ==0)
	{
		Tryb_AT_komunikat_ERROR(BLAD_ZAPISU_DO_PAMIECI_KONFIG);
		return;
	} 
	else
	{
		mt9d111_kontrola_ustawienia_AE_standard(flagi_opcji_kamery_stan);
	}
	Tryb_AT_potwierdzenie_OK();
}
//----------------------------------------------------------------------
//obsuga rozkazu ATS404?	-Odczyt informacji o dostpnych ustawieniach flag opcji pracy kamery
void Obsluga_rozkazu_ATS404p(void)
{
int ile_znakow_wysylanych;
extern const char ats404p_lista_flag_string[];

	snprintf(p_BUFOR_TX_USART_GP, ROZMIAR_BUF_RX_TX, "AT$S404=%s\r", &ats404p_lista_flag_string[0]); 

	ile_znakow_wysylanych =strlen(p_BUFOR_TX_USART_GP);

	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;  
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	Selektor_Start_Tx();
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wysanie odpowiedzi 
}
//----------------------------------------------------------
//obsuga rozkazu ATS404>	-odczyt ustawie flag opcji pracy kamery 
void Obsluga_rozkazu_ATS404w(void)
{
char status, flagi_opcji_kamery_stan, x;
char string_odpowiedzi_tab[30],pozycja, znak_ustawien;
int ile_znakow_wysylanych;

	status =EepOdczyt(EEP_ADR_FLAGI_OPCJI_PRACY_KAMERY, 
			1, &flagi_opcji_kamery_stan);
	if (status ==0)
	{
		Tryb_AT_komunikat_ERROR(BLAD_ODCZYTU_Z_PAMIECI_KONFIG);
		return;
	} 
	//formowanie stringu odpowiedzi z ustwieniami flag opcji pracy kamery 
	pozycja =0;
	for (x=0; x<ILOSC_OPCJI_PRACY_KAMERY; x++)
	{
		if ((flagi_opcji_kamery_stan & (1<<x)) !=0) znak_ustawien =FLAGA_USTAWIONA;
		else znak_ustawien =FLAGA_SKASOWANA;
		snprintf(&string_odpowiedzi_tab[pozycja], sizeof(string_odpowiedzi_tab), "%d=%c ",(x+1), znak_ustawien); 
		pozycja =strlen(&string_odpowiedzi_tab[0]);
	}
	snprintf(p_BUFOR_TX_USART_GP, ROZMIAR_BUF_RX_TX, "AT$S404< %s\r", &string_odpowiedzi_tab[0]);

	ile_znakow_wysylanych =strlen(p_BUFOR_TX_USART_GP);

	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;  
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	Selektor_Start_Tx();
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wysanie odpowiedzi 

}
