/**
  ******************************************************************************
  * plik    Tryb_Formatowany.c	 
  * funkcja	Plik procedur zwizanych z transmisj w Trybie Formatowanym 
  * autor 	ARIES Ryszard Szymaniak www.arskam.com
  * wersja 	1.0
  * data    15-08-2012
  * opis   	ARS3-MT9D111 sterownik ARS3 obsugujcy modu kamery MT9D111 firmy APTINA
  * 		STM32F217VGT
  ******************************************************************************
  */  
#include "stm32f2xx_conf.h"
#include "Tryb_Formatowany.h"
#include "Definicje_sprzet.h"
#include "Definicje_program.h"
#include "USART_procedury.h"
#include "EEPROM_procedury.h"
#include "Kamera.h"
#include "Kamera_DMA.h"
#include "Tryb_AT.h"
#include "BMP_enkoder.h"
#include "MT9D111.h"
#include "MT9D111_JPG.h"
#include "Procedury Ogolne.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

char buf_tryb_format_tmp[ROZMIAR_BUFORU_LINIOWEGO];
char identyfikator_transmisji_odebranej_biezacej;

//Procedury Oglne
//struktura zmiennych kontrolujcych spjo wymiany danych
extern struct CNL_SPOJ_WYMIANY_DANYCH{
	char identyfikator_transmisji_wysylanej;
	char identyfikator_ostatniej_trans_odebranej;
	char ile_powtorzen_transmisji;
	uint16_t czas_oczekiwania_na_odpowiedz_ms;
	char *p_bufor_bak;
	int  rozmiar_transmisji_bak;
	char *p_koniec_danych_bak;
	char rodzaj_portu_wyjsciowego_bak;
	}cnl_spoj_wymiany_danych; 

extern struct PREFIKS{
	char flaga_retransmisji;
	char alias_odbiorcy[ROZMIAR_ALIASU_MAX+1];
	char kanal;
	}prefiks;

extern struct OZO_STRUCT{
	char *p_ozo_buf_plik_obraz;//wskanik do buforu z plikiem jpg obrazka
	char ozo_format;			//format ozo
	char ozo_format_bak;		//poprzedni format ozo
	int ozo_obraz_szer_pix;		//szeroko obrazka w pikselach
	int ozo_obraz_wys_lin;		//wysoko obrazka w liniach
	char ozo_obraz_bajty_na_pix;//ilo bajtw na pikselc w tym formacie obrazka
	int rozmiar_pliku_ozo;		//rozmiar pliku obrazka
	char ozo_plik_obraz_typ;	//typ pliku obrazka ozo
	}ozo_struct;

//zmienne USART1
extern char *p_BUFOR_TX_USART_GP;		//wskanik do pocztku buforu transmisji UARTA1
extern int BUFOR_TX_USART_GP_ROZMIAR;	//bajt okrelajcy cakowity rozmiar buforu transmisji UARTA1
extern char *p_TX_USART_GP_IN;			//wskanik biecej pozycji w buf.nadawczym dla danych zapisywanych w buf.
extern char *p_TX_USART_GP_OUT;			//wskanik biecej pozycji w buf.nadawczym dla danych odczytywanych z buf.
extern char tx_USART1_restart_flag;		//poprzednia transmisja zakoczona, nowa musi by zainicjowana

extern char *p_BUFOR_RX_USART_GP;		//wskanik do pocztku buforu odbiorczego UARTA1
extern int BUFOR_RX_USART_GP_ROZMIAR;	//bajt okrelajcy cakowity rozmiar buforu odbiorczego UARTA1
extern char *p_RX_USART_GP_IN;			//wskanik biecej pozycji w buf.odbiorczym dla danych zapisywanych w buf.
extern char *p_RX_USART_GP_OUT;			//wskanik biecej pozycji w buf.odbiorczym dla danych odczytywanych z buf.

//----------------------------------------------------
//Gwna procedura obsugi polece Trybu Formatowanego
void Obsluga_Trybu_Formatowanego(char *p_bufor, uint16_t bufor_rozmiar, char *p_dane_pocz, char *p_dane_kon)
//we:	*p_bufor 		-wskanik do pocztku buforu danych	  	bufor_rozmiar	-cakowity rozmiar buforu	
//		*p_dane_pocz	-wskanik do pocztku danych w buforze	*p_dane_kon 	-wskanik do koca danych w buforze
{
uint16_t ile_danych;
char *p_buf_tryb_format_tmp, status;

	ile_danych =Oblicz_Ilosc_Danych_w_Buf_Kolowym(p_bufor, bufor_rozmiar, p_dane_pocz, p_dane_kon);
	if (ile_danych ==0 || ile_danych >=ROZMIAR_BUFORU_LINIOWEGO) return;
	Odczyt_z_Buf_Kolowego_do_Buf_Liniowego(p_bufor, bufor_rozmiar, p_dane_pocz, 
                                         ile_danych, &buf_tryb_format_tmp[0]);
    //wyszukiwanie w buforze przesanych danych pocztku sygnatury polecenia
	p_buf_tryb_format_tmp =&buf_tryb_format_tmp[0];
	*(p_buf_tryb_format_tmp + ile_danych) ='\0';//dla pewnoci koczenie odebranego acucha zerem 

	while (p_buf_tryb_format_tmp < &buf_tryb_format_tmp[ile_danych])
	{
	    p_buf_tryb_format_tmp =strchr(p_buf_tryb_format_tmp, '$');
		if (p_buf_tryb_format_tmp ==NULL) return;//w buforze nie ma ju znaku sygnatury identyfikujcej pocztek polecenia
		p_buf_tryb_format_tmp++;//wskanik na pozycj kolejnego znaku polecenia
		if (*p_buf_tryb_format_tmp ==TRYB_FORMAT_KOMENDA_A)
		{
			//tu bdzie obsuga odbioru potwierdzenia transmisji
		}
		//sprawdzenie czy na pozycji identyfikatora transmisji jest dozwolony znak
		if (*p_buf_tryb_format_tmp >=TRYB_FORMAT_IDENTYFIKATOR_TRANS_MIN &&
		    *p_buf_tryb_format_tmp <=TRYB_FORMAT_IDENTYFIKATOR_TRANS_MAX)
		{
		    //zapamitanie do porwna identyfikatora transmisji
			identyfikator_transmisji_odebranej_biezacej =*p_buf_tryb_format_tmp;
			//jeeli identyfikator biecej transmisji jest taki sam jak identyfikator poprzedniej
			//naley ponownie przesa poprzednie dane
			if (identyfikator_transmisji_odebranej_biezacej ==cnl_spoj_wymiany_danych.identyfikator_ostatniej_trans_odebranej)
			{
				do
				{
					status =Ponowienie_Transmisji();
				}
				while (status ==FALSE);
				//jeeli zostanie odebrana transmisja z kilkoma rozkazami po ponowieniu transmiji
				//ptla poszuka znaku $ inicjujcego kolejny z przesanych rozkazw
			}
			else
			{
				//dekodowanie i obsuga identyfikatora transmisji
				p_buf_tryb_format_tmp++;//wskanik na pozycj identyfikatora polecenia
				//w zalenoci od aktualnych ustawie i przesanego polecenia obsuga
				//odebranych polece transmisji formatowanej
				p_buf_tryb_format_tmp =Dekodowanie_Obsluga_polecenia_Tr_Format(p_buf_tryb_format_tmp);
			    //aktualizacja identyfikatorw odebranej transmisji
				cnl_spoj_wymiany_danych.identyfikator_ostatniej_trans_odebranej =identyfikator_transmisji_odebranej_biezacej;
	
				if (p_buf_tryb_format_tmp ==NULL) return;//przerwana bdem obsuga polecenia lub grupy polece
			}
		}
		else
		{
			//bdny znak na pozycji identyfikatora transmisji
			return;
		}
	}
}
//---------------------------------------------------------
//procedura dekodowania i obsugi polece Trybu Formatowanego
char * Dekodowanie_Obsluga_polecenia_Tr_Format(char *p_buf_tryb_format_tmp)
//we: p_buf_tryb_format_tmp -wskanik aktualnie przetwarzanej pozycji bufora
//wy: wskanik na kolejn przetwarzan pozycj bufora, NULL -bd dekodowania lub obsugi komendy
{
char sygnatura_komendy_Tr_Format;
char *p_tmp;
char err_koment[100];
uint16_t rozmiar_parametru;

	sygnatura_komendy_Tr_Format =*p_buf_tryb_format_tmp;

	if (sygnatura_komendy_Tr_Format !=TRYB_FORMAT_PREFIKS_f)
	{

		//jeeli przesanie nie rozpoczyna si prefiksem standardowa obsuga polecenia
    	p_buf_tryb_format_tmp++;//wskanik na pozycj cyfry licznika bajtw parametru
		//odszukannie znaku koca bajtw liczby okrelajcej ilo bajtw parametru
		p_tmp =strchr(p_buf_tryb_format_tmp, '\r');
		if (p_tmp ==NULL) return NULL;
		//dekodowanie liczby bajtw parametru komendy
		rozmiar_parametru =atoi(p_buf_tryb_format_tmp);
		p_buf_tryb_format_tmp =(p_tmp+1);//wskanik na pierwszy bajt parametru 
	}
	else
	{
		//na pocztku przesania zosta umieszczony prefiks z adresem odbiorcy
		p_buf_tryb_format_tmp++;//wskanik na pozycj pierwszego znaku alisu identyfikujcego 
								//odbiorc komunikatu
	    p_tmp =strchr(p_buf_tryb_format_tmp, '$');//odszukanie znaku sygnatury polecenia przesyanego
												  //za prefiksem
	    if (p_tmp ==NULL) return NULL;//bd brak sygnatury polecenia za prefiksem
		rozmiar_parametru =(p_tmp - p_buf_tryb_format_tmp);
	}

	while (1)
	{
		switch (sygnatura_komendy_Tr_Format)
		{
			case TRYB_FORMAT_KOMENDA_P:
			{
				p_tmp =Tr_Format_Obsluga_polecenia_P(p_buf_tryb_format_tmp,rozmiar_parametru);
				return p_tmp;
			}
			case TRYB_FORMAT_KOMENDA_D:
			{
				p_tmp =Tr_Format_Obsluga_polecenia_D(p_buf_tryb_format_tmp,rozmiar_parametru);
				return p_tmp;
			}
			case TRYB_FORMAT_KOMENDA_C:
			{
				snprintf(&err_koment[0], 100, "rozkaz '%c'", TRYB_FORMAT_KOMENDA_C);
				Tr_Format_ERROR(BLAD_ROZKAZ_NIE_ROZPOZNANY, &err_koment[0]);
				return NULL;//sygnatura nie rozpoznanej komendy
			}
			case TRYB_FORMAT_KOMENDA_U:
			{
				snprintf(&err_koment[0], 100, "rozkaz '%c'", TRYB_FORMAT_KOMENDA_U);
				Tr_Format_ERROR(BLAD_ROZKAZ_NIE_ROZPOZNANY, &err_koment[0]);
				return NULL;//sygnatura nie rozpoznanej komendy
			}
			case TRYB_FORMAT_KOMENDA_I:
			{
				snprintf(&err_koment[0], 100, "rozkaz '%c'", TRYB_FORMAT_KOMENDA_I);
				Tr_Format_ERROR(BLAD_ROZKAZ_NIE_ROZPOZNANY, &err_koment[0]);
				return NULL;//sygnatura nie rozpoznanej komendy
			}
			case TRYB_FORMAT_KOMENDA_F:
			{
				p_tmp =Tr_Format_Obsluga_polecenia_F(p_buf_tryb_format_tmp,rozmiar_parametru);
				return p_tmp;
			}
			case TRYB_FORMAT_PREFIKS_f:
			{
				snprintf(&err_koment[0], 100, "rozkaz '%c'", TRYB_FORMAT_PREFIKS_f);
				Tr_Format_ERROR(BLAD_ROZKAZ_NIE_ROZPOZNANY, &err_koment[0]);
				return NULL;//sygnatura nie rozpoznanej komendy
			}
			case TRYB_FORMAT_KOMENDA_M:
			{
				snprintf(&err_koment[0], 100, "rozkaz '%c'", TRYB_FORMAT_KOMENDA_M);
				Tr_Format_ERROR(BLAD_ROZKAZ_NIE_ROZPOZNANY, &err_koment[0]);
				return NULL;//sygnatura nie rozpoznanej komendy
			}
			default: 
			{
				snprintf(&err_koment[0], 100, "rozkaz '%c'", sygnatura_komendy_Tr_Format);
				Tr_Format_ERROR(BLAD_ROZKAZ_NIE_ROZPOZNANY, &err_koment[0]);
				return NULL;//sygnatura nie rozpoznanej komendy
			}
	   	}
	}
}
//--------------------------------------------------------------
//odpowied potwierdzajca odebranie i wykonanie polecenia
void Tr_Format_A(void)
{
int ile_znakow_wysylanych;

	ile_znakow_wysylanych =snprintf(p_BUFOR_TX_USART_GP, ROZMIAR_BUF_RX_TX, "$A%c\r",
													identyfikator_transmisji_odebranej_biezacej);
	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wys-anie odpowiedzi 

	p_RX_USART_GP_IN =p_BUFOR_RX_USART_GP;
	p_RX_USART_GP_OUT =p_RX_USART_GP_IN;
}

//--------------------------------------------------------------
//odpowied komunikatem o wystpieniu bdu
void Tr_Format_ERROR(char kod_bledu, char *p_koment)
{
int ile_znakow_wysylanych;
int ile_znakow_komentu;

	ile_znakow_komentu =strlen(p_koment);
	ile_znakow_wysylanych =snprintf(p_BUFOR_TX_USART_GP, ROZMIAR_BUF_RX_TX, "$E%c%d\r%03d %s\r",
													identyfikator_transmisji_odebranej_biezacej,
													ile_znakow_komentu+3,
													kod_bledu, p_koment);	
	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wysanie odpowiedzi 

	p_RX_USART_GP_IN =p_BUFOR_RX_USART_GP;
	p_RX_USART_GP_OUT =p_RX_USART_GP_IN;
}
//-----------------------------------------------------------
//wysyanie potwierdzenia z poleceniem wstrzymania (czas_pauzy_ms w milisekundach) przez partnera kolejnych transmisji
//potwierdzenie z komend wstrzymania bdzie wysyane zgodnie z otwarty_kanal_transmisji
//p_bufor_rx2, bufor_rx2_rozmiar, p_rx_out2, p_tmp
void Tr_Format_Potwierdzenie_Break(uint8_t otwarty_kanal_transmisji, uint16_t czas_pauzy_ms)
{
char *p_buf_tryb_format_tmp, test_tab[6], ile_cyfr;
uint16_t ile_danych;
int ile_znakow_wysylanych;
extern uint8_t	proc_ogol_stan_urzadzenia_info;
//zmienne USART1
extern char PRZERWA_TRANSMISJI_USART_GP_FLAG;
extern char *p_BUFOR_RX_USART_GP;
extern int BUFOR_RX_USART_GP_ROZMIAR;
extern char *p_RX_USART_GP_IN;
extern char *p_RX_USART_GP_OUT;

extern char *p_BUFOR_TX_USART_GP;
extern int BUFOR_TX_USART_GP_ROZMIAR;
extern char *p_TX_USART_GP_IN;
extern char *p_TX_USART_GP_OUT;
extern char tx_USART1_restart_flag;


	if (proc_ogol_stan_urzadzenia_info ==STAN_URZADZENIA_TRYB_TRANS_UART)
	{
	    if (PRZERWA_TRANSMISJI_USART_GP_FLAG ==FALSE) return;//brak odebranych bajtw portem UART 
		PRZERWA_TRANSMISJI_USART_GP_FLAG =FALSE;
		ile_danych =Oblicz_Ilosc_Danych_w_Buf_Kolowym(p_BUFOR_RX_USART_GP, BUFOR_RX_USART_GP_ROZMIAR, 
														p_RX_USART_GP_OUT, p_RX_USART_GP_IN);
		if (ile_danych ==0 || ile_danych >=ROZMIAR_BUFORU_LINIOWEGO) return;
		Odczyt_z_Buf_Kolowego_do_Buf_Liniowego(p_BUFOR_RX_USART_GP, BUFOR_RX_USART_GP_ROZMIAR, 
														p_RX_USART_GP_OUT, ile_danych, &buf_tryb_format_tmp[0]);
	    //wyszukiwanie w buforze przesanych danych pocztku sygnatury komendy
		p_buf_tryb_format_tmp =&buf_tryb_format_tmp[0];
		p_buf_tryb_format_tmp =strchr(p_buf_tryb_format_tmp, '$');
			if (p_buf_tryb_format_tmp ==NULL) return;//w buforze nie ma znaku sygnatury identyfikujcej pocztek komendy
		p_buf_tryb_format_tmp++;//wskanik na pozycj znaku identyfikacji kolejnej transmisji
		identyfikator_transmisji_odebranej_biezacej =*p_buf_tryb_format_tmp;
		//jeeli identyfikator biecej transmisji jest taki sam jak identyfikator poprzedniej
		//naley ponownie przesa poprzednie dane
		if (identyfikator_transmisji_odebranej_biezacej ==cnl_spoj_wymiany_danych.identyfikator_ostatniej_trans_odebranej)
		{
			Ponowienie_Transmisji();
			return;
		}
		cnl_spoj_wymiany_danych.identyfikator_ostatniej_trans_odebranej =identyfikator_transmisji_odebranej_biezacej;
	}
	//jeeli to nie standardowa transmisja kanaem UART przerwanie procedury
	if (proc_ogol_stan_urzadzenia_info !=STAN_URZADZENIA_TRYB_TRANS_UART) return;

	snprintf(&test_tab[0], 6, "%d", czas_pauzy_ms);
	ile_cyfr =strlen(&test_tab[0]);
	ile_znakow_wysylanych =snprintf(p_BUFOR_TX_USART_GP, ROZMIAR_BUF_RX_TX, "$B%c%d\r%d\r", 
	                                               identyfikator_transmisji_odebranej_biezacej, ile_cyfr, czas_pauzy_ms);
	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wys-anie odpowiedzi 

	p_RX_USART_GP_IN =p_BUFOR_RX_USART_GP;
	p_RX_USART_GP_OUT =p_RX_USART_GP_IN;
}
//------------------------------------------------------------------
//obsuga polecenia przesania danych obrazka z okrelonej lokalizacji
char * Tr_Format_Obsluga_polecenia_D(char *p_buf_tryb_format_tmp, uint16_t rozmiar_parametru)
//we: p_buf_tryb_format_tmp -wskanik aktualnie przetwarzanej pozycji bufora: pierwszy bajt parametru
//	  rozmiar_parametru     -rozmiar parametru polecenia w bajtach
//wy: wskanik na kolejn przetwarzan pozycj bufora, NULL -bd obsugi komendy
{
int porownanie, ile_znakow_wysylanych, ile_znakow_danych;
uint32_t rozmiar_pliku;
uint32_t tmp_uint, ozo_data, ozo_czas;
char rozmiar_sciezki_dostepu;


//struktura zawierajca parametry wykorzystywane podczas transmisji plikw
extern struct OBSLUGA_PRZESLANIA_PLIKU{
		char *p_poz_kolejnych_danych_biezaca;
	    char *p_poz_kolejnych_danych_poprzednia;
		int rozmiar_pojedynczego_przeslania_bajty;
		int ile_przeslan_pelnych;
		int rozmiar_ostatniego_przeslania_bajty;
		int rozmiar_buforu_RAM;
		int rozmiar_pliku;
		int ile_danych_przeslano;
		int data_utworzenia_pliku;
		int czas_utworzenia_pliku;
		char sciezka_dostepu_do_pliku_tab[ROZMIAR_SCIEZKI_DOSTEPU_MAX+1];
	  }obsluga_przeslania_pliku;
extern uint8_t	proc_ogol_stan_urzadzenia_info;

	if (rozmiar_parametru !=0)
	{
		//jeeli parametr !=0 oznacza to, e zawiera lokalizacj i nazw pliku z daniem przesania
		//sprawdzenie czy dany plik istnieje i mona go przesa
		//sprawdzenie czy chodzi o plik ozo (ostatnio zarejestrowanego obrazka)
		porownanie =strncmp(TRYB_FORMAT_NAZWA_POLOZENIE_OBRAZKA_OZO_JPG, p_buf_tryb_format_tmp,
		                                                               ILE_PIERWSZYCH_ZNAKOW_IDENTYFIKACJI_OZO);
		if (porownanie ==0)
		{
			obsluga_przeslania_pliku.p_poz_kolejnych_danych_biezaca =ozo_struct.p_ozo_buf_plik_obraz;//wsk.do.kolejnych danych pliku
		    obsluga_przeslania_pliku.p_poz_kolejnych_danych_poprzednia =ozo_struct.p_ozo_buf_plik_obraz;//wsk.do.kolejnych danych pliku bak
			obsluga_przeslania_pliku.ile_danych_przeslano =0;//licznik iloci przesanych danych wykorzystywany 
			                                                 //przy przesaniach z wielokrotnym odczytem kolejnych
															 //czci pliku z SD

			if ((proc_ogol_stan_urzadzenia_info ==STAN_URZADZENIA_RX_TX) || 
				(proc_ogol_stan_urzadzenia_info ==STAN_URZADZENIA_TRYB_TRANS_UART))
			{
	
				obsluga_przeslania_pliku.rozmiar_pojedynczego_przeslania_bajty =
				                           ROZMIAR_POJEDYNCZEGO_PRZESLANIA_DANYCH_PLIKU;//arbitralnie wybrany rozmiar przesania
			} 
			//jakiegokolwiek inne kanay nie bd obsugiwane
			if ((proc_ogol_stan_urzadzenia_info ==STAN_URZADZENIA_RX_TX) && 
				(proc_ogol_stan_urzadzenia_info ==STAN_URZADZENIA_TRYB_TRANS_UART))
			{
			 	return NULL;//BD!
			}

			rozmiar_pliku =ozo_struct.rozmiar_pliku_ozo;
			if (rozmiar_pliku ==0)
			{
				//jeeli rozmiar pliku ozo ustawiony na zero, oznacza to e konwersja JPG nie powioda si
				Tr_Format_ERROR(BLAD_WYNIKOWY_JPG_ZBYT_DUZY, NULL);
				return NULL;
			}
			if (rozmiar_pliku <0xFFFF)
			{
				//plik do przesania <od max.rozmiaru buforu, bdzie tylko 1 odczyt z karty SD
			    obsluga_przeslania_pliku.rozmiar_buforu_RAM	=rozmiar_pliku;
			}
			else
			{
				//plik do przesania >od max.rozmiaru buforu, bdzie wicej ni jeden odczyt z karty SD
				//a rozmiar buforu RAM =wielokrotnoci rozmiaru pojedynczego przesania w bajtach
			    tmp_uint =0xFFFF /obsluga_przeslania_pliku.rozmiar_pojedynczego_przeslania_bajty; 
				tmp_uint =tmp_uint *obsluga_przeslania_pliku.rozmiar_pojedynczego_przeslania_bajty;
				obsluga_przeslania_pliku.rozmiar_buforu_RAM	=tmp_uint;
			}
			obsluga_przeslania_pliku.rozmiar_pliku =rozmiar_pliku;
			rozmiar_sciezki_dostepu =strlen(p_buf_tryb_format_tmp);
			if (rozmiar_sciezki_dostepu >ROZMIAR_SCIEZKI_DOSTEPU_MAX) rozmiar_sciezki_dostepu =ROZMIAR_SCIEZKI_DOSTEPU_MAX;
			strncpy(&obsluga_przeslania_pliku.sciezka_dostepu_do_pliku_tab[0], 
			         p_buf_tryb_format_tmp, rozmiar_sciezki_dostepu+1);


			obsluga_przeslania_pliku.ile_przeslan_pelnych=
			             ozo_struct.rozmiar_pliku_ozo / obsluga_przeslania_pliku.rozmiar_pojedynczego_przeslania_bajty;
			obsluga_przeslania_pliku.rozmiar_ostatniego_przeslania_bajty =
						 (ozo_struct.rozmiar_pliku_ozo % obsluga_przeslania_pliku.rozmiar_pojedynczego_przeslania_bajty);
			ozo_data =1;
			ozo_czas =1;
			ile_znakow_wysylanych =snprintf(p_BUFOR_TX_USART_GP, 
														  ROZMIAR_BUF_RX_TX, "$A%cDs:%d p:%d d:%d t:%d\r",
														  identyfikator_transmisji_odebranej_biezacej,
														  ozo_struct.rozmiar_pliku_ozo,
														  obsluga_przeslania_pliku.rozmiar_pojedynczego_przeslania_bajty,
														  ozo_data,
														  ozo_czas);

			BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
			p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;
			p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
			while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wys-anie odpowiedzi 
		
			p_RX_USART_GP_IN =p_BUFOR_RX_USART_GP;
			p_RX_USART_GP_OUT =p_RX_USART_GP_IN;

			return (p_buf_tryb_format_tmp + rozmiar_parametru);//OK	
		}
		else
		{
			//w tym wariancie nie jest obsugiwany odczyt innego pliku ni ozo
			Tr_Format_ERROR(BLAD_PLIK_NIE_ISTNIEJE, NULL);
			return NULL;
		}
	}
	else
	{ 
		//przesanie kolejnej porcji danych pliku
		//sprawdzenie czy co zostao do wysania
		if ((obsluga_przeslania_pliku.ile_przeslan_pelnych ==0) && 
		    (obsluga_przeslania_pliku.rozmiar_ostatniego_przeslania_bajty ==0))
		{
			//wszystkie dane zostay ju wysane, odesany zostanie jedynie komunikat potwierdzenia	A
		    ile_znakow_wysylanych =snprintf(p_BUFOR_TX_USART_GP, ROZMIAR_BUF_RX_TX, "$A%c\r", identyfikator_transmisji_odebranej_biezacej);

			BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
			p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;
			p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
			while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wys-anie odpowiedzi 
		
			p_RX_USART_GP_IN =p_BUFOR_RX_USART_GP;
			p_RX_USART_GP_OUT =p_RX_USART_GP_IN;

			return (p_buf_tryb_format_tmp + rozmiar_parametru);//OK	
		}
		else
		{
			{
				//obsuga polecenia D -przesanie kolejnych porcji danych pliku
				//kanaem UART-a

				//jako pierwsze przed bajtami danych zostan zapisane do bufora transmisji znaki etykiety przesania
			    ile_znakow_wysylanych =snprintf(p_BUFOR_TX_USART_GP, ROZMIAR_BUF_RX_TX, "$A%cd", identyfikator_transmisji_odebranej_biezacej);
				//dopisanie	do buforu bajtw danych
				if (obsluga_przeslania_pliku.ile_przeslan_pelnych !=0)
				{
					memcpy(p_BUFOR_TX_USART_GP+ile_znakow_wysylanych, 
							obsluga_przeslania_pliku.p_poz_kolejnych_danych_biezaca,
						    obsluga_przeslania_pliku.rozmiar_pojedynczego_przeslania_bajty);
					ile_znakow_wysylanych =ile_znakow_wysylanych +
											obsluga_przeslania_pliku.rozmiar_pojedynczego_przeslania_bajty;
					ile_znakow_danych =obsluga_przeslania_pliku.rozmiar_pojedynczego_przeslania_bajty;
					obsluga_przeslania_pliku.ile_przeslan_pelnych--;
				}
				else
				{
					memcpy(p_BUFOR_TX_USART_GP+ile_znakow_wysylanych, 
							obsluga_przeslania_pliku.p_poz_kolejnych_danych_biezaca,
						    obsluga_przeslania_pliku.rozmiar_ostatniego_przeslania_bajty);
					ile_znakow_wysylanych =ile_znakow_wysylanych +
											obsluga_przeslania_pliku.rozmiar_ostatniego_przeslania_bajty;
					ile_znakow_danych =obsluga_przeslania_pliku.rozmiar_ostatniego_przeslania_bajty;
					obsluga_przeslania_pliku.rozmiar_ostatniego_przeslania_bajty =0;
				}
				obsluga_przeslania_pliku.p_poz_kolejnych_danych_poprzednia =
											  obsluga_przeslania_pliku.p_poz_kolejnych_danych_biezaca;
			    obsluga_przeslania_pliku.p_poz_kolejnych_danych_biezaca =
				    						(obsluga_przeslania_pliku.p_poz_kolejnych_danych_biezaca+ile_znakow_danych);

				BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
				p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;
				p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
				while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wys-anie odpowiedzi 
			
				p_RX_USART_GP_IN =p_BUFOR_RX_USART_GP;
				p_RX_USART_GP_OUT =p_RX_USART_GP_IN;
				return (p_buf_tryb_format_tmp + rozmiar_parametru);//OK	
			}
		}
	}
}	 
//------------------------------------------------------------------
//obsuga polecenia dodania do transmisji prefiksu z adresem odbiorcy
char * Tr_Format_Obsluga_polecenia_F(char *p_buf_tryb_format_tmp, uint16_t rozmiar_parametru)
//we: p_buf_tryb_format_tmp -wskanik aktualnie przetwarzanej pozycji bufora: pierwszy bajt parametru
//	  rozmiar_parametru     -rozmiar parametru polecenia w bajtach
//wy: wskanik na kolejn przetwarzan pozycj bufora, NULL -bd obsugi polecenia
{
char *p_pozycja;

	//polecenie F przesane Gwnym Portem Szeregowym przekazuje string aliasu
	//ktry automatycznie jako prefiks bdzie dodawany do kadej transmiji radiowej	
	if (rozmiar_parametru ==0)
	{
		//rozmiar_parametru =0 oznacza, e prefiks z adresem odbiorcy nie bdzie dodawany do transmisji
		prefiks.flaga_retransmisji =FALSE;
	}
	else
	{
		//odszukanie na ostatniej pozycji znaku okrelajcego kana transmisji
		p_pozycja =strstr(p_buf_tryb_format_tmp, " R");
		if ((p_pozycja !=NULL) && (p_pozycja - p_buf_tryb_format_tmp) >rozmiar_parametru)
		{
			//bd sekwencje " R" zlokalizowano poza przesanym parametrem
			p_pozycja =NULL;
		}
		if (p_pozycja !=NULL)
		{
			prefiks.kanal ='R';			
			*p_pozycja ='\0';
		}
		strcpy(&(prefiks.alias_odbiorcy[0]), p_buf_tryb_format_tmp);
		prefiks.flaga_retransmisji =TRUE;
	}
	Tr_Format_A();
	return (p_buf_tryb_format_tmp + rozmiar_parametru);	
}
//------------------------------------------------------------------
//obsuga polecenia wykonania i natychmiastowego przesania obrazka o okrelonej rozdzielczoci
char * Tr_Format_Obsluga_polecenia_P(char *p_buf_tryb_format_tmp, uint16_t rozmiar_parametru)
//we: p_buf_tryb_format_tmp -wskanik aktualnie przetwarzanej pozycji bufora: pierwszy bajt parametru
//	  rozmiar_parametru     -rozmiar parametru polecenia w bajtach
//wy: wskanik na kolejn przetwarzan pozycj bufora, NULL -bd obsugi polecenia
{
int ile_znakow_wysylanych;
char buf_tmp[16];
int rozmiar_naglowka_jpg, rozmiar_buforu_jpg_max;
uint32_t status_transfer_danych_JPG;

extern char* encode_image (char typ_kodowania_jpg ,char *output_ptr, int image_width, int image_height);
//char *p_tmp;
uint16_t numer_pliku;
//char *p_kodowanie;
char status, quality_factor;
char flagi;

extern int obraz_ozo_parametry_tab[][4];
extern /*__IO */uint8_t bufor_RAM_danych_obrazka[];
extern /*__IO */uint8_t bufor_RAM_pliku_obrazka_format[];



	//identyfikator obrazka moe by pojedyncz liczb lub znakami '<' '>'
	if (rozmiar_parametru <1 || rozmiar_parametru >2) return NULL;//bd rozmiaru identyfikatora obrazka
	memset(&buf_tmp[0], 0, 4);
	memcpy(&buf_tmp[0], p_buf_tryb_format_tmp, rozmiar_parametru);

	while(1)
	{
		if (buf_tmp[0] ==TRYB_FORMAT_IDENTYFIKATOR_OBRAZKA_MINI)
		{
			ozo_struct.ozo_format =FORMAT_OBRAZU_MINIMALNY;
			break;
		}
		if (buf_tmp[0] ==TRYB_FORMAT_IDENTYFIKATOR_OBRAZKA_MAXI)
		{
			ozo_struct.ozo_format =FORMAT_OBRAZU_MAKSYMALNY;
			break;
		}
		ozo_struct.ozo_format =atoi(&buf_tmp[0]);
		break;
	}
	//sprawdzenie czy dany format jest obsugiwany
	if ((ozo_struct.ozo_format <FORMAT_OBRAZU_MINIMALNY)||(ozo_struct.ozo_format >FORMAT_OBRAZU_MAKSYMALNY))
	{
		//bd nie obsugiwany format obrazka
		return NULL;
	}
	//okrelanie typu pliku przyporzdkowanego do numeru formatu
	ozo_struct.ozo_plik_obraz_typ =obraz_ozo_parametry_tab[ozo_struct.ozo_format][OBRAZ_PARAM_PLIK_TYP_POZ];

	//wysanie potwierdzenia otrzymania i rozpoczcia realizacji polecenia P
	//nazwa ozo_jpg zostanie uzupeniona biecym numerem
#if (ZEGAR_RTC ==ZAMONTOWANY_ZEGAR_RTC)
	numer_pliku =BKP_ReadBackupRegister(BKP_REJESTR_NUMER_PLIKU);
	if (numer_pliku>ILOSC_ZDJEC_DZIENNIE_MAX) numer_pliku =1;
#else
	//UWAGA dla ukadu bez zegara RTC
	numer_pliku =1;
#endif

#if (ZAMONTOWANA_KAMERA	==BRAK_ZAMONTOWANEJ_KAMERY)
	//UWAGA dla ukadu bez zamontowanej kamery
	Tr_Format_ERROR(BLAD_ROZKAZ_NIE_ROZPOZNANY, &err_koment[0]);
	return;
#endif

	//odsyanie potwierdzenia z nazw ozo_xxx.xxx z rozszerzeniem zalenym od formatu pliku 
	memchr(p_buf_tryb_format_tmp, 'B', rozmiar_parametru);
	if (ozo_struct.ozo_plik_obraz_typ ==OBRAZ_PLIK_BMP)//if (p_kodowanie !=NULL)
	{
		//format BMP			
		strncpy(&buf_tmp[0], TRYB_FORMAT_NAZWA_POLOZENIE_OBRAZKA_OZO_BMP, ILE_PIERWSZYCH_ZNAKOW_IDENTYFIKACJI_OZO); 
		snprintf (&buf_tmp[ILE_PIERWSZYCH_ZNAKOW_IDENTYFIKACJI_OZO], sizeof(buf_tmp), "%04d%s", 
		                      numer_pliku, &TRYB_FORMAT_NAZWA_POLOZENIE_OBRAZKA_OZO_BMP[ILE_PIERWSZYCH_ZNAKOW_IDENTYFIKACJI_OZO]);
	}
	else
	{
		//inny format domylnie JPG
		strncpy(&buf_tmp[0], TRYB_FORMAT_NAZWA_POLOZENIE_OBRAZKA_OZO_JPG, ILE_PIERWSZYCH_ZNAKOW_IDENTYFIKACJI_OZO); 
		snprintf (&buf_tmp[ILE_PIERWSZYCH_ZNAKOW_IDENTYFIKACJI_OZO], sizeof(buf_tmp), "%04d%s", 
		                      numer_pliku, &TRYB_FORMAT_NAZWA_POLOZENIE_OBRAZKA_OZO_JPG[ILE_PIERWSZYCH_ZNAKOW_IDENTYFIKACJI_OZO]);
	}

	ile_znakow_wysylanych =snprintf(p_BUFOR_TX_USART_GP, ROZMIAR_BUF_RX_TX, "$A%cP%s\r", identyfikator_transmisji_odebranej_biezacej,
																				&buf_tmp[0]);
	BUFOR_TX_USART_GP_ROZMIAR =ROZMIAR_BUF_RX_TX;
	p_TX_USART_GP_IN =p_BUFOR_TX_USART_GP +ile_znakow_wysylanych;
	p_TX_USART_GP_OUT =p_BUFOR_TX_USART_GP;
	while (Selektor_Start_Tx() ==FALSE);//oczekiwanie na wys-anie odpowiedzi 

	p_RX_USART_GP_IN =p_BUFOR_RX_USART_GP;
	p_RX_USART_GP_OUT =p_RX_USART_GP_IN;


	//rozpoczcie pobierania obrazka z kamery i kodowania JPG
	//w tym czasie powinna by kontrolowana flaga main.c->przerwa_transmisji_flag
	//i jeeli zostanie ustawiona obsuenie w osobnym podprogramie odsyania 
	//rozkazu BREAK
	//wykonanie fotografii ikompresja do formatu JPEG
	//ustawienie rejestrw wymiarw obrazka zgodnie z numerem formatu standardowego wymiaru
	status =mt9d111_Zmiana_Standardowych_Rozmiarow_Obrazu_Kontekst_B(ozo_struct.ozo_format);
	status =EepOdczyt(EEP_ADR_FLAGI_OPCJI_PRACY_KAMERY, 1, &flagi);
	mt9d111_ustawienie_luster_obrazu(flagi);

	if (status ==TRUE)
	{
		memchr(p_buf_tryb_format_tmp, 'B', rozmiar_parametru);
		if (ozo_struct.ozo_plik_obraz_typ ==OBRAZ_PLIK_BMP)//if (p_kodowanie !=NULL)
		{
			//kodowanie .BMP
			Inicjacja_DMA_MT9D111_BMP_RAM((uint8_t *)&bufor_RAM_pliku_obrazka_format[0],
									 ozo_struct.ozo_obraz_szer_pix, 
									 ozo_struct.ozo_obraz_wys_lin, 
									 ozo_struct.ozo_obraz_bajty_na_pix); 
			mt9d111_zmian_formatu_danych_obrazu_kontekstu(KONTEKST_B);
			mt9d111_zapis_do_buf_ramki_RGB();
	
			ozo_struct.rozmiar_pliku_ozo =Kodowanie_z_bufor_RAM_danych_na_BMP_do_RAM_pliku(
												  //(uint8_t *)&bufor_RAM_danych_obrazka[0],			//bufor danych ramki obrazka
												  &bufor_RAM_pliku_obrazka_format[0],
												  ozo_struct.ozo_obraz_szer_pix,		//szeroko obrazka w pikselach
												  ozo_struct.ozo_obraz_wys_lin,			//wysoko obrazka w liniach
												  ozo_struct.ozo_obraz_bajty_na_pix,	//ilo bajtw na piksel
												  //(uint8_t *)&bufor_RAM_pliku_obrazka_format[0], 	//bufor wyjciowy pliku obrazka BMP
												  &bufor_RAM_pliku_obrazka_format[ROZMIAR_PLIKU_OBRAZKA_160_120+100],
												  //ROZMIAR_BUF_PLIKU_FORMAT_OBRAZ		//max. rozmiar buforu wyjciowego
												  ROZMIAR_BUF_PLIKU_FORMAT_OBRAZ -(ROZMIAR_PLIKU_OBRAZKA_160_120+100)												  
												  );	
			if (ozo_struct.rozmiar_pliku_ozo !=0) ozo_struct.p_ozo_buf_plik_obraz =(char *)&bufor_RAM_pliku_obrazka_format[ROZMIAR_PLIKU_OBRAZKA_160_120+100];
	   }
	   else
	   {
			//Kodowanie .JPG
			quality_factor =mt9d111_odczyt_wspolczynnika_kompresji(flagi);
			while (1)
			{
				rozmiar_naglowka_jpg =CreateJpegHeader((char *)&bufor_RAM_pliku_obrazka_format[0], 
										ozo_struct.ozo_obraz_szer_pix, 
										ozo_struct.ozo_obraz_wys_lin, 
										FORMAT_YCBCR422, 
										RESTART_MARKER_JPG_INSERT_DISABLE, 
										quality_factor);	 
	
			   MT9D111_JPG_Inicjacja_Rejestrow_1(FORMAT_YCBCR422, quality_factor, ozo_struct.ozo_format);
	
			   rozmiar_buforu_jpg_max =ROZMIAR_BUF_PLIKU_FORMAT_OBRAZ -rozmiar_naglowka_jpg -100;
			   Inicjacja_DMA_MT9D111_JPG_RAM((uint8_t *)&bufor_RAM_pliku_obrazka_format[rozmiar_naglowka_jpg], 
			   								  ROZMIAR_POJEDYNCZEGO_PRZESLANIA_DMA_DLA_JPG);
			   //ozo_struct.rozmiar_pliku_ozo =mt9d111_zapis_do_buf_ramki_JPG_1(&rozmiar_buforu_jpg_max,
			   //																  ROZMIAR_POJEDYNCZEGO_PRZESLANIA_DMA_DLA_JPG);
			   status_transfer_danych_JPG =mt9d111_zapis_do_buf_ramki_JPG_2((uint32_t *)&rozmiar_buforu_jpg_max,
			   														   ROZMIAR_POJEDYNCZEGO_PRZESLANIA_DMA_DLA_JPG);
			   if (status_transfer_danych_JPG ==ERR_JPG_DANE_BUFOR_ZA_MALY)
			   {
			   		//za may bufor, powtrne kodowanie ze zwikszonym wspczynnikiem kompresji
			   		quality_factor =quality_factor +10;
					if (quality_factor >=40) 
					{
						rozmiar_buforu_jpg_max =0;
						break;
					}
					else
					{
						writeRegister(JPEG_STATUS_REGISTER_0, 0x01);//kasowanie flag statusu
						Delay_ms(2000);
					 	continue;
					}	
			   }
			   break;
		   }
		   writeRegister(JPEG_STATUS_REGISTER_0, 0x01);//kasowanie flag statusu

		   ozo_struct.rozmiar_pliku_ozo =rozmiar_buforu_jpg_max;
		   ozo_struct.rozmiar_pliku_ozo =ozo_struct.rozmiar_pliku_ozo +rozmiar_naglowka_jpg;
		   ozo_struct.p_ozo_buf_plik_obraz =(char *)&bufor_RAM_pliku_obrazka_format[0];
	   }
	}

	return (p_buf_tryb_format_tmp + rozmiar_parametru);	

}
