/**
  ******************************************************************************
  * plik    EEPROM_procedury.h
  * funkcja	Plik nagwkowy procedur zwizanych z programow obsug pamici EEPROM 
  *         z 0/1/2 bajtowym adresowaniem
  * autor 	ARIES Ryszard Szymaniak www.arskam.com
  * wersja 	1.0
  * data    10-08-2012
  * opis   	ARS3-MT9D111 sterownik ARS3 obsugujcy modu kamery MT9D111 firmy APTINA
  * 		STM32F217VGT
  ******************************************************************************
  */  
/* Define to prevent recursive inclusion ------------------------------------ */
#ifndef __EEPROM_PROCEDURY_H
#define __EEPROM_PROCEDURY_H

char EepOdczyt(unsigned int adres_int, unsigned int ilosc_bajtow, char *p_bufor);
char EepOdczytBloku(unsigned int adres_int, unsigned char ilosc_bajtow, char *p_bufor);
char EepZapis(unsigned int adres_int, unsigned int ilosc_bajtow, char *p_bufor);
char EepZapisBloku(unsigned int adres_int, unsigned char ilosc_bajtow, char *p_bufor);
unsigned char I2c_(unsigned char ile_bajtow_adresu);
unsigned char I2c_byte_wr(unsigned char bajt);
unsigned char I2c_byte_rd(unsigned char ack_on);
unsigned char I2c_cnl(void);
void I2c_start(void);
void I2c_stop(void);

#define ADRES_I2C_1BAJT   0
#define ADRES_I2C_2BAJTY  1
#define ADRES_I2C_0BAJT   2

//deklaracje zwizane z EEPROM-em
#define ROZMIAR_STRONY_EEPROM			32	//rozmiar strony EEPROM w bajtach do zapisu grupowego
#define ROZMIAR_PAMIECI_EEPROM			16384
//rozmiar buforu do operacji zapisu/odczytu EEPROM
#define ROZMIAR_BUF_POSREDNI			0x100


//PROFIL REJESTRW RADIOWYCH
	//adres w EEPROM bajtu z iloci dostpnych profili ustawie rejestrw radiowych
#define EEP_ADR_ILOSC_PROFILI_REG_RADIO	0
#define ILOSC_PROFILI_REG_RADIO_MAX		10
#define ROZMIAR_PROFILU_REG_RADIO_MAX	380
	//adres pierwszego profilu ustawie rejestrw radiowych
#define EEP_ADR_PIERWSZY_PROFIL_REG_RADIO	(ROZMIAR_PAMIECI_EEPROM -(ILOSC_PROFILI_REG_RADIO_MAX *ROZMIAR_PROFILU_REG_RADIO_MAX))
	//adres tabeli 2 bajtowych adresw w EEPROM-ie pocztkw kolejnych profili
	//rozmiar tabel =ILOSC_PROFILI_REG_RADIO_MAX * 2
#define EEP_ADR_TAB_ADRESOW_PROFILI_RAD		(EEP_ADR_ILOSC_PROFILI_REG_RADIO +1)
	//adres bajtu zawierajcego numer wybranego profilu ustawie rejestrw radiowych
	//jednoczenie(numer szybkoci radiowej)
#define EEP_ADR_WYBRANY_PROFIL_REG_RADIO	(EEP_ADR_TAB_ADRESOW_PROFILI_RAD +(ILOSC_PROFILI_REG_RADIO_MAX * 2)+1)

//PROFIL PROGW MOCY TRANSMISJI RADIOWEJ
#define ROZMIAR_PROFILU_POW_RADIO_TRANS_MAX	200
	//adres profilu progw mocy transmisji radiowej
#define EEP_ADR_PROFIL_POW_RADIO_TRANS	(EEP_ADR_PIERWSZY_PROFIL_REG_RADIO -ROZMIAR_PROFILU_POW_RADIO_TRANS_MAX)
	//adres bajtu zawierajcego numer wybranego progu mocy
#define EEP_ADR_WYBRANY_NUM_PROGU_MOCY	(EEP_ADR_WYBRANY_PROFIL_REG_RADIO +1)
	//adres bajtu z iloci dostpnych do wyboru progw mocy
#define EEP_ADR_ILOSC_PROGOW_MOCY		(EEP_ADR_WYBRANY_NUM_PROGU_MOCY +1)

//KANA RADIOWY
	//adres bajtu z iloci dostpnych do wyboru kana-w radiowych
#define EEP_ADR_ILOSC_KANALOW_RADIO		(EEP_ADR_ILOSC_PROGOW_MOCY +1)
	//adres bajtu zawierajcy numer wybranego kana-u radiowego
#define EEP_ADR_NUMER_KANALU_RADIO		(EEP_ADR_ILOSC_KANALOW_RADIO+1)

//ALIAS URZDZENIA
#define ROZMIAR_ALIASU_MAX				16
	//adres (ROZMIAR_ALIASU_MAX) bajtowego buforu aliasu
#define EEP_ADR_ALIAS					(EEP_ADR_NUMER_KANALU_RADIO+1)

//UART
//ustawienia USART-a	urzdzenia i partnera
#define ILOSC_SZYBKOSCI_TRANSMISJI_UART	8
	//adres bajtu zawierajcego numer wybranej szybkoci transmisji
#define EEP_ADR_NUMER_SZYBKOSCI_UART	(EEP_ADR_ALIAS +ROZMIAR_ALIASU_MAX +1)

//kontrola przep-ywu portem USART (RTS/CTS)
#define USART_PRZEPLYW_OFF				0
#define USART_CTS_RTS_ON				1
	//adres bajtu ktrego zawarto steruje kontrol przep-ywu
#define EEP_ADR_USART_PRZEPLYW_CNT				(EEP_ADR_NUMER_SZYBKOSCI_UART +1)

//parametry USART-a partnera
#define ILOSC_SZYBKOSCI_TRANSMISJI_UART_PARTNER	8
	//adres bajtu zawierajcego numer szybkoci transmisji partnera
#define EEP_ADR_NUMER_SZYBKOSCI_UART_PARTNER	(EEP_ADR_USART_PRZEPLYW_CNT +1)

//PARAMETRY TRANSMISJI RADIOWEJ
//czas opnienia kolejnej transmisji radiowej 
#define CZAS_OPOZNIENIA_TRANS_RADIO_MSEK_MIN	5
#define CZAS_OPOZNIENIA_TRANS_RADIO_MSEK_MAX	65000//warto max. jeli Timer1 liczy 1ms
	//adres 2 bajtowego HL parametru czasu wyzwalania 
#define EEP_ADR_CZAS_OPOZNIENIA_TRANS_RADIO		(EEP_ADR_NUMER_SZYBKOSCI_UART_PARTNER +1)

//czas oczekiwania na potwierdzenie
#define CZAS_OCZEKIWANIA_NA_POTWIERDZENIE_MSEK_MIN	10
#define CZAS_OCZEKIWANIA_NA_POTWIERDZENIE_MSEK_MAX	65000//warto max. jeli Timer1 liczy 1ms
	//adres 2 bajtowego HL parametru czasu wyzwalania
#define EEP_ADR_CZAS_OCZEKIWANIA_NA_POTWIERDZENIE	(EEP_ADR_CZAS_OPOZNIENIA_TRANS_RADIO +2)

//ilo powtrze transmisji radiowej przy braku potwierdzenie w trybie formatowanym
#define ILOSC_POWTORZEN_TRANS_RADIO_MIN			0
#define ILOSC_POWTORZEN_TRANS_RADIO_MAX			255
	//adres bajtu parametru iloci powtrze
#define EEP_ADR_ILOSC_POWTORZEN_TRANS_RADIO		(EEP_ADR_CZAS_OCZEKIWANIA_NA_POTWIERDZENIE +2)

//ustawienia trybu transmisji radiowej
#define TRYB_TRANS_RADIO_BRAK							0
#define TRYB_TRANS_RADIO_MODEMOWY						1 //transmisja kana-em radiowym w prostym trybie modemu bez bajtw dodatkowych ramek
#define TRYB_TRANS_RADIO_FORMATOWANY_Z_POTWIERDZENIEM	2
	//adres bajtu parametru trybu transmisj radiowej
#define EEP_ADR_TRYB_TRANS_RADIO				(EEP_ADR_ILOSC_POWTORZEN_TRANS_RADIO +1)

//KAMERA
//kamera ustawienie parametru poziom JASKRAWOCI
#define KAMERA_POZIOM_JASKRAWOSCI_MIN	0
#define KAMERA_POZIOM_JASKRAWOSCI_MAX	100
	//adres bajtu jaskrawoci
#define EEP_ADR_KAMERA_JASKRAWOSC				(EEP_ADR_TRYB_TRANS_RADIO +1)
//kamera ustawienie parametru poziom KONTRASTU
#define KAMERA_POZIOM_KONTRASTU_MIN		0
#define KAMERA_POZIOM_KONTRASTU_MAX		100
	//adres bajtu jaskrawoci
#define EEP_ADR_KAMERA_KONTRAST					(EEP_ADR_KAMERA_JASKRAWOSC +1)
//kamera ustawienie parametru poziom NASYCENIA
#define KAMERA_POZIOM_NASYCENIA_MIN		0
#define KAMERA_POZIOM_NASYCENIA_MAX		100
	//adres bajtu nasycenia
#define EEP_ADR_KAMERA_NASYCENIE				(EEP_ADR_KAMERA_KONTRAST +1)
//kamera rczne ustawienie warunkw ekspozycji
#define KAMERA_WARUNKI_EKSPOZYCJI_MIN	0
#define KAMERA_WARUNKI_EKSPOZYCJI_MAX	100
	//adres bajtu numeru migawki
#define EEP_ADR_WARUNKI_EKSPOZYCJI				(EEP_ADR_KAMERA_NASYCENIE +1)
	//adres bajtu flag opcji pracy kamery
//b.0 lustro pozim, b.1 lustro pion
#define EEP_POZ_KAM_FLAG_LUSTRO_POZIOM	0
#define EEP_POZ_KAM_FLAG_LUSTRO_PION	1
#define EEP_POZ_KAM_FLAG_KOMPRESJA_JPG	2
#define EEP_POZ_KAM_FLAG_AE_STANDARD	3

#define EEP_ADR_FLAGI_OPCJI_PRACY_KAMERY		(EEP_ADR_WARUNKI_EKSPOZYCJI +1)


//REJESTRY ZDARZE
	//adres bajtu blokady lub zezwolenia na reakcj na zdarzenie pochodzce ze rd-a
	//NIE UYWANE! b.0 pierwsze rd-o LINIA1 ... b.4 ostatnie rd-o zdarze zegar RTC
#define EEP_ADR_ODBLOKOWANIE_ZRODLA_ZDARZEN		(EEP_ADR_FLAGI_OPCJI_PRACY_KAMERY +1)
//scenariusze reakcji na zdarzenie ze rd-a
#define ROZMIAR_SCENARIUSZA_REAKCJII_MAX	64//bajty
	//adres grupy rejestrw z rozmiarem scenariusza w bajtach dla kadego rd-a
	//rozmiar scenariusza dla rd-a przechowywane s w pamici w kolejnoci okrelonej w tabeli numeracja_zrodel_zdarzen_tab[]
#define EEP_ADR_ROZMIARY_SCENARIUSZY_ZRODEL		(EEP_ADR_ODBLOKOWANIE_ZRODLA_ZDARZEN +1)
	//adres grupy obszarw przechowujcych scenariusze reakcji na zdarzenia 
	//kody scenariusza reakcji rd-a przechowywane s w pamici w kolejnoci okrelonej w tabeli numeracja_zrodel_zdarzen_tab[]
#define EEP_ADR_DANE_SCENARIUSZY_REAKCJI		(EEP_ADR_ROZMIARY_SCENARIUSZY_ZRODEL +(ILOSC_ZRODEL_ZDARZEN*1)+1)
	//adres grupy rejestrw z warunkami zdarzenia
	//warunki zdarzenia przechowywane s w pamici w kolejnoci okrelonej w tabeli numeracja_zrodel_zdarzen_tab[] 
#define EEP_ADR_WARUNKI_ZDARZEN					(EEP_ADR_DANE_SCENARIUSZY_REAKCJI +(ILOSC_ZRODEL_ZDARZEN *ROZMIAR_SCENARIUSZA_REAKCJII_MAX) +1)
	//adres 4 bajtw parametru zdarzenia bdcego alarmem zegara RTC (czas wyraony w liczbie sekund od 00:00:00) HL
#define EEP_ADR_PARAMETR_WARUNKU_ZDARZENIA_ALARMU_RTC	(EEP_ADR_WARUNKI_ZDARZEN +(ILOSC_ZRODEL_ZDARZEN*1)+1)
	//adres 2 bajtw parametru zdarzenia bdcego powtrzeniem alarmu zegara RTC w minutach	 HL
#define EEP_ADR_PARAM_WARUNKU_ZDARZENIA_ALARMU_RTC_POWT_MINUTY	(EEP_ADR_PARAMETR_WARUNKU_ZDARZENIA_ALARMU_RTC +4)
	//adres grupy rejestrw przechowujcych numer stanu (ON/OFF) wszystkich rde- przywracany po w-czeniu zasilania
	//stany rde- przechowywane s w pamici w kolejnoci okrelonej w tabeli numeracja_zrodel_zdarzen_tab[]
#define EEP_ADR_STAN_ZRODLA_PO_WLACZENIU_ZASILANIA	(EEP_ADR_PARAM_WARUNKU_ZDARZENIA_ALARMU_RTC_POWT_MINUTY +2)	

//ostatni zajty adres obszaru danych EEPROM od pocztku
#define EEP_ADR_KONIEC				EEP_ADR_STAN_ZRODLA_PO_WLACZENIU_ZASILANIA +(ILOSC_ZRODEL_ZDARZEN*1)+1		
//rozmiar kocowego zajtego obszaru EEPROM 3700 bajtw

/*
//--------------
//sterowanie liniami portw
#define FUNKCJA_LINII_OFF				0
#define FUNKCJA_LINII_IN				1
#define FUNKCJA_LINII_OUT				2
#define FUNKCJA_LINII_TRANS_INI			3
#define FUNKCJA_LINII_POTWIERDZENIE		4
#define FUNKCJA_LINII_AD_IN				5
#define FUNKCJA_LINII_DF_OUT			6
#define FUNKCJA_LINII_PRZESLIJ_ODCZYT	7

#define ILOSC_LINII_PORTOW	8
	//adres 16 bajtowej tabeli przypisa funkcji do linii portw
	//EEP_ADR_FUNKCJE_LINII_PORTOW_TAB -linia1 ... EEP_ADR_FUNKCJE_LINII_PORTOW_TAB+15 linia16
#define EEP_ADR_FUNKCJE_LINII_PORTOW_TAB		(EEP_ADR_USART_PRZEPLYW_CNT+1)

//ustawienie po zerowaniu domylnych poziomw linii wyjciowych
#define DOMYSLE_POZIOMY_LINII_IO_L		0
#define DOMYSLE_POZIOMY_LINII_IO_H		255
	//adres 2 bajtowego parametru ktrego bity ustawiaj poziom linii po zerowaniu
	//b.7 Linia8 ... b.0 Linia1
#define EEP_ADR_DOMYSLE_POZIOMY_LINII_IO		(EEP_ADR_FUNKCJE_LINII_PORTOW_TAB +16+1)
//ustawienie po zerowaniu domylnych wartoci dla linii pracujcych jako 
//wyjcia przetwornikw DF
#define DOMYSLNE_WARTOSCI_DF_MIN		0
#define DOMYSLNE_WARTOSCI_DF_MAX		4095
	//adres 2 bajtowego parametru domylnej wartoci dla Linii1 
	//pracujcej jako wyjcie DF
#define EEP_ADR_DOMYSLNE_WARTOSCI_DF_LINIA1		(EEP_ADR_DOMYSLE_POZIOMY_LINII_IO +2)
	//adres 2 bajtowego parametru domylnej wartoci dla Linii2 
	//pracujcej jako wyjcie DF
#define EEP_ADR_DOMYSLNE_WARTOSCI_DF_LINIA2		(EEP_ADR_DOMYSLNE_WARTOSCI_DF_LINIA1 +2)
	//adres 2 bajtowego parametru domylnej wartoci dla Linii3 
	//pracujcej jako wyjcie DF
#define EEP_ADR_DOMYSLNE_WARTOSCI_DF_LINIA3		(EEP_ADR_DOMYSLNE_WARTOSCI_DF_LINIA2 +2)
	//adres 2 bajtowego parametru domylnej wartoci dla Linii4 
	//pracujcej jako wyjcie DF
#define EEP_ADR_DOMYSLNE_WARTOSCI_DF_LINIA4		(EEP_ADR_DOMYSLNE_WARTOSCI_DF_LINIA3 +2)
*/


#endif

