/**
  ******************************************************************************
  * plik    Kamera.h	 
  * funkcja	Plik nagwkowy procedur poredniczcych zwizanych z obsug rnych moduw kamer
  * autor 	ARIES Ryszard Szymaniak www.arskam.com
  * wersja 	1.0
  * data    15-08-2012
  * opis   	ARS3-MT9D111 sterownik ARS3 obsugujcy modu kamery MT9D111 firmy APTINA
  * 		STM32F217VGT
  ******************************************************************************
  */  
/* Define to prevent recursive inclusion ------------------------------------ */
#ifndef __KAMERA_H
#define __KAMERA_H

#include "stm32f2xx_conf.h"

//bufor danych obrazu
#define ROZMIAR_BUF_OBRAZU				400	//QQVGA 160 x 120 x 2
#define ROZMIAR_BUF_PLIKU_FORMAT_OBRAZ	106000
#define ROZMIAR_PLIKU_OBRAZKA_160_120   160*120*2

//dostpne formaty obrazw kamery
#define NUMER_FORMATU_OZO_QVGA_BMP		0	//format nie wystpuje
#define NUMER_FORMATU_OZO_VGA_BMP		0	//format nie wystpuje

#define NUMER_FORMATU_OZO_VGA_JPG		4
#define NUMER_FORMATU_OZO_QVGA_JPG		3
#define NUMER_FORMATU_OZO_QQVGA_JPG		2
#define NUMER_FORMATU_OZO_QQVGA_BMP		1
#define NUMER_FORMATU_OBRAZU_ZADEN		0

#define VGA_ILOSC_PIKSELI				640
#define VGA_ILOSC_LINII					480
#define VGA_ILOSC_BAJTOW_NA_PIKSEL		2

#define QVGA_ILOSC_PIKSELI				320
#define QVGA_ILOSC_LINII				240
#define QVGA_ILOSC_BAJTOW_NA_PIKSEL		2

#define QQVGA_ILOSC_PIKSELI				160
#define QQVGA_ILOSC_LINII				120
#define QQVGA_ILOSC_BAJTOW_NA_PIKSEL	2

//sygnatury formatu min/max obrazu 
//UWAGA musi by zachowana zgodno z info_parametry_kamery_tab[] Globalne_Stale_Zmienne.c
#define FORMAT_OBRAZU_MINIMALNY			NUMER_FORMATU_OZO_QQVGA_BMP
#define FORMAT_OBRAZU_MAKSYMALNY		NUMER_FORMATU_OZO_VGA_JPG



//pozycje parametrw w tabeli Globalne_Stale_Zmienne.c->obraz_ozo_parametry_tab[][]
#define OBRAZ_PARAM_PIKSELE_POZ		0
#define OBRAZ_PARAM_LINIE_POZ		1
#define OBRAZ_PARAM_BAJTY_NA_PIKS	2
#define OBRAZ_PARAM_PLIK_TYP_POZ	3

//typ pliku obrazka
#define OBRAZ_PLIK_BMP					'B'
#define OBRAZ_PLIK_JPG					'J'

//definicja zwizane z trybem kodowania JPG
#define KODOWANIE_JPG_Z_EXT_SRAM		0
#define KODOWANIE_JPG_Z_PLIKU_NA_SD	    1

#define TRYB_FORMAT_IDENTYFIKATOR_OBRAZKA_MINI	'<'
#define TRYB_FORMAT_IDENTYFIKATOR_OBRAZKA_MAXI	'>'
#define TRYB_FORMAT_NAZWA_POLOZENIE_OBRAZKA_OZO_JPG	"/ozo_.jpg"	//identyfikator ostatnio zarejestrowanego obrazka (ozo)
#define TRYB_FORMAT_NAZWA_POLOZENIE_OBRAZKA_OZO_BMP "/ozo_.bmp"	//identyfikator ostatnio zarejestrowanego obrazka (ozo.bmp)
#define ILE_PIERWSZYCH_ZNAKOW_IDENTYFIKACJI_OZO	5


//definicje linii magistrali danych kamery
#define KAM_DATn	8

typedef enum 
{
	KAM_DAT0				=0,
	KAM_DAT1				=1,
	KAM_DAT2				=2,
	KAM_DAT3				=3,
	KAM_DAT4				=4,
	KAM_DAT5				=5,
	KAM_DAT6				=6,
	KAM_DAT7				=7						
}Kam_DAT_TypeDef;
//Linia0 danych
#define	KAM_DAT0_PORT    		GPIOC
#define	KAM_DAT0_CLK			RCC_AHB1Periph_GPIOC
#define	KAM_DAT0_PIN			GPIO_Pin_6
#define KAM_DAT0_PIN_SOURCE		GPIO_PinSource6
//Linia1 danych
#define	KAM_DAT1_PORT    		GPIOC
#define	KAM_DAT1_CLK			RCC_AHB1Periph_GPIOC
#define	KAM_DAT1_PIN			GPIO_Pin_7
#define KAM_DAT1_PIN_SOURCE		GPIO_PinSource7
//Linia2 danych
#define	KAM_DAT2_PORT    		GPIOC
#define	KAM_DAT2_CLK			RCC_AHB1Periph_GPIOC
#define	KAM_DAT2_PIN			GPIO_Pin_8
#define KAM_DAT2_PIN_SOURCE		GPIO_PinSource8
//Linia3 danych
#define	KAM_DAT3_PORT    		GPIOC
#define	KAM_DAT3_CLK			RCC_AHB1Periph_GPIOC
#define	KAM_DAT3_PIN			GPIO_Pin_9
#define KAM_DAT3_PIN_SOURCE		GPIO_PinSource9
//Linia4 danych
#define	KAM_DAT4_PORT    		GPIOE
#define	KAM_DAT4_CLK			RCC_AHB1Periph_GPIOE
#define	KAM_DAT4_PIN			GPIO_Pin_4
#define KAM_DAT4_PIN_SOURCE		GPIO_PinSource4
//Linia5 danych
#define	KAM_DAT5_PORT    		GPIOB
#define	KAM_DAT5_CLK			RCC_AHB1Periph_GPIOB
#define	KAM_DAT5_PIN			GPIO_Pin_6
#define KAM_DAT5_PIN_SOURCE		GPIO_PinSource6
//Linia6 danych
#define	KAM_DAT6_PORT    		GPIOE
#define	KAM_DAT6_CLK			RCC_AHB1Periph_GPIOE
#define	KAM_DAT6_PIN			GPIO_Pin_5
#define KAM_DAT6_PIN_SOURCE		GPIO_PinSource5
//Linia7 danych
#define	KAM_DAT7_PORT    		GPIOE
#define	KAM_DAT7_CLK			RCC_AHB1Periph_GPIOE
#define	KAM_DAT7_PIN			GPIO_Pin_6
#define KAM_DAT7_PIN_SOURCE		GPIO_PinSource6

//definicje portw sterujcych liniami kamery 
#define KAM_IOn	6

typedef enum 
{							//kierunek sygnaw na portach mikrokontrolera
	KAM_MCLK			=0,	//OUT 	gwny zegar 24MHz
	KAM_HSYNC			=1,	//IN  	H sygnalizacja transmisji pikseli obrazu linii
	KAM_STDB			=2,	//OUT	H przeczanie kamery w tryb czuwania	 
	KAM_VSYNC			=3,	//IN	H sygnalizacja transmisji ramki (linii obrazu)
	KAM_PCLK			=4,	//IN	Narastajce zbocze taktujce dane wyjciowe z kamery
	KAM_RSTB			=5	//OUT	L sygna zerowania kamery
} Kam_IO_TypeDef;
//gwny zegar 24MHz
#define KAM_MCLK_PORT     		GPIOA
#define KAM_MCLK_CLK 			RCC_AHB1Periph_GPIOA
#define KAM_MCLK_PIN			GPIO_Pin_8
//sygnalizacja transmisji pikseli obrazu linii 	  		DCMI_HSYNC
#define KAM_HSYNC_PORT    		GPIOA
#define KAM_HSYNC_CLK 			RCC_AHB1Periph_GPIOA
#define KAM_HSYNC_PIN			GPIO_Pin_4
#define KAM_HSYNC_PIN_SOURCE	GPIO_PinSource4
//H przeczanie kamery w tryb czuwania
#define KAM_STDB_PORT    		GPIOA
#define KAM_STDB_CLK 			RCC_AHB1Periph_GPIOA
#define KAM_STDB_PIN			GPIO_Pin_11
//sygnalizacja transmisji ramki (linii obrazu)			DCMI_VSYNC
#define KAM_VSYNC_PORT    		GPIOB		   
#define KAM_VSYNC_CLK 			RCC_AHB1Periph_GPIOB
#define KAM_VSYNC_PIN			GPIO_Pin_7
#define KAM_VSYNC_PIN_SOURCE	GPIO_PinSource7
//narastajce zbocze taktujce dane wyjciowe z kamery	DCMI_PIXCLK
#define KAM_PCLK_PORT    		GPIOA
#define KAM_PCLK_CLK 			RCC_AHB1Periph_GPIOA
#define KAM_PCLK_PIN			GPIO_Pin_6
#define KAM_PCLK_PIN_SOURCE		GPIO_PinSource6
//L sygna zerowania kamery
#define KAM_RSTB_PORT    		GPIOB
#define KAM_RSTB_CLK 			RCC_AHB1Periph_GPIOB
#define KAM_RSTB_PIN			GPIO_Pin_8


void Kamera_Inicjacja(int image_width, int image_height, char bajty_na_piksel);
void KAM_Magistrala_danych_inicjacja(Kam_DAT_TypeDef Linia, GPIOMode_TypeDef tryb_pracy, GPIOOType_TypeDef typ_wyjscia, 
									 GPIOPuPd_TypeDef typ_podciagania);
void KAM_IO_Linie_Inicjacja(Kam_IO_TypeDef Linia, GPIOMode_TypeDef tryb_prac, GPIOOType_TypeDef typ_wyjscia, 
							GPIOPuPd_TypeDef typ_podciagania);
void KAM_IO_Linie_High(Kam_IO_TypeDef Linia);
void KAM_IO_Linie_Low(Kam_IO_TypeDef Linia);
uint32_t KAM_IO_Linie_Odczyt(Kam_IO_TypeDef Linia);
char Kamera_Zmiana_Formatu_Obrazu(char num_formatu_obrazu);
unsigned char Kamera_Jaskrawosc_regulacja(unsigned char param_jaskrawosc);
unsigned char Kamera_Kontrast_regulacja(unsigned char param_kontrast);
unsigned char Kamera_Nasycenie_regulacja(unsigned char param_nasycenie);

#endif

