/**
  ******************************************************************************
  * plik    MT9D111.h
  * funkcja	plik nagwkowy procedur obsugi moduu kamery MT9D111
  * autor 	ARIES Ryszard Szymaniak www.arskam.com
  * wersja 	1.01
  * data    26-10-2012
  * opis   	ARS3-MT9D111 sterownik ARS3 obsugujcy modu kamery MT9D111 firmy APTINA
  * 		STM32F217VGT
  ******************************************************************************
  */  
/* Define to prevent recursive inclusion ------------------------------------ */
#ifndef __MT9D111_H
#define __MT9D111_H

#include "stm32f2xx_conf.h"

#define CHIP_ID 0x1519 
/******************************************************************************************************************/
/*                                                                                                                */
/*                                              direct addressable registers                                      */
/*                                                                                                                */
/******************************************************************************************************************/
//        NAME                          page,address
#define CHIP_VERSION                       0,0x00
#define PCLK1_PCLK2_CONFIGURATION          2,0x0E  // set to 0xF0F for max divider and 0 slew rate
#define PCLK3_CONFIGURATION                2,0x0F  // set to 0xF for max divider and 0 slew rate
#define CONTEXT_CONTROL                    0,0xF2

#define ROW_SPEED                          0,0x0A

#define READ_MODE_B						   0,0x20

/******************************************************************************************************************/
/*                                                                                                                */
/*                                              Firmware / MCU registers                                          */
/*                                                                                                                */
/******************************************************************************************************************/

/*

R0xC6:1[15:0], Indirect Access Address Register
Bit Description
7:0 Bits            7:0   of address for physical access; driver variable offset for logical access.
12:8 Bits           12:8  of address for physical access; driver ID for logical access.
14:13 Bits          14:13 of address for physical access; R0xC6:1[14:13] = 01 select logical access.
15 1 = 8-bit access; 0 = 16-bit access
*/
#define MON_VER                     0xA00C // mon.ver (ID = 0, Offset = 12, 8-bit variable) // firmware version #

#define BRAK_MASKI_ZAPISU_DO_MEM	0xFFFF

/*
The current context mode can also be read through the serial interface from the variable
mode.context (DriverID = 7, Offset = 0x02). If mode.context is read back as logic "1",
the capture mode is active. When the bit is cleared, the current mode is preview.
*/
#define MODE_CONTEXT            0xA702
#define KONTEKST_A	0
#define KONTEKST_B	1

/*
To switch from preview to the capture state, set the following variables:
&#65533; seq.captureParams.mode[1] = 1// ID = 1, Offset = 0x20
&#65533; seq.cmd = 2// ID = 1, Offset = 0x03
To switch from capture back into preview mode, use the following settings:
&#65533; seq.captureParams.mode[1] = 0// ID = 1, Offset = 0x20
&#65533; seq.cmd = 1// ID = 1, Offset = 0x03
*/
#define VMT                         0x2000 // state
#define SEQ_CAPTUREPARAMS_MODE      0xA120 // seq.captureParams.mode[1] = 1// ID = 1, Offset = 0x20
#define SEQ_CAPTUREPARAMS_NUMFRAMES 0xA121 // seq.captureParams.numFrames  (ID = 1, Offset = 0x21)
#define SEQ_CMD                     0xA103 // seq.cmd = 1// ID = 1, Offset = 0x03
#define SEQ_STATE                   0xA104 // state

/*
To change the output size of a context, the associated context image height and width
values can be updated before switching to the targeted context. If the size is changed for
the current (active) context, then a refresh command (seq.cmd = 5) needs to be
executed additionally.
For context A, use the variable mode.Output Width A (ID = 7, offset = 3) and
mode.Output Height A (ID = 7, offset = 5).
For context B, use the variable mode.Output Width B (ID = 7, offset = 7) and
mode.Output Height B (ID = 7, offset = 9). These settings only change the output image
size and do not have any effect on the frame rate or field of view.

*/
#define SZEROKOSC_OBRAZU_MAX	1600
#define WYSOKOSC_OBRAZU_MAX		1200
#define SZEROKOSC_OBRAZU_MIN	0
#define WYSOKOSC_OBRAZU_MIN		0


#define MODE_OUTPUT_HEIGHT_A    0x2705
#define MODE_OUTPUT_WIDTH_A     0x2703
#define MODE_OUTPUT_HEIGHT_B    0x2709
#define MODE_OUTPUT_WIDTH_B     0x2707
/*
mode.crop_X0_B// ID = 7, Offset = 0x35
mode.crop_X1_B// ID = 7, Offset = 0x37
mode.crop_Y0_B// ID = 7, Offset = 0x39
mode.crop_Y1_B// ID = 7, Offset = 0x3B
*/
#define MODE_CROP_X0_B          0x2735
#define MODE_CROP_Y0_B          0x2739
#define MODE_CROP_X1_B          0x2737
#define MODE_CROP_Y1_B          0x273B

#define MODE_CROP_X0_A          0x2727
#define MODE_CROP_Y0_A          0x272B
#define MODE_CROP_X1_A          0x2729
#define MODE_CROP_Y1_A          0x272D

/*
To select between YUV and RGB, bit 5 of variables mode.output_format_A (ID = 7,
Offset = 0x7D) and mode.output_format_B (ID = 7, Offset = 0x7E) should be used,
depending on which context mode is of interest.
*/
#define MODE_OUTPUT_FORMAT_A    0xA77D
#define MODE_OUTPUT_FORMAT_B    0xA77E

//stae rejestrw regulacji jakoci obrazu
//JASKRAWO
#define AE_TARGET_JASKRAWOSC		0xA206	  //ae.Target
#define GAM_CONT_B_KONTRAST			0xA744	  //gam_cont_B
#define AWB_SATURATION_NASYCENIE	0xA352	  //awb.saturation

/* JPEG 
0 vmt void* 58451 RW Reserved.
2 width uint 1600 R Image width (see mode.output_width).
4 height uint 1200 R Image height (see mode.output_height).
6 format uchar 0 RW Image format:
0&#65533;YCbCr 4:2:2
1&#65533;YCbCr 4:2:0
2&#65533;Monochrome
7 config uchar 52 RW Configuration and handshaking:
Bit 0&#65533;if 1, video; if 0, still snapshot
Bit 1&#65533;enable handshaking with host at every error frame
Bit 2&#65533;enable retry after an unsuccessful encode or transfer
Bit 3&#65533;host indicates it is ready for next frame
Bit 4&#65533;enable scaled quantization table generation
Bit 5&#65533;enable auto-select quantization table
Bit 7:6&#65533;quantization table ID
8 restartInt uint 0 RW Restart marker interval:
0 = restart marker is disabled
10 qscale1 uchar 6 RWBit 6:0&#65533;scaling factor for first set of quantization tables
Bit 7&#65533;1, new scaling factor value
11 qscale2 uchar 9 RWBit 6:0&#65533;scaling factor for second set of quantization tables
Bit 7&#65533;if 1, new scaling factor value
12 qscale3 uchar 12 RW Bit 6:0&#65533;scaling factor for third set of quantization tables
Bit 7&#65533;if 1, new scaling factor value
13 timeoutFrames uchar 10 RW Number of frames to time out when host is not responding
(setting bit 3 of config) to an unsuccessful JPEG frame while bit
1 of config is set.
14 state uchar 0 R JPEG driver state.
15 dataLengthMSB uchar R Bit [23:16] of previous frame JPEG data length.
16 dataLengthLSBs uint R Bit [15:0] of previous frame JPEG data length.
*/

#define JPEG_STATE                             0xA90E
#define JPEG_DATALENGTH_MSB                    0xA90F
#define JPEG_DATALENGTH_LSB                    0x2910
#define JPEG_IMG_FORMAT						   0xA906

#define JPEG_CONFIG							   0xA907
#define ENABLE_RETRY_AFTER_UNSUCCESS_ENCODE	   0x04
#define ENABLE_AUTO_SELECT_QUANTIZATION TAB	   0x20

#define JPEG_RESTART_MARKER					   0xA908
#define RESTART_MARKER_JPG_INSERT_DISABLE		0
#define JPEG_QSCALE_1						   0xA90A
#define JPEG_QSCALE_2						   0xA90B
#define JPEG_QSCALE_3						   0xA90C
#define JPG_QSCALE_WSPOLCZYNNIK_6				6
#define JPG_QSCALE_WSPOLCZYNNIK_36				36

// JPEG related
#define JPEG_CONTROL_REGISTER              2,0x00
#define JPEG_STATUS_REGISTER_0             2,0x02
#define JPEG_STATUS_REGISTER_1             2,0x03
#define JPEG_STATUS_REGISTER_2             2,0x04
#define JPEG_FRONTEND_CONFIG_REGISTER      2,0x05
#define JPEG_CORE_CONFIG_REGISTER          2,0x06
#define JPEG_ENCODER_BYPASS                2,0x0A

#define OUTPUT_CONFIGURATION               2,0x0D  // set to 0x58 for JPEG SOI+EOI within frame valid + variable pixel clock rate + jpeg status insertion at end of frame
#define PCLK1_PCLK2_CONFIGURATION          2,0x0E  // set to 0xF0F for max divider and 0 slew rate
#define PCLK3_CONFIGURATION                2,0x0F  // set to 0xF for max divider and 0 slew rate

//JPEG enable: mode.mode_config[5] = 0 // ID = 7, offset = 0x0B
#define MODE_MODE_CONFIG        0x270B



unsigned char pageChange(int page);
unsigned short readMCU(unsigned short address);
unsigned char readMCUMemory(unsigned short address, char *value);
unsigned char writeMCUMemory(unsigned short address, unsigned short value, unsigned short mask);
unsigned char readRegister(int page, unsigned char address, char *value);
unsigned short usReadRegister(int page,unsigned char address);
unsigned char writeRegister(int page, unsigned char address, unsigned short value/*, unsigned short mask*/);
unsigned char writeRegisterMask(int page, unsigned char address, unsigned short value, unsigned short mask);
void mt9d111_init(void);
char mt9d111_Zmiana_Standardowych_Rozmiarow_Obrazu_Kontekst_B(char format_obrazu);
unsigned char mt9d111_zmiana_rozdzielczosci_obrazu_kontekstu(char kontekst, int szerokosc, int wysokosc);
unsigned char mt9d111_zmian_formatu_danych_obrazu_kontekstu(char kontekst);
unsigned char mt9d111_zapis_do_buf_ramki_RGB(void);
unsigned char MT9D111_Jaskrawosc_regulacja(unsigned char param_jaskrawosc);
unsigned char MT9D111_Kontrast_regulacja(unsigned char param_kontrast);
unsigned char MT9D111_Nasycenie_regulacja(unsigned char param_nasycenie);
unsigned char mt9d111_ustawienie_luster_obrazu(char flagi_luster);
unsigned char mt9d111_kontrola_ustawienia_AE_standard(char flaga_AE);
unsigned char mt9d111_odczyt_wspolczynnika_kompresji(char flaga_wpolczynnika_kompresji);

#endif
